﻿using System;
using System.Data;
using System.Data.Common;
using System.Data.EntityClient;
using System.Data.Objects;
using System.Data.Spatial;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Linq;
using de.WWWings.GO;

namespace Konsole45
{
 public class EF45Demo
 {

  public static void Run()
  {
   WWWings6Entities modell = new WWWings6Entities();

   var q = modell.Person.OfType<Passagier>().Take(10);


   foreach (Passagier p in q.ToList())
   {

    Console.WriteLine(p.Name);

   }


   modell.Dispose();

  }





  // ######################################################################################################
  public static void EF5_EinfacheDemo()
  {

   WWWings6Entities modell = new WWWings6Entities();

   var q = from f in modell.Flug where f.Abflugort == "Rom" && f.FreiePlaetze > 10 select f;

   foreach (var f in q.ToList())
   {
    Console.WriteLine("Flug Nr {0} fliegt von {1} nach {2} und hat {3} freie Plätze", f.FlugNr, f.Abflugort, f.Zielort, f.FreiePlaetze);
   }

   modell.Dispose();
  }



  public static void Demo_TVF_ADONET()
  {

   WWWings6Entities modell = new WWWings6Entities();
   string connectionString = (modell.Connection as EntityConnection).StoreConnection.ConnectionString;


   SqlConnection conn = new SqlConnection(connectionString);
   conn.Open();

   String q = "select * from GetFluegeVon(@Abflugort) where FreiePlaetze > 10 and Zielort = 'Moskau' order by FreiePlaetze desc";

   SqlCommand command = conn.CreateCommand();

   command.CommandText = q;

   SqlParameter p = new SqlParameter("@Abflugort", SqlDbType.NVarChar);
   p.Value = "Rom";
   command.Parameters.Add(p);

   var dr = command.ExecuteReader();

   while (dr.Read())
   {
    Console.WriteLine(dr["FlugNr"].ToString());
   }

   dr.Close();
   conn.Close();
  }

  // ######################################################################################################
  public static void Demo_TVF()
  {

   WWWings6Entities modell = new WWWings6Entities();

   Console.WriteLine("Anzahl Flüge: " + modell.Flug.Count());
  
    modell = new WWWings6Entities();
    // Verwendung eines c View ab EF 1.0:
    var fluege2 = (from f in modell.GetFluegeVonTVF("Rom") where f.FreiePlaetze > 10 && f.Zielort == "Moskau" orderby f.Datum select f).Take(20).ToList(); ;
   
    Console.WriteLine("Test#3 TVF: " + fluege2.Count);
    //Console.WriteLine(((ObjectQuery)fluege2).ToTraceString());

    foreach (var f in fluege2)
    {
     Console.WriteLine("Flug Nr {0} fliegt von {1} nach {2} und hat {3} freie Plätze", f.FlugNr, f.Abflugort, f.Zielort, f.FreiePlaetze);
    }

  

  
  }


  // ######################################################################################################
  public static void Demo_TVF_SpeedTest()
  {
   int Durchlaeufe = 5;
   WWWings6Entities modell = new WWWings6Entities();

   Console.WriteLine("Anzahl Flüge: " + modell.Flug.Count());
   Stopwatch sw0 = new Stopwatch();
   Stopwatch sw1 = new Stopwatch();
   Stopwatch sw2 = new Stopwatch();
   Stopwatch sw3 = new Stopwatch();
   // ----------------------------------------
   for (int i = 0; i < Durchlaeufe; i++)
   {

    sw0.Start();
    modell = new WWWings6Entities();
    // LINQ-to-Entities ab EF 1.0: 
    var fluege0 = (from f in modell.Flug where f.Abflugort == "Rom" && f.FreiePlaetze > 10 && f.Zielort == "Moskau" select f).Take(20).ToList();
    sw0.Stop();
    Console.WriteLine("Test#1 LINQ-to-Entities: " + fluege0.Count);
    foreach (var f in fluege0)
    {
     //  Console.WriteLine("Flug Nr {0} fliegt von {1} nach {2} und hat {3} freie Plätze", f.FlugNr, f.Abflugort, f.Zielort, f.FreiePlaetze);
    }
   }

   // ----------------------------------------
   for (int i = 0; i < Durchlaeufe; i++)
   {
    sw1.Start();
    modell = new WWWings6Entities();
    // SP-Verwendung in LINQ-to-Entities ab EF 1.0: 
    var fluege1 = (from f in modell.GetFluegeVonSP("Rom") where f.FreiePlaetze > 10 && f.Zielort == "Moskau" select f).Take(20).ToList();
    sw1.Stop();
    Console.WriteLine("Test#2 SP: " + fluege1.Count);
    foreach (var f in fluege1)
    {
     // Console.WriteLine("Flug Nr {0} fliegt von {1} nach {2} und hat {3} freie Plätze", f.FlugNr, f.Abflugort, f.Zielort, f.FreiePlaetze);
    }
   }

   // ----------------------------------------
   for (int i = 0; i < Durchlaeufe; i++)
   {
    sw2.Start();
    modell = new WWWings6Entities();
    // Verwendung eines c View ab EF 1.0:
    var fluege2 = (from f in modell.GetFluegeVonTVF("Rom") where f.FreiePlaetze > 10 && f.Zielort == "Moskau" select f).Take(20).ToList(); ;
    sw2.Stop();

    Console.WriteLine("Test#3 TVF: " + fluege2.Count);
    //Console.WriteLine(((ObjectQuery)fluege2).ToTraceString());

    foreach (var f in fluege2)
    {
     // Console.WriteLine("Flug Nr {0} fliegt von {1} nach {2} und hat {3} freie Plätze", f.FlugNr, f.Abflugort, f.Zielort, f.FreiePlaetze);
    }

   }

   // ----------------------------------------
   for (int i = 0; i < Durchlaeufe; i++)
   {
    sw3.Start();
    modell = new WWWings6Entities();
    // TVF-Verwendung in LINQ-to-Entities ab EF 5.0:
    var fluege3 = (from f in modell.FluegeVonRom where f.FreiePlaetze > 10 && f.Zielort == "Moskau" select f).Take(20).ToList(); ;
    sw3.Stop();
    Console.WriteLine("Test#2 View: " + fluege3.Count);
    //Console.WriteLine(((ObjectQuery)fluege2).ToTraceString());

    foreach (var f in fluege3)
    {
     // Console.WriteLine("Flug Nr {0} fliegt von {1} nach {2} und hat {3} freie Plätze", f.FlugNr, f.Abflugort, f.Zielort, f.FreiePlaetze);
    }

   }
   Console.WriteLine("Dauer LINQ-to-Entities: " + sw0.ElapsedMilliseconds / Durchlaeufe);
   Console.WriteLine("Dauer SP: " + sw1.ElapsedMilliseconds / Durchlaeufe);
   Console.WriteLine("Dauer TVF: " + sw2.ElapsedMilliseconds / Durchlaeufe);
   Console.WriteLine("Dauer View: " + sw3.ElapsedMilliseconds / Durchlaeufe);
  }


  // ######################################################################################################
  //public static void Demo_SPMuliResult()
  //{
  //    WWWings6Entities modell = new WWWings6Entities();
  //    var fluege = modell.GetFluegeMitAnzahl();

  //    foreach (var f in fluege)
  //    {
  //        Console.WriteLine(f.FlugNr);
  //    }

  //}

  // ######################################################################################################
  public static void Demo_Geo()
  {

   var Essen = DbGeography.FromText("POINT(51.455643 7.011555)");
   //http://maps.google.de/maps?q=Essen&hl=en&sll=51.455643,7.011555

   var Graz = DbGeography.FromText("POINT(47.0666 15.4333)");
   //http://maps.google.de/maps?q=Graz&hl=en&ie=UTF8&sll=47.077838,15.439568
   var Entfernung = Essen.Distance(Graz);

   Console.WriteLine("Entfernung zwischen Essen und Graz: " + Math.Round((decimal)Entfernung / 1000, 2) + " Meilen");

  }


  public static void Demo_GeoEnum()
  {
   WWWings6Entities modell = new WWWings6Entities();
   //modell.ExecuteStoreCommand("delete from Flughafen");

   // Objekt anlegen
   var f = modell.Flughafen.CreateObject();
   // Key zuweisen
   f.Name = "Essen/Mülheim";
   // Zuweisen eines Enumerationswertes
   f.Typ =  Flughafenart.Hubflughafen;
   // Setzen eines WGS84-Punktes (Flughafen Essen/Mülheim)
   // Bei Eingabe in Textbox: SRID=4326;POINT(51.4023 6.9373)
   f.Position = DbGeography.FromText("POINT(51.4023 6.9373)", 4326);
   // Anfügen
   modell.Flughafen.AddObject(f);
   // Speichern
   modell.SaveChanges();


   var q = modell.Flughafen;
   Console.WriteLine("Anzahl: " + q.Count());
   foreach (Flughafen fh in q.ToList())
   {
    Console.WriteLine(fh.Name + " (" + fh.Typ + ") :" + fh.Position.Latitude + " / " + fh.Position.Longitude);
   }



   //f.Name = "Test " + DateTime.Now.ToString();

   // Setzen eines WGS84-Punktes
   // Essen: Latitude: 51.4023 Longitude: 6.9373

   // Graz: 47.066667°, 15.433333°
   // f.Position = DbGeography.Parse("POINT(51.4023 6.9373)");

   // Haibach
   //  48.409722°, 13.916389°

   // Köln
   //  50.89004,7.13974

  }
 }



}

