﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;

namespace Konsole45
{
   public class AsyncDemo
    {

        public static void Run()
        {

            DownloadAsync(new Uri("http://www.IT-Visions.de"));
            Console.WriteLine("Warten auf Tastendruck");
            WarteAnzeige(200); // wird hier bewusst blockierend aufgerufen!"

       
        }


        /// <summary>
        /// Asynchroner Download (Rückgabe: nichts)
        /// </summary>
        static private async void DownloadAsync(Uri URL)
        {
            WriteLineColor("DownloadAsync() - Schritt 1: Download Starten...", ConsoleColor.Green);
            Task<byte[]> task = GetHttpAsync(new Uri("http://www.IT-Visions.de"));
            WriteLineColor("DownloadAsync() - Schritt 2: Download ist gestartet", ConsoleColor.Green);

            byte[] ergebnis = await task;

            // Alternative:
            // byte[] ergebnis = await GetHttpAsync(URL);

            WriteLineColor("DownloadAsync() - Schritt 3: Download fertig", ConsoleColor.Green);
            WriteLineColor("DownloadAsync() - Ergebnis: " + ergebnis.Length + " Bytes empfangen", ConsoleColor.Green);
        }

        /// <summary>
        /// Asynchroner Download (Rückgabe: Task<T></T>)
        /// </summary>
        static private async Task<byte[]> GetHttpAsync(Uri currentURI)
        {
            WriteLineColor("GetHttpAsync() -   Schritt 1. URL = " + currentURI, ConsoleColor.Yellow);
            HttpClient client = new HttpClient() { MaxResponseContentBufferSize = 1000000 };
            WriteLineColor("GetHttpAsync() -   Schritt 2. URL = " + currentURI, ConsoleColor.Yellow);
            HttpResponseMessage httpRM = await client.GetAsync(currentURI);
            WriteLineColor("GetHttpAsync() -   Schritt 3. URL = " + currentURI + " Status = " + httpRM.StatusCode, ConsoleColor.Yellow);
            var Content = await httpRM.Content.ReadAsByteArrayAsync();
            WriteLineColor("GetHttpAsync() -   Schritt 4. URL = " + currentURI + " Inhalt = " + Content.Length, ConsoleColor.Yellow);
            return Content;
        }

        /// <summary>
        /// Warteanzeige mit eigenem Task-Objekt
        /// </summary>
        static public async Task WarteAnzeige(int WartezeitMS = 100)
        {
            var t = new Task(() => WriteWarteanzeige(1000, WartezeitMS));
            t.Start();
            await t;

        }

        /// <summary>
        /// Synchrone Methode für Task in WarteAnzeige()
        /// </summary>
        static private void WriteWarteanzeige(int Anzahl, int WartezeitMS)
        {
            for (int i = 0; i < Anzahl; i++)
            {
                Console.Write(".");
                System.Threading.Thread.Sleep(WartezeitMS);
            }
        }

       public static void WriteLineColor(string s, ConsoleColor c)
       {

           var calt = Console.ForegroundColor;
           Console.ForegroundColor = c;
           Console.WriteLine(s);
           Console.ForegroundColor = calt;

       }

    }
}
