﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading.Tasks.Dataflow;
using System.IO;
using System.Threading;
using System.Diagnostics;
namespace TDF
{
 class Dataflowdemo
 {
  public class Kunde
  {
   public string Name { get; set; }
   public double Umsatz { get; set; }
  }
  public static void Basisbeispiel1()
  {
   Console.WriteLine("TDF Basisbeispiel1");
   var options = new ExecutionDataflowBlockOptions() { MaxDegreeOfParallelism = 3, MaxMessagesPerTask = 100 };
   var Berechnungen = new ActionBlock<Kunde>(kunde =>
   {
    Console.WriteLine("Kunde {0}: {1}! (Thread: {2})", kunde.Name, kunde.Umsatz, System.Threading.Thread.CurrentThread.ManagedThreadId);
    System.Threading.Thread.Sleep(500);
   }, options);

   for (int i = 1; i <= 10; i++)
   {
    var k = new Kunde() { Name = "Kunde " + i, Umsatz = i * 1000 };
    var gesendet = Berechnungen.Post(k);

    Console.WriteLine("gesendet: " + gesendet);
   }
   Berechnungen.Complete();

   Berechnungen.Completion.ContinueWith(x => { Console.WriteLine("Alle Kunden bearbeitet!"); });


  }


  public static void Basisbeispiel2()
  {
   Console.WriteLine("TDF Basisbeispiel2");
   var options = new ExecutionDataflowBlockOptions() { MaxDegreeOfParallelism = 10, MaxMessagesPerTask = 100 };
   var Umsatzziele = new TransformBlock<Kunde, double>(async kunde => //async
   {
    Console.WriteLine("Kunde {0}: {1}! (Thread: {2})", kunde.Name, kunde.Umsatz, System.Threading.Thread.CurrentThread.ManagedThreadId);
    await System.Threading.Tasks.Task.Delay(500);
    // Thread.Sleep(500);
    return kunde.Umsatz * 2;
   }, options);

   for (int i = 1; i <= 10; i++)
   {
    var k = new Kunde() { Name = "Kunde " + i, Umsatz = i * 1000 };
    var gesendet = Umsatzziele.Post(k);
    Console.WriteLine("gesendet: " + gesendet);
   }

   Umsatzziele.Completion.ContinueWith(x =>
   {
    Console.WriteLine("Ende!");
   });

   Umsatzziele.Complete();




   while (Umsatzziele.OutputCount < 10) ;

   //Task<double> t = Umsatzziele.ReceiveAsync();
   //t.Wait();
   //Console.WriteLine(t.Result);


   Console.WriteLine("Zusammenfassung der Ziele:");
   Console.WriteLine("Anzahl der Ergebnisse: " + Umsatzziele.OutputCount);
   double Ziel;
   while (Umsatzziele.TryReceive(out Ziel))
   {
    Console.WriteLine("Umsatzziel: " + Ziel);
   }

   //var Ausgabe = new ActionBlock<double>(Ziel =>
   //{
   //    Console.WriteLine("Umsatzziel: " + Ziel);

   //}, options);


   //Umsatzziele.LinkTo(Ausgabe);


  }


  public static void BeispielBestellImport()
  {
   var options = new ExecutionDataflowBlockOptions() { MaxDegreeOfParallelism = 2 };

   Stopwatch sw = new Stopwatch();
   sw.Start();

   var csvFileReader = new TransformManyBlock<string, string>(fileName =>
   {
    Console.WriteLine("Importiere Datei: " + fileName + " Thread: " + System.Threading.Thread.CurrentThread.ManagedThreadId); 
    return File.ReadAllText(fileName).Split('\n');
   }, options);

   var broadcast = new BroadcastBlock<string>(str => str);

   var idExtractor = new TransformBlock<string, string>(line => {
    Console.WriteLine("Produkt: " + line.Split(';')[0] + " Thread: " + System.Threading.Thread.CurrentThread.ManagedThreadId);
    Thread.Sleep(1000);
    return line.Split(';')[0];
   }, options);

   var cardChecker = new TransformBlock<string, bool>(line =>
   {
    Console.WriteLine("Kreditkarte: " + line.Split(';')[1] + " Thread: " + System.Threading.Thread.CurrentThread.ManagedThreadId); 
    Thread.Sleep(1500);
    return line.Split(';')[1] != "0000-0000-0000-0000";
   }, options);

   var companyChecker = new TransformBlock<string, bool>(line =>
   {

    Console.WriteLine("Verkäufer: " + line.Split(';')[2].Trim() + " Thread: " + System.Threading.Thread.CurrentThread.ManagedThreadId); ;
    Thread.Sleep(3000);
    return line.Split(';')[2].Trim() == "Kauf GmbH";
   }, options);



   var join = new JoinBlock<string, bool, bool>();

   var sendDecision = new ActionBlock<Tuple<string, bool, bool>>(tuple =>
   {
    string id = tuple.Item1;
    bool ok1 = tuple.Item2;
    bool ok2 = tuple.Item3;

    if (ok1 && ok2)
    {
     Console.ForegroundColor = ConsoleColor.Green;
     Console.WriteLine(DateTime.Now.ToString() + ": " + id + " ok");
     Console.ForegroundColor = ConsoleColor.Gray;
    }
    else
    {
     Console.ForegroundColor = ConsoleColor.Red;
     Console.WriteLine(DateTime.Now.ToString() + ": " + id + " not ok [Kreditkarte:" + ok1 + "/Verkäufer: " + ok2 + "]");
     Console.ForegroundColor = ConsoleColor.Gray;
    }

    Console.WriteLine("Dauer: " + sw.ElapsedMilliseconds);
   });

   // Verbinden der DataflowBlocks
   csvFileReader.LinkTo(broadcast);

   broadcast.LinkTo(companyChecker);
   broadcast.LinkTo(cardChecker);
   broadcast.LinkTo(idExtractor);

   idExtractor.LinkTo(join.Target1);
   cardChecker.LinkTo(join.Target2);
   companyChecker.LinkTo(join.Target3);

   join.LinkTo(sendDecision);

   Console.WriteLine("Einrichtung des Dataflows abgeschlossen");

   Console.WriteLine("Importer Startet...");
   // Starten des Data Flows
   csvFileReader.Post(@"Parallel\order-1.txt");

   // Starten des Data Flows
   csvFileReader.Post(@"Parallel\order-2.txt");

   Console.WriteLine("--- Alle Imports gestartet!");



  }
 }
}
