﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using de.WWWings.GL;
using de.WWWings.GO;
using Microsoft.QualityTools.Testing.Fakes;

namespace NET45Tests
{
 [TestClass]
 public class Test
 {

  [TestMethod]
  public void Test_AddFlug_RealerKontext()
  {
   WWWings6Entities ctx = new WWWings6Entities();

   var f = new Flug();
   f.FlugNr = 912345;
   f.Abflugort = "Essen/Mülheim";
   f.Zielort = "Rom";
   f.Datum = DateTime.Now;
   f.Plaetze = 200;
   f.FreiePlaetze = 199;

   ctx.Flug.AddObject(f);

   ctx.SaveChanges();

  }

  [TestMethod]
  public void Test_AddFlug_ShimKontext()
  {
   using (ShimsContext.Create())
   {
    WWWings6Entities ctx = new WWWings6Entities();
    var f = new Flug();
    f.FlugNr = 912345;
    f.Abflugort = "Essen/Mülheim";
    f.Zielort = "Rom";
    f.Datum = DateTime.Now;
    f.Plaetze = 200;
    f.FreiePlaetze = 199;

    // SaveChanges() mit Attrappe neu belegen
    System.Data.Objects.Fakes.ShimObjectContext.AllInstances.SaveChanges =
     (x) => { return 1; };

    // Dieser Aufruf wird nicht abgefangen, da keine Attrappe vorhanden
    Console.WriteLine("Datenbank vorhanden? " + ctx.DatabaseExists());
    // Dieser Aufruf wird nicht abgefangen, da keine Attrappe vorhanden
    ctx.Flug.AddObject(f);
    // Dieser Aufruf wird abgefangen und durch Attrappe ersetzte
    ctx.SaveChanges();
   }
  }

  [TestMethod]
  public void Test_GetFlug_RealesManagerObjekt()
  {
   var Ort = "Rom";

   var bl = new FlugBLManager();
   var liste = bl.GetFluegeVonOrt(Ort);
   Assert.IsTrue(liste.Count > 0);
   foreach (var f in liste)
   {
    Assert.AreEqual(f.Abflugort, Ort);
   }
  }


  [TestMethod]
  public void Test_Buchung_RealesManagerObjekt()
  {
   int FlugNr = 100;
   int PassagierNr = 1;
   var fbl = new FlugBLManager();
   var bm = new BuchungsManager(fbl);

   var buchung = bm.BuchungenErstellen(FlugNr, PassagierNr);
   Assert.IsTrue(buchung);
  }


  [TestMethod]
  public void Test_Buchung_StubObjekt()
  {
   int FlugNr = 100;
   int PassagierNr = 1;

   // Stub erzeugen
   var stub = new de.WWWings.GL.Fakes.StubIFlugBLManager();
   // Reaktion auf Aufruf von GetFlug() definieren
   stub.GetFlugInt64 = (flugnr) => new Flug() { FlugNr = (int)flugnr };

   // Buchungsmanager mit dem Stub verbinden
   var bm = new BuchungsManager(stub);

   var buchung = bm.BuchungenErstellen(FlugNr, PassagierNr);
   Assert.IsTrue(buchung);
  }


 }
}
