﻿//----------------------------------------------------------------
// Dieses Beispiel basiert auf dem Beispiel "Advanced Filters" aus den WCF 4.0-Beispielen von Microsoft
//----------------------------------------------------------------
using System;
using System.ServiceModel.Channels;
using System.ServiceModel.Dispatcher;

namespace WCF_Router
{
 /// <summary>
 /// World Wide Wings 
 /// Eigener Filter ohne eigene Filtertabelle für WCF 4.0
 /// Steuerung über SOAP-Header oder Zufall
 /// </summary>
 public class CustomFilterOhneFilterTable : MessageFilter
 {
  public string FilterID;
  private static bool Selected = false;
  private static int? Zufall = null;

  public CustomFilterOhneFilterTable(string filterData)
  {
   if (string.IsNullOrEmpty(filterData)) { throw new ArgumentNullException("groupName"); }

   this.FilterID = filterData;
  }

  /// <summary>
  /// Diese Routine wird für jeden aktivierten Filter aufgerufen.
  /// Der sich zuständig fühlende Filter muss mit "true" antworten.
  /// Es darf aber nur einer "true" sagen, sonst Fehler! :-)
  /// </summary>
  public override bool Match(Message message)
  {

   // Vorgabe des Clients durch einen Header? Dann beachte das
   foreach (var h in message.Headers)
   {
    // Ausnahme: Steuerung durch SOAP-Header
    if (h.Name.StartsWith("Ziel"))
    {
     int ZielID = message.Headers.GetHeader<int>(h.Name, h.Namespace);
     Console.WriteLine("Steuerung der Weiterleitung durch Header: " + h.Name + ": " + ZielID);

     // Wenn diese Filter die gleiche Nummer hat wie die übergebene ID
     if (this.FilterID == "CustomFilter" + ZielID) return true;
     else return false;

    }
   }

   // --- sonst Zufallsauswahl (ziemlich umständlich)

   // Wenn es Filter 1 ist: setzte statische Variablen zurück!
   if (this.FilterID == "CustomFilter1") { Selected = false; Zufall = null; }
   // Wenn schon ein Filter gewählt ist: Dann dieser nicht mehr
   if (Selected) return false;
   // Zufallszahl erzeugen, wenn noch nicht vorhanden
   if (Zufall == null) Zufall = new System.Random(System.DateTime.Now.Millisecond).Next(300);

   // Ist gemäß Zufallszahl dieser Filter der gewählte?
   int mod = (Zufall.Value % 3)+1;
   if (this.FilterID == "CustomFilter" + mod && !Selected) 
   {     
    Console.WriteLine("Weiterleiten an CustomFilter #" + mod);
    Selected = true;
    return true;
   }

   return false;
  }

  public override bool Match(MessageBuffer buffer)
  {
   return Match(buffer.CreateMessage());
  }

  protected override IMessageFilterTable<TFilterData> CreateFilterTable<TFilterData>()
  {
   return null;
  }
 }
}
