using System;
using System.Collections.Generic;
using System.Text;
using System.Messaging;
using WWWings_WCFServer;
using System.ServiceProcess;
using System.ServiceModel;

namespace de.WWWings.WCFServer
{
 class Program
 {
  static void Main(string[] args)
  {
 
   Print("World Wide Wings WCF Anwendungsserver Konsole Test Mode", ConsoleColor.Cyan);
   Print("--------------------------------------------------------", ConsoleColor.Cyan);
   try
   {
    // Vorbereitung fr alle Flle (Console + Dienst)
    string queueName = @".\private$\WWWings";
    MessageQueue messageQueue;
    ////  Erzeuege MSMQ queue, falls notwendig
    if (MessageQueue.Exists(queueName))
     messageQueue = new MessageQueue(queueName);
    else
     messageQueue = MessageQueue.Create(queueName, true);
   }
   catch (Exception ex)
   {
    Print("Fehler bei Initialisierung von MSMQ: " + ex.Message, ConsoleColor.Red);


   }

   // Start als Konsolenanwendung
   if (args.Length > 0 && args[0].ToLower().StartsWith("debug"))
   {

    // nderung der Portnummer angewiesen?
    // Hintergrund: Zum Testen mit mehreren Instanzen des Servers braucht man verschiedene Portnummer!

    int BasePort = 0;
    if (args.Length >= 2) Int32.TryParse(args[1], out BasePort);
    Util.UI.SetConsolePos(700, BasePort);
    Console.Title = "WCF Server - DEBUG - " + BasePort + " - ProcessID=" + System.Diagnostics.Process.GetCurrentProcess().Id; 
     if (args.Length > 2) Console.Title = Console.Title + " - " + args[2];

    Print("WCF-Server wird gestartet...", ConsoleColor.Cyan);
    Print("Identitt des Server-Prozesses: " + System.Environment.UserDomainName + @"\" + System.Environment.UserName, ConsoleColor.White);

    //Print(String.Format("Es warten {0} MSMQ-Nachrichten.", messageQueue.GetAllMessages().Length), ConsoleColor.White);
    //Message lastMessage = messageQueue.Receive(new TimeSpan(0, 0, 4));
    //Console.WriteLine(lastMessage.Body.ToString());
    //Console.ReadLine();
    de.WWWings.WCFServer.WWWingsWCFServiceHost.StartService();

    Print("WCF-Server ist jetzt gestartet!", ConsoleColor.Cyan);

    SelbstTests();


    Console.WriteLine("\nServer luft, bis Taste gedrckt wird!");
    Console.ReadLine();
    Print("WCF-Server wird beendet...", ConsoleColor.Cyan);
    de.WWWings.WCFServer.WWWingsWCFServiceHost.StopService();
    Print("WCF-Server ist jetzt beendet!", ConsoleColor.Cyan);

    //Dim s As New RemotingServerPingService
    //s.Timer1.Enabled = True
    //s.Timer1.Interval = 1000
    //s.Timer1.Start()
    Console.ReadLine();
   }
   // Start als Systemdienst
   else
   {
    Console.WriteLine("Hinweis: Start im Modus 'Windows-Dienst' nicht mglich an der Konsole. Nutzen Sie zum Starten an der Konsole den Parameter 'debug', also: WWWings_WCFServer.exe debug!");
    ServiceBase[] services = { new WorldWideWingsWCFApplicationServer() };
    ServiceBase.Run(services);
   }


  }

  private static void SelbstTests(int BasePort = 0)
  {

   Print("Datenbank: " + System.Configuration.ConfigurationManager.ConnectionStrings["CS_WWWings"].ConnectionString, ConsoleColor.Green);
   // Interner Selbsttest (ohne Protokollstack)
   try
   {
    de.WWWings.Dienste.PingInfo p = new de.WWWings.Dienste.TestService().Ping();
    Print("Interner Selbsttest (ohne Protokollstack): OK (" + p.DBInfo.AnzFluege + "/" + p.DBInfo.AnzPassagiere + "/" + p.DBInfo.AnzBuchungen + ")", ConsoleColor.Green);
   }
   catch (Exception ex)
   {
    Print("Interner Selbsttest (ohne Protokollstack): Fehler (" + HoleFehlerText(ex) + "/" + ex.StackTrace + ")", ConsoleColor.Red);
   }

   //  Externer Selbsttest (mit Protokollstack)
   try
   {
    WWWings_WCFServer.Selbsttest.TestServiceClient tcp = new WWWings_WCFServer.Selbsttest.TestServiceClient("TestService_TCP");

    if (BasePort > 0)
    {
     var uribuilder = new UriBuilder(tcp.Endpoint.Address.Uri);
     if (tcp.Endpoint.Address.Uri.Port > 0) uribuilder.Port = tcp.Endpoint.Address.Uri.Port + BasePort;
     tcp.Endpoint.Address = new EndpointAddress(uribuilder.Uri);
    }

    WWWings_WCFServer.Selbsttest.PingInfo p = tcp.Ping();
    Print("Externer Selbsttest (mit TCP-Protokollstack): OK (" + p.DBInfo.AnzFluege + "/" + p.DBInfo.AnzPassagiere + "/" + p.DBInfo.AnzBuchungen + ")", ConsoleColor.Green);
    tcp.Close();
   }
   catch (Exception ex)
   {
    Print("Externer Selbsttest (mit TCP-Protokollstack): Fehler (" + HoleFehlerText(ex) + "/" + ex.StackTrace + ")", ConsoleColor.Red);
   }

   if (BasePort == 0) // Pipe gibt es nur wemm BasePort = 0!
   { 
   //  Externer Selbsttest (mit Protokollstack)
   try
   {
    WWWings_WCFServer.Selbsttest.TestServiceClient np = new WWWings_WCFServer.Selbsttest.TestServiceClient("TestService_PIPE");
    WWWings_WCFServer.Selbsttest.PingInfo p = np.Ping();
    Print("Externer Selbsttest (mit Named Pipes): OK (" + p.DBInfo.AnzFluege + "/" + p.DBInfo.AnzPassagiere + "/" + p.DBInfo.AnzBuchungen + ")", ConsoleColor.Green);
    np.Ping();
   }
   catch (Exception ex)
   {
    Print("Externer Selbsttest (mit Named Pipes): Fehler (" + HoleFehlerText(ex) + "/" + ex.StackTrace + ")", ConsoleColor.Red);
   }
   }
  }

  /// <summary>
  /// Hilfsmethode fr Protokollierung
  /// </summary>
  private static void Print(string Text, System.ConsoleColor Farbe)
  {
   Console.ForegroundColor = Farbe;
   Console.WriteLine(Text);
   Console.ForegroundColor = ConsoleColor.Gray;
  }

  public static string HoleFehlerText(Exception ex)
  {
   string ausgabe = ex.GetType().ToString() + ": " + ex.Message + "\n";
   if (ex.InnerException != null) ausgabe += HoleFehlerText(ex.InnerException);
   return ausgabe;
  }
 }
}
