﻿using System;
using System.Collections.Generic;
using System.Data.Services;
using System.Linq;
using System.Linq.Expressions;
using System.ServiceModel.Web;
using System.Web;
using de.WWWings.GO_STE;

namespace WWWings_EF4_AppServer
{
 public class WWWingsWebDataService : DataService<WWWings6STE>
 {
  // This method is called only once to initialize service-wide policies.
  public static void InitializeService(DataServiceConfiguration config)
  {
   // Protokollversion setzen
   config.DataServiceBehavior.MaxProtocolVersion = System.Data.Services.Common.DataServiceProtocolVersion.V3;

   // Fehlermeldungen einschalten
   config.UseVerboseErrors = true;

   // Zugriffsrechte setzen
   //config.SetEntitySetAccessRule("*", EntitySetRights.All);
   config.SetEntitySetAccessRule("Flug", EntitySetRights.All);
   config.SetEntitySetAccessRule("Pilot", EntitySetRights.AllRead | EntitySetRights.WriteAppend);
   config.SetEntitySetAccessRule("Mitarbeiter", EntitySetRights.AllRead | EntitySetRights.WriteAppend);
   config.SetEntitySetAccessRule("Person", EntitySetRights.AllRead | EntitySetRights.WriteAppend | EntitySetRights.WriteMerge);
   config.SetEntitySetAccessRule("Passagier", EntitySetRights.AllRead | EntitySetRights.WriteAppend | EntitySetRights.WriteMerge);
   config.SetServiceOperationAccessRule("*", ServiceOperationRights.All);

   // Beschränkungen der Datenmenge
   //config.MaxResultsPerCollection = 2;
   config.MaxExpandDepth =1;
   System.Diagnostics.Debug.WriteLine(config.MaxResultsPerCollection);
   System.Diagnostics.Debug.WriteLine(config.MaxExpandDepth);
   //config.MaxExpandCount
   //config.MaxChangesetCount
   
   // Bsp: Expand: http://localhost:87/WWWingsDataService.svc/Flug(101)?$expand=Passagier/Person,Pilot/Mitarbeiter/Person
  }

  // Interceptor für Einschränkung der Operationen
  // Normale Benutzer dürfen nur Flüg nutzen von Orten mit "R" in der Zukunft, die nicht ausgebucht sind
  [QueryInterceptor("Flug")]
  public Expression<Func<Flug, bool>> FilterFluege()
  {
   if (HttpContext.Current != null && HttpContext.Current.User.IsInRole("Administrators"))
    // Zugriff für Administratoren
    return o => true; // alle Flüge
   else // Zugriff für Standardbenutzer
    return o => o.Abflugort.StartsWith("R") == true && o.Datum > DateTime.Now && o.FreiePlaetze > 0; // nur Flüge in der der Zukunft, die noch nicht ausgebucht sind
  }

  // Interceptor für Piloten
  //[QueryInterceptor("Pilot")]
  //[PrincipalPermission(SecurityAction.Demand, Role = "FlugAdminsEuropa")]
  //public Expression<Func<Pilot, bool>> FilterPilot()
  //{
  // return o => true;
  //}



  // Zusätzliche Dienstoperation
  // Aufruf: http://localhost:87/WWWingsDataService.svc/GetInfo
  [WebGet]
  public string GetInfo()
  {
   return "User: " + System.Environment.UserName + " Admin: " + HttpContext.Current.User.IsInRole("Administrators") + " Machine: " + System.Environment.MachineName;
  }


  // Zusätzliche Dienstoperation mit einem Parameter und elementarem Datentyp als Rückgabewert
  // Aufruf: http://localhost:87/WWWingsDataService.svc/CountFluegeAusEinerStadt?Stadt='Rom'
  [WebGet]
  public int CountFluegeAusEinerStadt(string Stadt)
  {
   if (string.IsNullOrEmpty(Stadt))
   {
    throw new ArgumentNullException("Stadt", "Stadt darf nicht leer sein!");
   }
   return (from f in this.CurrentDataSource.Flug where f.Abflugort == Stadt select f).Count();
  }

  // Zusätzliche Dienstoperation mit Liste von elementarem Datentyp als Rückgabewert
  // Aufruf: http://localhost:87/WWWingsDataService.svc/GetOrte
  [WebGet]
  public List<string> GetOrte()
  {
   throw new ApplicationException("Datenbank nicht verfügbar");
   return (from f in this.CurrentDataSource.Flug select f.Abflugort).Distinct().ToList();
  }



  // Zusätzliche Dienstoperation mit generischem Filter-Parameter
  // Aufruf: http://localhost:87/WWWingsDataService.svc/GetCount?where='it.Abflugort="Rom"'
  [WebGet]
  public int GetCount(string where)
  {
   return String.IsNullOrEmpty(where) ? CurrentDataSource.Flug.Count() : CurrentDataSource.Flug.Where(where).Count();
  }

  // Zusätzliche Dienstoperation (komplett generisch)
  // Aufruf: http://localhost:87/WWWingsDataService.svc/FluegeVon?von='Rom'
  [WebGet]
  public IQueryable<de.WWWings.GO_STE.Flug> FluegeVon(string Stadt)
  {
   if (string.IsNullOrEmpty(Stadt))
   {
    throw new ArgumentNullException("Stadt",
              "Stadt darf nicht leer sein!");
   }
   return (from f in this.CurrentDataSource.Flug where f.Abflugort == Stadt select f);
  }

  // Zusätzliche Dienstoperation
  // Aufruf: http://localhost:87/WWWingsDataService.svc/FluegeVonNach?von='Rom'&nach='Berlin'
  [WebGet]
  public List<de.WWWings.GO_STE.Flug> FluegeVonNach(string von, string nach)
  {
   if (string.IsNullOrEmpty(von))
   {
    throw new ArgumentNullException("von", "von darf nicht leer sein!");
   }
   if (string.IsNullOrEmpty(nach))
   {
    throw new ArgumentNullException("nach", "nach darf nicht leer sein!");
   }
   return (from f in this.CurrentDataSource.Flug where f.Abflugort == von && f.Zielort == nach select f).ToList();
  }

  // Zusätzliche Dienstoperation
  // Aufruf: http://localhost:87/WWWingsDataService.svc/FluegeVonNach2?von='Rom'&nach='Moskau'&$filter=FreiePlaetze gt 0&$skip=0&$top=5
  [WebGet]
  public IQueryable<de.WWWings.GO_STE.Flug> FluegeVonNach2(string von, string nach)
  {
   if (string.IsNullOrEmpty(von))
   {
    throw new ArgumentNullException("von", "von darf nicht leer sein!");
   }
   if (string.IsNullOrEmpty(nach))
   {
    throw new ArgumentNullException("nach", "nach darf nicht leer sein!");
   }
   return (from f in this.CurrentDataSource.Flug where f.Abflugort == von && f.Zielort == nach select f);
  }

 }
}
