using System;
using System.Collections.Generic;
using System.Text;

using System.Management.Automation;
using System.Management;
using de.WWWings.PassagierSystem;
using de.WWWings;

namespace de.ITVisions.PowerShell
{
 [Cmdlet(VerbsCommon.New, "Buchung", DefaultParameterSetName = "")]
 public class New_Buchung : PSCmdlet
 {

  [Parameter(Position = 1, ValueFromPipeline = true, HelpMessage = "Pflichtangabe: Passagiernummer")]
  public long PassagierID = 0;

  [Parameter(Position = 0, ValueFromPipeline = true, HelpMessage = "Pflichtangabe: Flugnummer")]
  public long FlugID = 0;

  [Parameter(ValueFromPipeline = true, HelpMessage = "Pflichtangabe: Passagiernummer")]
  public de.WWWings.PassagierSystem.Passagier Passagier = null;

  [Parameter(ValueFromPipeline = true, HelpMessage = "Pflichtangabe: Flugnummer")]
  public de.WWWings.Flug Flug = null;

  protected override void ProcessRecord()
  {
   Util.SetConnection(this);
   // Kein Passagierobjekt.  Passagiernummer?
   if ((this.Passagier == null) && (this.PassagierID > 0))
    try
    {
     this.Passagier = de.WWWings.Passagier_BLManager.HolePassagier(this.PassagierID);
    }
    catch (Exception)
    {
     this.WriteError(new ErrorRecord(new ApplicationException("Ungltige Passagiernummer!"), "1", ErrorCategory.InvalidData, this.PassagierID));
     return;
    }

   // Kein Flugobjekt. Flugnummer?
   if ((this.Flug == null) && (this.FlugID > 0))
    try
    {
     this.Flug = de.WWWings.FlugBLManager.HoleFlug(this.FlugID);
    }
    catch (Exception)
    {
     this.WriteError(new ErrorRecord(new ApplicationException("Ungltige Flugnummer!"), "2", ErrorCategory.InvalidData, this.PassagierID));
     return;
    }


   if (this.Flug == null)
   {
    this.WriteError(new ErrorRecord(new ApplicationException("Angaben zum Flug fehlen!"), "3", ErrorCategory.InvalidData, this.PassagierID));
    return;
   }


   if (this.Passagier == null)
   {
    this.WriteError(new ErrorRecord(new ApplicationException("Angaben zum Passagier fehlen!"), "4", ErrorCategory.InvalidData, this.PassagierID));
    return;
   }



   if ((this.FlugID > 0) && (this.PassagierID > 0))
   {

    try
    {
     Buchung b = Buchung_BLManager.Buchen(this.PassagierID, this.FlugID);
     this.WriteObject(b);
    }
    catch (Exception)
    {
     this.WriteError(new ErrorRecord(new ApplicationException("Buchungsproblem"), "5", ErrorCategory.InvalidData, this.PassagierID));
    }

   }
   else
   {
    try
    {
     Buchung b = de.WWWings.Buchung_BLManager.Buchen(this.Flug, this.Passagier);
     this.WriteObject(b);
    }
    catch (Exception)
    {
     this.WriteError(new ErrorRecord(new ApplicationException("Buchungsproblem"), "6", ErrorCategory.InvalidData, this.PassagierID));
    }
   }
  }
 }
}


