﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.Linq;
using System.IO;

namespace ITVisions.SilverlightUtil
{

  /// <summary>
  /// Hilfsfunktionen für CSV-Dateiexporte
  /// </summary>
  public class CSVUtil
  {

    public char TRENNER = ';';

    public string SaveToFile(string data)
    {
      SaveFileDialog sfd = new SaveFileDialog()
      {
        DefaultExt = "csv",
        Filter = "CSV Files (*.csv)|*.csv|All files (*.*)|*.*",
        FilterIndex = 1
      };
      if (sfd.ShowDialog() == true)
      {
        using (Stream stream = sfd.OpenFile())
        {
          using (StreamWriter writer = new StreamWriter(stream, System.Text.Encoding.Unicode))
          {
            //writer.Write(System.Text.Encoding.Unicode.GetPreamble());
            writer.Write(data);
            writer.Close();
          }
          stream.Close();
          return sfd.SafeFileName;
        }

      }
      return "";
    }

    /// <summary>
    /// Aus ITVAppUtil
    /// </summary>
    private string FormatCSVField(string data)
    {
      return String.Format("\"{0}\"",
          data.Replace("\"", "\"\"\"")
          .Replace("\n", "")
          .Replace("\r", "")
          );
    }



    /// <summary>
    /// Aus ITVAppUtil
    /// </summary>
    public string ExportDataGrid(bool withHeaders, DataGrid grid, System.Collections.IList source = null)
    {
      string colPath;
      System.Reflection.PropertyInfo propInfo;
      System.Windows.Data.Binding binding;
      System.Text.StringBuilder strBuilder = new System.Text.StringBuilder();
      if (source == null) source = (grid.ItemsSource as System.Collections.IList);
      if (source == null) return "";

      List<string> headers = new List<string>();
      grid.Columns.ToList().ForEach(col =>
      {
        if (col is DataGridBoundColumn)
        {
          headers.Add(FormatCSVField(col.Header.ToString()));
        }
      });
      strBuilder
      .Append(String.Join(TRENNER.ToString(), headers.ToArray()))
      .Append("\r\n");

      foreach (Object data in source)
      {
        List<string> csvRow = new List<string>();
        foreach (DataGridColumn col in grid.Columns)
        {
          if (col is DataGridBoundColumn)
          {
            binding = (col as DataGridBoundColumn).Binding;
            colPath = binding.Path.Path;
            propInfo = data.GetType().GetProperty(colPath);
            if (propInfo != null)
            {
              csvRow.Add(FormatCSVField(propInfo.GetValue(data, null).ToString()));
            }
          }
        }
        strBuilder
            .Append(String.Join(TRENNER.ToString(), csvRow.ToArray()))
            .Append("\r\n");
      }


      return strBuilder.ToString();
    }
  }
}
