﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.IO.IsolatedStorage;
using System.Runtime.Serialization;
using System.IO;
using System.Linq;

namespace ITVisions.SilverlightUtil
{

  /// <summary>
  /// Hilfsroutinen für Isolated Storage
  /// </summary>
  public class ISOUtil
  {

   public string fileName = "Cache.xml";

    /// <summary>
    /// Laden einer beliebigen Objektmenge aus Isolated Storgae
    /// </summary>
    public LISTENTYP LoadFromCache<EINZELTYP, LISTENTYP>()
      where LISTENTYP : class
    {
      IsolatedStorageFile iso = IsolatedStorageFile.GetUserStoreForApplication();
      if (!iso.FileExists(fileName))
      { iso.Dispose(); MessageBox.Show("Kein Cache vorhanden!", "Laden aus Cache", MessageBoxButton.OK); return null; }

      // Deserialisieren in Stream
      IsolatedStorageFileStream stream = iso.OpenFile(fileName, FileMode.Open);
      DataContractSerializer dc = new DataContractSerializer(typeof(LISTENTYP), new List<System.Type>() { typeof(EINZELTYP) });
      LISTENTYP fluege = dc.ReadObject(stream) as LISTENTYP;
      stream.Close();
      iso.Dispose();
      return fluege;
    }



    /// <summary>
    /// Speichern einer beliebigen Objektmenge in Isolated Storgae
    /// </summary>
    public string SaveInCache<EINZELTYP, LISTENTYP>(LISTENTYP fluege)
    {

      if (fluege == null || (fluege as IEnumerable<EINZELTYP>).Count() == 0) { MessageBox.Show("Keine Datensätze vorhanden!", "Fehler beim Speichern im Cache", MessageBoxButton.OK); return ""; }

      // Speichern
      using (IsolatedStorageFile iso =
        IsolatedStorageFile.GetUserStoreForApplication())
      {

        // Datei löschen, falls schon da
        if (iso.FileExists(fileName)) { iso.DeleteFile(fileName); }

        // Stream aufmachen
        IsolatedStorageFileStream fileStream;
        fileStream = new IsolatedStorageFileStream(fileName, FileMode.OpenOrCreate, iso);

        // Serialisierung
        DataContractSerializer dc = new DataContractSerializer(typeof(LISTENTYP), new List<System.Type>() { typeof(EINZELTYP) });

        try
        {
          dc.WriteObject(fileStream, fluege);
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.Message, "Fehler beim Speichern im Cache", MessageBoxButton.OK);
        }

        // Aufräumen
        fileStream.Close();

        string s = (fluege as IEnumerable<EINZELTYP>).Count() + " Datensätze im Cache gespeichert (" + Math.Round((iso.AvailableFreeSpace / 1048576.0f), 2).ToString() + " von " + Math.Round((iso.Quota / 1048576.0f), 2).ToString() + " MB im Cache frei!)";

        return s;
      }
    }

    

  }
}
