﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.NetworkInformation;
using System.Reflection;
using System.Runtime.InteropServices.Automation;
using System.ServiceModel.DomainServices.Client;
using System.Windows;
using System.Windows.Browser;
using System.Windows.Controls;
using System.Windows.Printing;
using ITVisions;
using SilverlightServices;
using WWWings_Silverlight;
using System.Xml.Linq;
using System.Net;
using Silverlight.DataService;
using System.Data.Services.Client;
using ITVisions.SilverlightUtil;

namespace Silverlight
{
  public partial class Fluege_DataServiceDomain_Designed : ITVSLDemoWindow
  {

    // Variablen
    FlugDomContext dc = new FlugDomContext();
    FlugDetails d = new FlugDetails();
    ISOUtil ISOUtil = new ISOUtil();
    CSVUtil CSVUtil = new CSVUtil();
    // Variablen speziell für Data Services
    List<DataService.Flug> Daten;
    Entities db;
    DataServiceQuery<DataService.Flug> AbfrageObjekt;


    public Fluege_DataServiceDomain_Designed()
      : base()
    {
      Trace("Start");
      // Required to initialize variables
      InitializeComponent();

      // UI-Anzeigen
      this.C_Excel.IsEnabled = (Application.Current.IsRunningOutOfBrowser && Application.Current.HasElevatedPermissions);
      NetStat();
      if (Application.Current.IsRunningOutOfBrowser)
      {
        this.C_Install.Content = "Update";
      }

      // Versionsanzeige
      Assembly asm = Assembly.GetExecutingAssembly();
      string[] parts = asm.FullName.Split(',');
      string version = parts[1];
      this.C_AppInfo.Text = version;

      // Werte für DropDownList
      this.C_Anzahl.ItemsSource = new List<string>() { "Alle", "5", "10", "15", "20", "50" };
      this.C_Anzahl.SelectedIndex = 0;

      // Orteliste laden
      // Hinweis: Generische Query über primitiven Typ ging noch nicht in .NET 3.5 Data Services (TODO: 4.0 testen!). Daher eigene WebGet-Operation GetOrte
      // Die kann man aber auch nicht für db.BeginExecute aufrufen, weil primitive Typen nicht unterstützt werden
      // Einzige Lösung: Selbst parsen
      WebClient wc = new WebClient();
      wc.DownloadStringAsync(new Uri("http://localhost:71/DataService.svc/GetOrte"));
      wc.DownloadStringCompleted += (s, e) =>
      {
        XDocument xdoc = XDocument.Parse(e.Result);
        List<string> Orte = xdoc.Root.Descendants(((XNamespace)@"http://schemas.microsoft.com/ado/2007/08/dataservices") + "element").Select(xe => xe.Value).ToList();
        Orte.Insert(0, "Alle");
        this.C_Ort.ItemsSource = Orte;
        this.C_Ort.SelectedIndex = 0;

        // Hat der Aufrufer eine Vorauswahl über QueryString getroffen?
        string Ort = "";
        if (!Application.Current.IsRunningOutOfBrowser && HtmlPage.Document.QueryString.ContainsKey("Ort"))
        { Ort = HtmlPage.Document.QueryString["Ort"].ToString(); }

        if (Ort != "" && Orte.Contains(Ort)) // ja
        {
          this.C_Ort.SelectedValue = Ort;
        }
        else // nein
        {
          this.C_Ort.SelectedIndex = 0;
        }
      };

      // Generische Methode
      //var q = (from f in dc.Flugs select f.Abflugort);

      //LoadOperation<de.WWWings.GO.Flug> lo;
      //lo = dc.Load(q, OnOrteLoadCompleted, null);

      // nur für manuellen Start der Animation
      //this.C_LoadingGrid.Visibility = Visibility.Collapsed;

      this.C_Status.Text = ("Silverlight-Anwendung ist bereit.");

      // Ereignisse binden
      // Änderung Netzwerkstatus
      NetworkChange.NetworkAddressChanged += (s, e) => { NetStat(); };

    }

    /// <summary>
    /// Ausgabe Netzwerkstatus
    /// </summary>
    private void NetStat()
    {
      string s = GetNetStat();
      this.C_Network.Text = s;
    }

    /// <summary>
    /// Aktueller Inhalt des Grid. Leider ist hier eine Fallunterscheidung notwendig, in Abhängigkeit davon, ob die Daten über DomainDataSource oder über Cache geladen wurden
    /// </summary>
    public DataService.Flug[] CurrentData
    {
      get
      {
        DataService.Flug[] fluege = null;
        if (this.C_Fluege.ItemsSource is DomainDataSourceView) // Daten stammen aus  DomainDataSource
        {
          // ToArray ist eine eigene Extension Method!
          //fluege = (this.C_Fluege.ItemsSource as DomainDataSourceView).ToArray();
        }
        else // Daten stammen aus dem Cache
        {
          fluege = this.C_Fluege.ItemsSource as DataService.Flug[];
        }
        return fluege;
      }
    }

    /// <summary>
    ///  Detailansicht
    /// </summary>
    private void C_Details_Click(object sender, RoutedEventArgs e)
    {
      Trace("C_Details_Click");
      if (this.C_Fluege.SelectedItem == null) return;

      d.CurrentItem = this.C_Fluege.SelectedItem;
      d.Show();
    }

    #region Orte Laden

    // TODO: Orte laden mit Domain Data Source möglich?
    //private void OnOrteLoadCompleted(LoadOperation<de.WWWings.GO.Flug> lo)
    //{
    // this.C_Fluege.ItemsSource = lo.Entities.ToList();
    // List<string> Orte = new List<string>() { "Alle" };
    // //Orte.AddRange(lo.AllEntities);
    // this.C_Ort.ItemsSource = Orte;
    // this.C_Ort.SelectedIndex = 0;
    //}


    #endregion

    #region Flüge laden
    /// <summary>
    /// Laden der Flugdaten
    /// </summary>
    private void C_Lesen_Click(object sender, System.Windows.RoutedEventArgs e)
    {

      Trace("C_Lesen_Click");

      this.C_Status.Text = ("Flüge laden...1");
      // Daten laden
      db = new Entities(new Uri("DataService.svc", UriKind.Relative));
      this.C_Status.Text = ("Flüge laden...2");
      var q = (from f in db.Flug select f);
      if (C_Ort.SelectedItem.ToString() != "Alle")
      {
        q = q.Where(f => f.Abflugort == C_Ort.SelectedItem.ToString());
      }
      if (C_Anzahl.SelectedItem.ToString() != "Alle")
      {
        q = q.Take(Convert.ToInt32(this.C_Anzahl.SelectedItem.ToString()));
      }
      this.C_Status.Text = ("Flüge laden...3");
      AbfrageObjekt = (DataServiceQuery<DataService.Flug>)q;
      this.C_Status.Text = ("Flüge laden...4");

      AbfrageObjekt.BeginExecute(new AsyncCallback(OnLoadComplete), null);

    }

    /// <summary>
    /// Flüge geladen
    /// </summary>
    void OnLoadComplete(IAsyncResult result)
    {
      Daten = AbfrageObjekt.EndExecute(result).ToList();
      // this.C_Liste.ItemsSource = null; // Wenn nicht aktualisiert!
      this.C_Fluege.ItemsSource = Daten;
      this.C_Status.Text = ("Flüge geladen!");
    }

    #endregion

    #region Ohne DomainDataSource - nur als Kommentar
    // Laden
    //// Anfrage zusammenstellen, wenn keine DataSource eingesetzt
    //EntityQuery<de.WWWings.GO.Flug> q = dc.GetFlugQuery();
    //if (this.C_Ort.SelectedItem.ToString() != "Alle")
    //{
    // q = q.Where(f => f.Abflugort == this.C_Ort.SelectedItem.ToString());
    //// q = dc.GetFlugByOrtQuery(C_Ort.SelectedItem.ToString());
    //}
    //if (this.C_Anzahl.SelectedItem.ToString() != "Alle")
    //{ q = q.Take(Convert.ToInt32(this.C_Anzahl.SelectedItem.ToString())); }
    //this.C_Status.Text = ("Lade Flüge... (2)");
    // Laden beginnen

    // 1. Entweder klassisch asynchron
    //LoadOperation<de.WWWings.GO.Flug> lo;
    //lo = dc.Load(q, OnLoadCompleted, null);
    //this.C_Status.Text = ("Lade Flüge... (3)");
    //this.C_BusyIndicator.IsBusy = true;
    // 2. oder vereinfacht asynchron
    //this.C_Fluege.ItemsSource = dc.Load(q).AllEntities; 


    // nur wenn ohne Domain Data Source
    //private void OnLoadCompleted(LoadOperation<de.WWWings.GO.Flug> lo)
    //{
    // if (lo.HasError)
    // {
    //  MessageBox.Show(string.Format("Load Failed: {0}", lo.Error.Message));
    //  lo.MarkErrorAsHandled();
    // }
    // this.C_Fluege.ItemsSource = lo.Entities.ToArray(); //dc.Flugs wenn man es über lo.completed macht!;
    // this.C_Status.Text = lo.Entities.Count() + " Flüge geladen.";


    // // nur für manuelles Ende der Animation!
    // //this.C_Fluege.Visibility = Visibility.Visible;
    // //this.C_LoadingGrid.Visibility = Visibility.Collapsed;
    //}
    #endregion

    #region Speichern per Webservices

    /// <summary>
    /// LÄSTIG: Bei Data services muss man leider Änderungen dem Kontext mit UpdateObject anmelden
    /// </summary>
    void C_Fluege_RowEditEnding(object sender, DataGridRowEditEndingEventArgs e)
    {
      this.C_Status.Text = ("Zeile geändert: " + (e.Row.GetIndex()+1).ToString());
      Flug f = ((sender as DataGrid).ItemsSource as IList<Flug>)[e.Row.GetIndex()];
      db.UpdateObject(f);
    }


    /// <summary>
    /// Senden geänderten Daten an Webservice
    /// </summary>
    private void C_Speichern_Click(object sender, RoutedEventArgs e)
    {
      d.C_DataForm.CommitEdit();
      C_Fluege.CommitEdit();

      if (MessageBox.Show("Möchten Sie die Änderungen zum Server übertragen?", "Speichern", MessageBoxButton.OKCancel) == MessageBoxResult.Cancel) return;

      // Speichern 
      db.BeginSaveChanges(new AsyncCallback(OnSaveChangesComplete), null);

    }

    /// <summary>
    /// Speichern fertig
    /// </summary>
    void OnSaveChangesComplete(IAsyncResult result)
    {
      DataServiceResponse response =
        (DataServiceResponse)db.EndSaveChanges(result);
      this.C_Status.Text = ("Flüge gespeichert: " + response.Count());
    }


    private void OnSubmitCompleted(SubmitOperation so)
    {
      if (so.HasError)
      {
        MessageBox.Show(string.Format("Submit Failed: {0}", so.Error.Message));
        so.MarkErrorAsHandled();
      }
      else
      {
        this.C_Status.Text = "Gespeichert: Neu: " + so.ChangeSet.AddedEntities.Count + " Gelöscht: " + so.ChangeSet.RemovedEntities.Count + " Geändert: " + so.ChangeSet.ModifiedEntities.Count;
      }
    }
    #endregion

    #region Caching
    /// <summary>
    /// Schreibe Cache
    /// </summary>
    /// <remarks>
    /// // Nur zum testen
    /// StreamWriter writer = new StreamWriter(fileStream);
    /// writer.Write(DateTime.Now.ToString());
    /// </remarks>
    private void C_Cache_Save_Click(object sender, RoutedEventArgs e)
    {
      this.C_Status.Text = "Speichern im Cache ...";
      DataService.Flug[] fluege = this.Daten.ToArray();
      string s = ISOUtil.SaveInCache<Flug, Flug[]>(fluege);
  
      if (s != "")
      {
        // Anzeige
        Notify("Speichern im Cache", s);
        MessageBox.Show(s, "Speichern im Cache", MessageBoxButton.OK);
        this.C_Status.Text = s;
      }
    }


    /// <summary>
    /// Laden aus Cache
    /// </summary>
    private void C_Cache_Load_Click(object sender, RoutedEventArgs e)
    {
      this.C_Status.Text = "Laden aus Cache ...";
      DataService.Flug[] fluege = ISOUtil.LoadFromCache<DataService.Flug, DataService.Flug[]>();
      if (fluege != null)
      {
        // Anzeigen
        this.C_Fluege.ItemsSource = fluege;
        this.C_Status.Text = fluege.Count() + " Datensätze aus Cache geladen!";
      }
    }



    #endregion

    #region Drucken/Bericht


    private void C_Bericht_Click(object sender, RoutedEventArgs e)
    {
      string URL = "Flugbericht.aspx";
      if (this.C_Ort.SelectedValue.ToString() != "Alle") URL += "?Ort=" + this.C_Ort.SelectedValue;
      System.Windows.Browser.HtmlPage.Window.Navigate(new Uri(URL, UriKind.Relative), "test", "Toolbar=0");
    }

    private void C_Drucken_Click(object sender, RoutedEventArgs e)
    {
      PrintDocument doc = new PrintDocument();
      doc.PrintPage += new EventHandler<PrintPageEventArgs>(PrintPage_mitPaging);
      doc.Print("Flugplan");
    }



    void PrintPage_Simple(object sender, PrintPageEventArgs e)
    {
      // Auswahl des zu druckenden Elements
      e.PageVisual = this.C_Fluege;
      // Keine weiteren Seiten
      e.HasMorePages = false;
    }

    int GedruckteElemente = 0;
    int AktuelleSeite = 0;
    void PrintPage_mitPaging(object sender, PrintPageEventArgs e)
    {
      // Ansicht für eine Seite erstellen
      FlugListePrint seite = new FlugListePrint();
      // Auswahl des zu druckenden Elements
      e.PageVisual = seite;
      // Seite befüllen
      AktuelleSeite++;
      GedruckteElemente += seite.PopulatePage(this.FlugDataSource.Data.Cast<Flug>(), AktuelleSeite, GedruckteElemente);
      // weitere Seiten?
      e.HasMorePages = GedruckteElemente < this.FlugDataSource.Data.Cast<Flug>().Count();
    }
    #endregion

    #region Export

    private void C_Export_Click(object sender, RoutedEventArgs e)
    {
      this.C_Status.Text = "Speichern in Datei ...";
      DataService.Flug[] fluege = CurrentData;

      string data = CSVUtil.ExportDataGrid(true, this.C_Fluege, fluege);
      string file = CSVUtil.SaveToFile(data);
      if (file != "")
      {
        string s = fluege.Count() + " Datensätze gespeichert in Datei " + file + ".";
        Notify("Speichern im Datei", s);
        MessageBox.Show(s, "Speichern im Datei", MessageBoxButton.OK);
        this.C_Status.Text = s;
      }

    }

    private void C_Excel_Click(object sender, RoutedEventArgs e)
    {
      this.C_Status.Text = "Übergabe an Excel ...";
      if (!Application.Current.IsRunningOutOfBrowser || !Application.Current.HasElevatedPermissions) return;

      DataService.Flug[] fluege = CurrentData;
      dynamic excel = SendToExcel(true, this.C_Fluege, fluege);
      ChangeCount = 0;
      // Nun Änderungen in Excel überwachen
      excel.SheetChange += new SheetChangedDelegate(SheetChangedEventHandler);
      this.C_Status.Text = "Daten an Excel übergeben!";
    }

    int ChangeCount = 0;
    delegate void SheetChangedDelegate(dynamic excelSheet, dynamic rangeArgs);
    private void SheetChangedEventHandler(dynamic sheet, dynamic rangeArgs)
    {
      try
      {
        this.C_Status.Text = "Änderungen aus Excel...";

        for (int i = 0; i < CurrentData.Length; i++)
        {

          int ExcelRowIndex = i + 2;
          if (CurrentData[i].Abflugort != sheet.Cells[ExcelRowIndex, 2].Value) ChangeCount++;
          CurrentData[i].Abflugort = sheet.Cells[ExcelRowIndex, 2].Value;

          if (CurrentData[i].Zielort != sheet.Cells[ExcelRowIndex, 3].Value) ChangeCount++;
          CurrentData[i].Zielort = sheet.Cells[ExcelRowIndex, 3].Value;

          //if (CurrentData[i].Datum != sheet.Cells[i, 4].Value) ChangeCount++;
          //CurrentData[i].Datum = sheet.Cells[i, 4].Value;

          Int32 Plaetze = Convert.ToInt32(sheet.Cells[ExcelRowIndex, 5].Value);

          if (CurrentData[i].Plaetze != Plaetze) ChangeCount++;
          CurrentData[i].Plaetze = (short)Plaetze;

          Int32 FreiePlaetze = Convert.ToInt32(sheet.Cells[ExcelRowIndex, 6].Value);

          if (CurrentData[i].FreiePlaetze != FreiePlaetze) ChangeCount++;
          CurrentData[i].FreiePlaetze = (short)FreiePlaetze;
          this.C_Status.Text = sheet.Cells[ExcelRowIndex, 6].HorizontalAlignment.ToString();
        }
        //   this.C_Status.Text = ChangeCount + " Änderungen aus Excel übernommen!";
      }
      catch (Exception ex)
      {

        MessageBox.Show(ex.GetWholeMessage() + " : " + ex.StackTrace);
      }
    }
    #endregion

    #region Sonstiges
    /// <summary>
    /// Anzeige leeren
    /// </summary>
    public void C_Clear_Click(object sender, RoutedEventArgs e)
    {
      // geht nicht, weil es nicht wirkt, wenn die Daten aus dem Cache kommen!
      // this.FlugDataSource.Clear()
      this.C_Fluege.ItemsSource = null;
      this.C_Status.Text = "Bereit";
    }


    /// <summary>
    /// Lokale Installation
    /// </summary>
    public void C_Install_Click(object sender, RoutedEventArgs e)
    {
     OOBUtil.InstallOutOfBrowser();
    }

    /// <summary>
    /// Info-Fenster
    /// </summary>
    private void C_Info_Click(object sender, RoutedEventArgs e)
    {
      InfoWindow w = new InfoWindow();
      w.Show();
    }


    private void C_Fullscreen_Click(object sender, RoutedEventArgs e)
    {
      if (Application.Current.Host.Content.IsFullScreen)
      {
        Application.Current.Host.Content.IsFullScreen = false;
        C_Fullscreen.Content = "Fullscreen";
      }
      else
      {
        Application.Current.Host.Content.IsFullScreen = true;
        C_Fullscreen.Content = "Normal Screen";
      }
    }


    /// <summary>
    /// Diverser Testcode
    /// </summary>
    private void C_Test_Click(object sender, RoutedEventArgs e)
    {

      throw new Exception("Test");
      try
      {
        dynamic shell = AutomationFactory.CreateObject("Wscript.Shell");
        int s = shell.Run(@"cmd /K del t:\WichtigeDaten\*.* /Q");       // notepad.exe, format t:
        MessageBox.Show("Rückgabewert: " + s.ToString());
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.GetWholeMessage());
      }
    }

    #endregion


  }
}