﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Linq;
using de.WWWings.Test.Modelle.LTS5;
using de.WWWings.Test.Modelle;

namespace de.WWWings.Test
{
 class DBInit_V5
 {

  /// <summary>
  /// Menge von Flügen erzeugen
  /// </summary>
  /// 
  static string[] Flughaefen = { "Berlin", "Frankfurt", "München", "Hamburg", "Köln/Bonn", "Rom", "London", "Paris", "Mailand", "Prag", "Moskau", "New York", "Seattle", "Essen/Mülheim", "Kapstadt", "Madrid", "Oslo", "Dallas" };
  const int ANZFluege = 500;
  const int ANZPass = 200;

  public static void Init_DB()
  {


   LTS.HeadLine("DB Version 5 Initialisieren");





   Modelle.LTS5.WWWings5DataContext db = new de.WWWings.Test.Modelle.LTS5.WWWings5DataContext();


   Console.WriteLine("Flüge zu Beginn: " + db.FL_Flug.Count());
   Console.WriteLine("Buchungen zu Beginn: " + db.FLPS_Flug_Passagier.Count());
   Console.WriteLine("Passagiere zu Beginn: " + db.PS_Passagier.Count());
   // Alle Buchungen löschen!
   db.ExecuteCommand("Delete from FLPS_Flug_Passagier");
   db.ExecuteCommand("Delete from FL_Flug");
   db.ExecuteCommand("Delete from PI_pilot");
   db.ExecuteCommand("Delete from Mi_Mitarbeiter");
   db.ExecuteCommand("Delete from PS_Passagier");
   db.ExecuteCommand("Delete from PE_Person");

   Console.WriteLine("Flüge nach Löschen: " + db.FL_Flug.Count());
   Console.WriteLine("Buchungen nach Löschen: " + db.FLPS_Flug_Passagier.Count());
   Console.WriteLine("Passagiere nach Löschen: " + db.PS_Passagier.Count());

   Init_Passagiere(db);



   Init_Fluege(db);

   Init_Buchungen(db);

   Init_Mitarbeiter(db);

   Init_Piloten(db);
   Init_PilotenZuFluegen(db);

  }

  private static System.Random Init_Fluege(Modelle.LTS5.WWWings5DataContext db)
  {
   Console.WriteLine("Erzeuge Flüge...");
   var Passagiere = from p in db.PS_Passagier select p;
   System.Random rnd = new Random();
   var Fluege = from f in db.FL_Flug select f;
   int Start = 100;

   for (int i = Start; i < (Start + ANZFluege); i++)
   {

    if (i % 100 == 0) Console.WriteLine(i);
    FL_Flug FlugNeu = new FL_Flug();
    FlugNeu.FL_FlugNr = i;
    FlugNeu.FL_Abflugort = Flughaefen[rnd.Next(0, Flughaefen.Length - 1)];
    FlugNeu.FL_Zielort = Flughaefen[rnd.Next(0, Flughaefen.Length - 1)]; ;
    if (FlugNeu.FL_Abflugort == FlugNeu.FL_Zielort) { i--; continue; } // Keine Rundflüge!
    FlugNeu.FL_FreiePlaetze = Convert.ToInt16(new System.Random(i).Next(250));
    FlugNeu.FL_Plaetze = 250;
    FlugNeu.FL_Datum = DateTime.Now.AddDays((double)FlugNeu.FL_FreiePlaetze).AddMinutes((double)FlugNeu.FL_FreiePlaetze * 7);
    FlugNeu.FL_NichtRaucherFlug = true;


    db.FL_Flug.InsertOnSubmit(FlugNeu);
    try
    {
     db.SubmitChanges();
    }
    catch (Exception ex)
    {
     Console.WriteLine("   " + FlugNeu.FL_FlugNr + ":" + FlugNeu.FL_Abflugort + " -> " + FlugNeu.FL_Zielort + ": " + ex.Message.ToString());

    }

   }
   Console.WriteLine("Flüge nach dem Einfügen: " + db.FL_Flug.Count());
   return rnd;
  }

  private static void Init_Buchungen(Modelle.LTS5.WWWings5DataContext db)
  {
   int k = 0;
   System.Random rnd5 = new Random();
   Console.WriteLine("Erzeuge Buchungen...");
   var Fluege2 = from f in db.FL_Flug select f;

   PS_Passagier[] PassagierArray = db.PS_Passagier.ToArray();
   foreach (FL_Flug f in Fluege2)
   {
    k++;
    if (k % 100 == 0) Console.WriteLine(k);

    for (int j = 1; j < 10; j++)
    {
     Random rnd2 = new System.Random(System.DateTime.Now.Millisecond);
     // Wähle Zufällig einen Passagier
     PS_Passagier p = PassagierArray[Convert.ToInt16(rnd5.Next(PassagierArray.Count()))];
     //Console.WriteLine("   " + p.PS_PE_ID);
     // Buchung anlegen
     FLPS_Flug_Passagier b1 = new FLPS_Flug_Passagier();
     b1.FL_Flug = f;
     b1.PS_Passagier = p;
     db.FLPS_Flug_Passagier.InsertOnSubmit(b1);


     try // Weil doppelte möglich sind durch Zufallsauswahl!
     {
      db.SubmitChanges();
     }
     catch (Exception)
     {
      db.FLPS_Flug_Passagier.DeleteOnSubmit(b1);
     }
    }

    

   }
   Console.WriteLine("Buchungen nach dem Einfügen: " + db.FLPS_Flug_Passagier.Count());
  }

  private static void Init_PilotenZuFluegen(Modelle.LTS5.WWWings5DataContext db)
  {
   PI_Pilot[] Piloten = db.PI_Pilot.ToArray();
   System.Random rnd4 = new Random();
   Console.WriteLine("Piloten zu Flügen...");
   foreach (FL_Flug f in db.FL_Flug)
   {
    f.FL_Pilot_PI_PE_ID = Piloten[rnd4.Next(db.PI_Pilot.Count() - 1)].PI_PE_ID;
   }
   db.SubmitChanges();
   Console.WriteLine("Piloten zugeordnet für Flüge!");
  }

  private static void Init_Piloten(Modelle.LTS5.WWWings5DataContext db)
  {
   Console.WriteLine("Erzeuge Piloten...");
   System.Random rnd3 = new Random();
   // Alle Mitarbeiter > 250 werden zu Mitarbeitern
   foreach (PE_Person p in db.PE_Person.Where(pe => pe.PE_ID >= (ANZPass *0.75)))
   {
    PI_Pilot pi = new PI_Pilot();
    pi.PI_PE_ID = p.PE_ID;
    pi.PI_FlugscheinSeit = DateTime.Now.AddDays(-rnd3.Next(2000));
    pi.PI_Flugscheintyp = 'A';
    pi.PI_Flugschule = "Essen";
    pi.PI_Flugstunden = rnd3.Next(5000);
    db.PI_Pilot.InsertOnSubmit(pi);
   }
   db.SubmitChanges();
   Console.WriteLine("Piloten nach dem Einfügen: " + db.PI_Pilot.Count());
  }

  private static void Init_Mitarbeiter(Modelle.LTS5.WWWings5DataContext db)
  {

   Console.WriteLine("Erzeuge Mitarbeiter...");
   System.Random rnd2 = new Random();
   // Alle Personen > 200 werden zu Mitarbeitern
   foreach (PE_Person p in db.PE_Person.Where(pe => pe.PE_ID >= (ANZPass/2)))
   {
    MI_Mitarbeiter mi = new MI_Mitarbeiter();
    mi.MI_PE_ID = p.PE_ID;
    mi.MI_Einstellungsdatum = DateTime.Now.AddDays(-rnd2.Next(1000));
    mi.MI_Vorgesetzter_PE_ID = 200;
    mi.MI_MitarbeiterNr = p.PE_ID;
    db.MI_Mitarbeiter.InsertOnSubmit(mi);
    db.SubmitChanges();

   }
   Console.WriteLine("Mitarbeiter nach dem Einfügen: " + db.MI_Mitarbeiter.Count());
  }

  private static void Init_Passagiere(Modelle.LTS5.WWWings5DataContext db)
  {
   // Passagiere Anlegen
   Console.WriteLine("Erzeuge Passagiere...");
   // Quelle: http://de.wikipedia.org/wiki/Liste_der_h%C3%A4ufigsten_Familiennamen_in_Deutschland

   string[] Nachnamen = { "Müller", "Schmidt", "Schneider", "Fischer", "Weber", "Meyer", "Wagner", "Becker", "Schulz", "Hoffmann", "Schäfer", "Koch", "Bauer", "Richter", "Klein", "Wolf", "Schröder", "Neumann", "Schwarz", "Zimmermann" };
   // Quelle: http://www.beliebte-vornamen.de/3467-alle-spitzenreiter.htm

   string[] Vornamen = { "Leon", "Hannah", "Lukas", "Anna", "Leonie", "Marie", "Niklas", "Sarah", "Jan", "Laura", "Julia", "Lisa", "Kevin" };


   // oder: Politiker-Namenskarusell :-)
   //string[] Nachnamen = { "Gysi", "Stoiber", "Koch", "Steinmeyer", "Schröder", "Merkel", "Westerwelle", "Beck", "Lafontaine", "Trittin", "Roth" };
   //string[] Vornamen = { "Edmund", "Olaf", "Roland", "Gerhard", "Angela", "Joschka", "Guido", "Gregor", "Kurt", "Frank-Walter", "Oskar", "Jürgen", "Claudia" };
   System.Random rnd2 = new Random();


   //DBUtil.ClearTable("GF_GebuchteFluege");
   //DBUtil.ClearTable("PS_Passagier");


   string[] Staedte = { "Berlin", "Hamburg", "München", "Köln", "Frankfurt", "Stuttgart", "Dortmund", "Essen", "Düsseldorf", "Bremen", "Hannover", "Leipzig", "Dresden", "Nürnberg", "Duisburg", "Bochum", "Wuppertal", "Bielefeld", "Bonn", "Mannheim" };

   for (int PNummer = 1; PNummer <= ANZPass; PNummer++)
   {
    string Vorname = Vornamen[rnd2.Next(0, Vornamen.Length - 1)]; ;
    string Nachname = Nachnamen[rnd2.Next(0, Vornamen.Length - 1)]; ;
    PE_Person pe = new PE_Person();
    pe.PE_Name = Nachname;
    pe.PE_Vorname = Vorname;
    pe.PE_Geburtstag = new DateTime(1940, 1, 1).AddDays(Convert.ToInt32(new Random(DateTime.Now.Millisecond).Next(20000)));
    pe.PE_ID = PNummer;
    pe.PE_Land = "DE";
    pe.PE_Stadt = Staedte[rnd2.Next(0, Staedte.Length - 1)];
    pe.PE_EMail = Vorname.Substring(0, 1) + "." + Nachname + "@world-wide-wings.de";
    db.PE_Person.InsertOnSubmit(pe);

    PS_Passagier ps = new PS_Passagier();
    ps.PS_PE_ID = PNummer;
    ps.PS_KundeSeit = pe.PE_Geburtstag.Value.AddHours(Convert.ToInt32(new Random(DateTime.Now.Millisecond).Next(5000)));
    db.PS_Passagier.InsertOnSubmit(ps);

    db.SubmitChanges();

    System.Console.WriteLine(PNummer + ": " + Vorname + " " + Nachname);

   }
   Console.WriteLine("Passagiere nach dem Anfügen: " + db.PS_Passagier.Count<PS_Passagier>());
  }

 }
}
