﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Objects;
using de.WWWings.Test.Modelle.EF6;

namespace de.WWWings.Test.Demos_EF
{
 class EF_Konfliktdemo
 {
  /// <summary>
  /// EF_Konflikt_BeiFlug
  /// </summary>
  public static void EF_Konflikt_BeiFlug()
  {

   Console.WriteLine("EFTest_Speichern/Beispiel4_Konflikt in Prozess ID=" + System.Diagnostics.Process.GetCurrentProcess().Id);
   int flugNr = 150;

   // Kontext instanziieren unter Verwendung der Verbindungszeichenfolge aus Konfigurationsdatei
   using (WWWings6Entities modell = new WWWings6Entities())
   {

    var f = modell.Flug.Where(x => x.FlugNr == flugNr).SingleOrDefault();

    Console.WriteLine(DateTime.Now.ToLongTimeString() + ": Freie Plätze VORHER: " + f.FreiePlaetze);
    Console.ReadLine(); // Warten (zum Starten eines zweiten Prozesses!)

    // Änderung
    f.FreiePlaetze -= (short)(new System.Random(DateTime.Now.Millisecond).Next(10));
    Console.WriteLine(DateTime.Now.ToLongTimeString() + ": Freie Plätze NEU: " + f.FreiePlaetze);
    try
    {
     // Speicherversuch
     modell.SaveChanges();
    }
    catch
    {
          Console.ForegroundColor = ConsoleColor.Red;
     // Zeige aktuellen Wert aus DB
     Console.WriteLine(DateTime.Now.ToLongTimeString() + ": Fehler: Ein anderer Benutzer hat bereits geändert!");
     // Eigenes Modell, um den aktuellen Wert zu holen
     WWWings6Entities modell2 = new WWWings6Entities();
     var f2 = modell2.Flug.Where(x => x.FlugNr == flugNr).SingleOrDefault();
     Console.WriteLine(DateTime.Now.ToLongTimeString() + ": Der andere Benutzer hat gespeichert: Freie Plätze=" + f2.FreiePlaetze);
     modell2.Dispose();

     // Frage nach
     Console.WriteLine("Möchten Sie den Wert des anderen Benutzers  übernehmen (Taste '1') oder überschreiben (Taste '2')?");
     ConsoleKeyInfo key = Console.ReadKey();
     if (key.Key == ConsoleKey.D1)
     {
      Console.WriteLine("Sie haben gewählt: Option 1: übernehmen");
      modell.Refresh(RefreshMode.StoreWins, f);
     }
     else
     {
      Console.WriteLine("Sie haben gewählt: Option 2: überschreiben");
      Console.ForegroundColor = ConsoleColor.Gray;

      modell.Refresh(RefreshMode.ClientWins, f);
      EF_Util.PrintChangeTrackerInfo(modell);
      int anz = modell.SaveChanges();
      Console.WriteLine("SaveChanges: Gespeicherte Änderungen: " + anz);
     }

    }
    Console.WriteLine(DateTime.Now.ToLongTimeString() + ": Freie Plätze NACHHER: " + f.FreiePlaetze);

   } // Ende using-Block -> Dispose() wird aufgerufen
  }


  /// <summary>
  /// EF_Konflikt_BeiMetadaten_MitRowversion
  /// </summary>
  public static void EF_Konflikt_BeiMetadaten_MitRowversion()
  {

   Console.WriteLine("EFTest_Speichern/Beispiel5_Konflikt in Prozess ID=" + System.Diagnostics.Process.GetCurrentProcess().Id);
   string Name = "LetzteAktion";

   // Kontext instanziieren unter Verwendung der Verbindungszeichenfolge aus Konfigurationsdatei
   using (WWWings6Entities modell = new WWWings6Entities())
   {

    var m1 = modell.Metadaten.Where(x => x.Name == Name).SingleOrDefault();

    Console.WriteLine(DateTime.Now.ToLongTimeString() + ": Letzte Aktion - Wert: " + m1.Value);
    Console.ReadLine(); // Warten (zum Starten eines zweiten Prozesses!)

    // Änderung
    m1.Value = DateTime.Now.ToString();
    Console.WriteLine(DateTime.Now.ToLongTimeString() + ":  Letzte Aktion - Wert NEU: " + m1.Value);
    try
    {
     // Speicherversuch
     EF_Util.PrintChangeTrackerInfo(modell);
     int anz = modell.SaveChanges();
     Console.WriteLine("SaveChanges: Gespeicherte Änderungen: " + anz);
    }
    catch
    {
     Console.ForegroundColor = ConsoleColor.Red;
     // Zeige aktuellen Wert aus DB
     Console.WriteLine(DateTime.Now.ToLongTimeString() + ": Fehler: Ein anderer Benutzer hat bereits geändert!");
    // Eigenes Modell, um den aktuellen Wert zu holen
     WWWings6Entities modell2 = new WWWings6Entities();
     var m2 = modell2.Metadaten.Where(x => x.Name == Name).SingleOrDefault();
     Console.WriteLine(DateTime.Now.ToLongTimeString() + ": Der andere Benutzer hat gespeichert: Wert: " + m2.Value);
     modell2.Dispose();

     // Frage nach
     Console.WriteLine("Möchten Sie den Wert des anderen Benutzers übernehmen (Taste '1') oder überschreiben (Taste '2')?");
     ConsoleKeyInfo key = Console.ReadKey();
     if (key.Key == ConsoleKey.D1)
     {
      Console.WriteLine("Sie haben gewählt: Option 1: übernehmen");
      modell.Refresh(RefreshMode.StoreWins, m1);
     }
     else
     {
      Console.WriteLine("Sie haben gewählt: Option 2: überschreiben");

      Console.ForegroundColor = ConsoleColor.Gray;
      modell.Refresh(RefreshMode.ClientWins, m1);
      EF_Util.PrintChangeTrackerInfo(modell);
      int anz = modell.SaveChanges();
      Console.WriteLine("SaveChanges: Gespeicherte Änderungen: " + anz);
     }

    }
    Console.WriteLine(DateTime.Now.ToLongTimeString() + ": Letzte Aktion -  Wert NACHHER: " + m1.Value);

   } // Ende using-Block -> Dispose() wird aufgerufen
  }
 }
}
