﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace de.WWWings.Test
{
 class LTO_Zahlen
 {
  public static void run()
  {

   Demo_LTO_Zahlen();
   //Demo_Zahlen_Filter_MethodSyntax();

   Console.ReadLine();
  }


  // --------------------------------------------

  static int[] Zahlen1 = { 15, 4, 11, 3, 19, 8, 16, 7, 12, 5, 9, 20, 1, 4, 8, 13, 14, 4, 1 };
  static int[] Zahlen2 = { 12, 5, 31, 24, 29, 20, 13, 31 };

  private static void Demo_Zahlen_Filter_MethodSyntax()
  {
   string s = "Geben Sie die Zahlen aus, die kleiner als 10 sind.";
   var Ergebnis = Zahlen1.Where((number => number < 10));
   Print(Ergebnis, s);
  }

  private static void Demo_LTO_Zahlen()
  {
   int i;
   double d;

   string s = "Geben Sie die Zahlen aus, die kleiner als 10 sind.";
   var Ergebnis =
       from n in Zahlen1
       where n < 10
       select n;
   Print(Ergebnis, s);

   s = "Geben Sie die Zahlen aus, die kleiner als 10 sind, aufsteigend sortiert aus.";
   Ergebnis =
       from n in Zahlen1
       where n < 10
       orderby n // optional
       select n;
   Print(Ergebnis, s);

   s = "Geben Sie die Zahlen aus, die kleiner als 10 sind, absteigend sortiert aus.";
   Ergebnis =
       from n in Zahlen1
       where n < 10
       orderby n descending
       select n;
   Print(Ergebnis, s);

   s = "Geben Sie die Zahlen aus, die kleiner als 10 sind, absteigend sortiert aus. Eliminieren Sie alle Duplikate.";
   Ergebnis =
       (from n in Zahlen1
        where n < 10
        orderby n descending
        select n).Distinct();
   Print(Ergebnis, s);

   s = "Geben die vierte bis achte Zahl aus.";
   Ergebnis =
       (from n in Zahlen1
        where n < 10
        select n).Skip(3).Take(4);
   Print(Ergebnis, s);

   s = "Geben Sie erste Zahl aus!";
   i =
      (from n in Zahlen1
       select n).First();
   Print(i, s);

   s = "Geben Sie letzte Zahl aus!";
   i =
      (from n in Zahlen1
       select n).Last();
   Print(i, s);

   s = "Geben Sie 10. Zahl aus!";
   i =
      (from n in Zahlen1
       select n).ElementAt(9);
   Print(i, s);

   s = "Geben Sie 50. Zahl aus! (Fangen Sie den Fehler ab!)";
   i =
      (from n in Zahlen1
       select n).ElementAtOrDefault(49);
   Print(i, s);

   s = "Geben Sie Anzahl der Zahlen aus.";
   i =
      (from n in Zahlen1
       select n).Count();
   Print(i, s);

   s = "Geben Sie nur die niedrigste Zahl aus.";
   i =
      (from n in Zahlen1
       select n).Min();
   Print(i, s);

   s = "Geben Sie nur die höchste Zahl aus.";
   i =
     (from n in Zahlen1
      select n).Max();
   Print(i, s);

   s = "Geben Sie den Durchschnitt aus.";
   d =
     (from n in Zahlen1
      select n).Average();
   Print(d, s);

   s = "Geben Sie die Summe aus.";
   d =
     (from n in Zahlen1
      select n).Sum();
   Print(d, s);

   s = "Geben Sie die das Produkt aller Werte aus.";
   d =
     (from n in Zahlen1
      select n).Aggregate((summe, wert) => summe *= wert);
   Print(d, s);

   s = "Gruppieren Sie die Werte.";
   IEnumerable<IGrouping<int, int>> GruppeErgebnis =
       (from n in Zahlen1
        group n by n);
   Print(GruppeErgebnis, s);

   s = "Geben Sie die Häufigkeit eines jeden Wertes aus!";
   IDictionary<int, int> GruppeHaeufigkeit =
         (from n in Zahlen1
          group n by n into g
          select new { Wert = g.Key, Anzahl = g.Count() }
          ).ToDictionary(y => y.Wert, y => y.Anzahl);
   Print(GruppeHaeufigkeit, s);


   s = "Verbinden Sie die Zahlenmengen 1 und 2:";
   Ergebnis = (from n in Zahlen1 select n).Union(from n2 in Zahlen2 select n2);
   Print(Ergebnis, s);

   s = "Verbinden Sie die Zahlenmengen 1 und 2 und sortieren Sie das Ergebnis:";
   Ergebnis = (from n in Zahlen1 select n).Union(from n2 in Zahlen2 select n2).OrderBy(n => n);
   Print(Ergebnis, s);

   s = "Bilden Sie die Schnittmenge aus den Zahlenmengen 1 und 2.";
   Ergebnis = (from n in Zahlen1 select n).Intersect(from n2 in Zahlen2 select n2).OrderBy(n => n);
   Print(Ergebnis, s);

   s = "Schließen die Zahlen aus Zahlenmengen 2 in Menge 1 aus.";
   Ergebnis = (from n in Zahlen1 select n).Except(from n2 in Zahlen2 select n2).OrderBy(n => n);
   Print(Ergebnis, s);

   s = "Prüfen Sie, ob Zahlenmenge 1 und 2 die gleichen Zahlen in der gleichen Reihenfolge enthalten.";
   bool Erfuellt = (from n in Zahlen1 select n).SequenceEqual(from n2 in Zahlen2 select n2);
   Print(Erfuellt, s);

   s = "Prüfen Sie, ob die Zahl 20 in der Menge vorkommt.";
   Erfuellt =
       (from n in Zahlen1
        orderby n descending
        select n).Contains(14);
   Print(Erfuellt, s);

   s = "Prüfen Sie, ob Zahlen größer als 20 in der Menge vorkommen.";
   Erfuellt =
       (from n in Zahlen1
        orderby n descending
        select n).Any(n => n > 20);
   Print(Erfuellt, s);

   s = "Prüfen Sie, ob alle Zahlen kleiner 20 sind.";
   Erfuellt =
       (from n in Zahlen1
        orderby n descending
        select n).All(n => n < 20);
   Print(Erfuellt, s);

   s = "Filtern Sie alle Integer-Werte heraus!";
   Ergebnis =
   (from n in Zahlen1 select n).OfType<int>();

   Print(Ergebnis, s);

   s = "Wandeln Sie alle Zahlen in Byte-Werte um!";
   var kleineZahlen =
     (from n in Zahlen1 select n).Cast<byte>();
   foreach (var x in kleineZahlen)
   {
    Console.WriteLine(x);
   }
   Print(kleineZahlen, s);
  }

  // --------------------------------------------------------------------

  private static void Print(object x, string s)
  {
   HeadLine(s);
   Console.WriteLine(x);
  }




  private static void Print(IEnumerable<int> Nums, string s)
  {
   HeadLine(s + "(" + Nums.GetType().FullName + ")");
   foreach (int x in Nums)
   {
    Console.WriteLine(x);
   }
  }

  private static void Print(IEnumerable<byte> lowNums, string s)
  {
   HeadLine(s);
   foreach (byte x in lowNums)
   {
    Console.WriteLine(x);
   }
  }

  private static void Print(IDictionary<int, int> gruppe, string s)
  {
   HeadLine(s);
   foreach (var x in gruppe)
   {
    Console.WriteLine(x.Key + ": " + x.Value);
   }
  }

  private static void Print(IEnumerable<IGrouping<int, int>> Gruppen, string s)
  {
   HeadLine(s);
   foreach (IGrouping<int, int> x in Gruppen)
   {
    Console.WriteLine("---- " + x.Key);
    foreach (int i in x)
    {
     Console.WriteLine(i);
    }
   }
  }

  private static void HeadLine(string s)
  {
   Console.ForegroundColor = ConsoleColor.Yellow;
   Console.WriteLine(s);
   Console.ForegroundColor = ConsoleColor.Gray;

   System.IO.StreamWriter sw = new System.IO.StreamWriter("Fragen.txt", true);
   sw.WriteLine(s);
   sw.Close();
  }

  //private static void Print(IEnumerable<int> lowNums, string s)
  //{
  // Console.ForegroundColor = ConsoleColor.Yellow;
  // Console.WriteLine(s);
  // Console.ForegroundColor = ConsoleColor.Gray;
  // foreach (var x in lowNums)
  // {
  //  Console.WriteLine(x);
  // }
  //}



 }
}
