﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using System.Data.Linq.Provider;
using System.Data.Linq;
using System.Diagnostics;

namespace de.WWWings.Test
{
 class LTO_Objektmodell
 {
  public static void run()
  {

   Demo_LTO_Objektmodell();
   //Demo_Flüge_Filter_MethodSyntax();

   Console.ReadLine();
  }


  // LINQ im RAM !!!
  public static void LTO_Projektion()
  {
   BO_Init.Init();


   string s = "Projektion mit Typumwandlung in LINQ-to-Objects";
   IEnumerable<Flug> Ergebnis = from f in Flug.AlleFluege select new Flug { FlugNr = f.FlugNr, AbflugOrt = f.AbflugOrt, ZielOrt = f.ZielOrt };


   Print(Ergebnis.ToList(), s);

  }


  private static void Demo_LTO_Objektmodell()
  {
   // Initialisiere das Objektmodell
   BO_Init.Init();
   string s;
   long i;
   Flug flug;
   double d;

   s = "Geben Sie alle Flüge von Rom aus!";
   var Ergebnis =
    from f in Flug.AlleFluege
    where f.AbflugOrt == "Rom"
    select f;
   Print(Ergebnis, s);

   s = "Geben Sie alle Flüge aus, die weniger als 100 freie Plätze haben.";
   Ergebnis =
      from n in Flug.AlleFluege
      where n.FreiePlaetze < 100
      select n;
   Print(Ergebnis, s);

   s = "Geben Sie alle Flüge aus, die weniger als 100 freie Plätze haben. Sortieren Sie die Liste aufsteigend nach Platzanzahl.";
   Ergebnis =
   from n in Flug.AlleFluege
   where n.FreiePlaetze < 100
   orderby n.FreiePlaetze
   select n;
   Print(Ergebnis, s);

   s = "Geben Sie alle Flüge aus, die weniger als 100 freie Plätze haben. Sortieren Sie die Liste absteigend nach Platzanzahl.";
   Ergebnis =
   from n in Flug.AlleFluege
   where n.FreiePlaetze < 100
   orderby n.FreiePlaetze descending
   select n;
   Print(Ergebnis, s);

   s = "Geben Sie Flug 101 aus.";
   flug = (from f in Flug.AlleFluege
           where f.FlugNr == 101
           select f).SingleOrDefault();
   Print(flug, s);

   s = "Geben Sie die Flüge aus, aber jeder Strecke nur einmal!";
   var Strecken =
        (from n in Flug.AlleFluege
         select new { n.AbflugOrt, n.ZielOrt }).Distinct();
   HeadLine(s);

   foreach (var f in Strecken)
   {
    Console.WriteLine(f.AbflugOrt + " -> " + f.ZielOrt);
   }

   s = "Geben Sie alle Ziele aus, die von Rom erreichbar sind.";
   var Ziele =
        (from n in Flug.AlleFluege
         where n.AbflugOrt == "Rom"
         select n.ZielOrt).Distinct();
   HeadLine(s);

   foreach (string f in Ziele)
   {
    Console.WriteLine(f);
   }

   s = "Geben Sie den vierten bis achten Flug aus in der nach freien Plätzen aufsteigend sortierten Liste aller Flüge, die in Berlin landen.";
   Ergebnis =
       (from n in Flug.AlleFluege
        where n.ZielOrt == "Berlin"
        orderby n.FreiePlaetze
        select n).Skip(3).Take(4);
   Print(Ergebnis, s);

   s = "Geben Sie den ersten Flug aus in der nach freien Plätzen aufsteigend sortierten Liste aller Flüge, die in Berlin landen.";
   flug =
      (from n in Flug.AlleFluege
       where n.ZielOrt == "Berlin"
       orderby n.FreiePlaetze
       select n).First();
   Print(flug, s);

   s = "Geben Sie den letzten Flug aus in der nach freien Plätzen aufsteigend sortierten Liste aller Flüge, die in Berlin landen.";
   flug =
      (from n in Flug.AlleFluege
       where n.ZielOrt == "Berlin"
       orderby n.FreiePlaetze
       select n).Last();
   Print(flug, s);

   s = "Geben Sie den 10. Flug aus in der nach freien Plätzen aufsteigend sortierten Liste aller Flüge, die in Berlin landen.";
   flug =
      (from n in Flug.AlleFluege
       where n.ZielOrt == "Berlin"
       orderby n.FreiePlaetze
       select n).ElementAt(9);
   Print(flug, s);

   s = "Geben Sie den 150. Flug aus in der nach freien Plätzen aufsteigend sortierten Liste aller Flüge, die in Berlin landen.";
   flug =
      (from n in Flug.AlleFluege
       where n.ZielOrt == "Berlin"
       orderby n.FreiePlaetze
       select n).ElementAtOrDefault(149);
   Print(flug, s);

   s = "Geben Sie Anzahl der Flüge aus (mit einem LINQ-Statement!)";
   i =
      (from n in Flug.AlleFluege
       select n).Count();
   Print(i, s);

   s = "Geben Sie geringste freie Platzzahl aus.";
   i =
      (from n in Flug.AlleFluege
       select n).Min(n => n.FreiePlaetze);
   Print(i, s);

   s = "Geben Sie höchste freie Platzzahl aus.";
   i =
      (from n in Flug.AlleFluege
       select n).Min(n => n.FreiePlaetze);
   Print(i, s);

   s = "Geben Sie durchschnittliche freie Platzzahl aus.";
   d =
      (from n in Flug.AlleFluege
       select n).Average(n => n.FreiePlaetze);
   Print(d, s);

   s = "Geben Sie Summe aller freien Plätze aus.";
   i =
      (from n in Flug.AlleFluege
       select n).Sum(n => n.FreiePlaetze);
   Print(i, s);

   s = "Gruppieren Sie die Flüge nach Abflugorten.";
   IEnumerable<IGrouping<string, Flug>> GruppeErgebnis =
       (from n in Flug.AlleFluege
        group n by n.AbflugOrt);
   Print(GruppeErgebnis, s);

   s = "Geben Sie die Häufigkeit eines jeden Abflugortes aus!";
   IDictionary<string, int> GruppeHaeufigkeit =
         (from n in Flug.AlleFluege
          group n by n.AbflugOrt into g
          select new { Name = g.Key, AnzFlug = g.Count() }
          ).ToDictionary(y => y.Name, y => y.AnzFlug);
   Print(GruppeHaeufigkeit, s);

   s = "Erstellen Sie eine gruppierte Liste aller Passagiere mit Ihren Buchungen!";
   var pass2 = from p in Passagier.AllePassagiere
               orderby p.GanzerName
               select new { p.GanzerName, p.Buchungen };
   foreach (var p in pass2)
   {
    Console.WriteLine(p.GanzerName);
    foreach (Buchung b in p.Buchungen)
     Console.WriteLine("\t" + b.Buchungscode);
   }

   s = "Erstellen Sie die Liste der 10 Passagiere mit den meisten Buchungen.";
   var pass = (from p in Passagier.AllePassagiere
               orderby p.Buchungen.Count descending
               select p).Take(10);
   Print(pass, s);

   s = "Erstellen den/die Passagier(e) mit den meisten Buchungen aus.";
   pass = (from n in Passagier.AllePassagiere where n.Buchungen.Count == Passagier.AllePassagiere.Max(x => x.Buchungen.Count) select n);
   Print(pass, s);

   s = "Finden Sie alle Passagiere, die nach Rom fliegen.";
   pass = (from p in Passagier.AllePassagiere
           where p.Buchungen.Any(b => b.Flug.ZielOrt == "Rom")
           select p);
   Print(pass, s);

   s = "Finden Sie alle Passagiere, die genauso viele Buchungen haben wie ein Flug freie Plätze.";
   var joinpass = (from p in Passagier.AllePassagiere
                   join f in Flug.AlleFluege
                   on p.Buchungen.Count equals f.FreiePlaetze
                   select new { p.GanzerName, f.FlugNr, p.Buchungen.Count, f.FreiePlaetze });
   HeadLine(s);
   foreach (var j in joinpass)
   {
    Console.WriteLine(j.GanzerName + " und Flug " + j.FlugNr + " haben die gleiche Zahl: " + j.Count + " / " + j.FreiePlaetze);
   }

   s = "Geben Sie alle Passagiere aus und optional dazu einen Flug, der genausoviele freie Plätze hat wie der Passagier Buchungen hat.";
   var joinpass2 = (from p in Passagier.AllePassagiere
                    join f in Flug.AlleFluege
                    on p.Buchungen.Count equals f.FreiePlaetze
                    into Fluege
                    select new { p.GanzerName, Fluege });
   HeadLine(s);
   foreach (var j in joinpass2)
   {
    Console.WriteLine(j.GanzerName + " hat " + j.Fluege.Count() + " korrespondierende Flüge!");
   }

   s = "Geben Sie alle Passagiere, die älter als 50 Jahre sind!";
   pass = (from p in Passagier.AllePassagiere
           where p.Geburtsdatum.AddYears(50) < DateTime.Now
           select p);
   Print(pass, s);

   s = "Geben Sie alle Flüge aus, mit Passagieren älter als 50 Jahre !";
   Ergebnis = (from p in Passagier.AllePassagiere
               where p.Geburtsdatum.AddYears(50) < DateTime.Now
               from b in p.Buchungen
               select b.Flug).Distinct();
   Print(Ergebnis, s);

   // Kontrolle:
   //foreach (Flug f in Ergebnis)
   //{
   // foreach (Passagier p in f.Passagiere)
   // {
   //  Console.WriteLine(p.Geburtsdatum);
   // }
   //}
 }



  // --------------------------------------------------------------------

  private static void Print(IDictionary<string, int> gruppe, string s)
  {
   HeadLine(s);
   foreach (var x in gruppe)
   {
    Console.WriteLine(x.Key + ": " + x.Value);
   }
  }


  private static void Print(IEnumerable<Flug> Menge, string s)
  {
   HeadLine(s);
   foreach (Flug x in Menge)
   {
    PrintFlug(x);
   }
  }

  private static void Print(IEnumerable<Passagier> Menge, string s)
  {
   HeadLine(s);
   foreach (Passagier x in Menge)
   {
    PrintPassagier(x);
   }
  }

  private static void Print(IEnumerable<string> Menge, string s)
  {
   HeadLine(s);
   foreach (string x in Menge)
   {
    Console.WriteLine(x);
   }
  }


  private static void Print(IEnumerable<IGrouping<string, Flug>> Gruppen, string s)
  {
   HeadLine(s);
   foreach (IGrouping<string, Flug> x in Gruppen)
   {
    Console.WriteLine("---- " + x.Key);
    foreach (Flug y in x)
    {
     PrintFlug(y);
    }
   }
  }

  private static void Print(Flug x, string s)
  {
   HeadLine(s);
   if (x == null) { Console.WriteLine("---"); return; }
   PrintFlug(x);

  }

  private static void PrintFlug(Flug x)
  {
   Console.WriteLine(x.FlugNr + ": " + x.AbflugOrt + " -> " + x.ZielOrt + ": " + x.FreiePlaetze + " freie Plätze");
  }

  private static void PrintPassagier(Passagier x)
  {
   Console.WriteLine(x.GanzerName + ": " + x.Geburtsdatum + " -> " + x.Buchungen.Count);
  }


  private static void Print(object x, string s)
  {
   HeadLine(s);

   Console.WriteLine(x);

  }


  private static void HeadLine(string s)
  {
   Console.ForegroundColor = ConsoleColor.Yellow;
   Console.WriteLine(s);
   Console.ForegroundColor = ConsoleColor.Gray;

   System.IO.StreamWriter sw = new System.IO.StreamWriter("Fragen.txt", true);
   sw.WriteLine(s);
   sw.Close();

  }

  //private static void Print(IEnumerable<int> lowNums, string s)
  //{
  // Console.ForegroundColor = ConsoleColor.Yellow;
  // Console.WriteLine(s);
  // Console.ForegroundColor = ConsoleColor.Gray;
  // foreach (var x in lowNums)
  // {
  //  Console.WriteLine(x);
  // }
  //}





 }
}
