﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using System.Data.Linq.Provider;
using System.Data.Linq;
using System.Data.SqlClient;
using System.Configuration;
using System.Linq.Dynamic;
using de.WWWings.Test.LINQExtensions;
using de.WWWings.Test.Modelle.LTS4;
namespace de.WWWings.Test
{
 class LTS_Misc
 {
  public static void run()
  {
   //A1_Update();
   //A2_UpdateConflict();
   //A3_Update_Detach();
   //A4_Navigation_Lazy();
   //A5_Navigation_Eager();
   //A6_Add();
   //A7_AddCollection();
   //A8_AddCollection2();
   //A8_CreateDatabase();
   A3B_Update_Detach_Collection();
  }

  public static void A1_Update()
  {
   LTS.HeadLine("Reduzieren Sie die Platzanzahl für Flug 101 um zwei Plätze. Geben Sie die Anzahl der modifizierten Objekte aus. Überprüfen Sie, ob der Wert modifiziert wurde.");

   DatenbankDataContext db = new DatenbankDataContext();

  
   var menge = from fl in db.FL_Flueges
               where fl.FL_FlugNr == 101
               select fl;

   Console.WriteLine(menge);
   Console.WriteLine("Anzahl Flüge: " + menge.Count<FL_Fluege>());

   FL_Fluege f = menge.Single<FL_Fluege>();
   Console.WriteLine("Before: " + f.FL_FlugNr + ": " + f.FL_Abflugort + " -> " + f.FL_Zielort + " Freie Plätze: " + f.FL_FreiePlaetze);

   f.FL_FreiePlaetze -= 2;

   Console.WriteLine("Modified: " + db.GetChangeSet().Updates.Count);
   db.SubmitChanges();

   var f_after = (from fl in db.FL_Flueges
                  where fl.FL_FlugNr == 101
                  select fl).Single<FL_Fluege>();

   Console.WriteLine("After: " + f_after.FL_FlugNr + ": " + f_after.FL_Abflugort + " -> " + f_after.FL_Zielort + " Freie Plätze: " + f.FL_FreiePlaetze);

  }


  /// <summary>
  /// Änderungskonflikt simulieren und Fehler abfangen
  /// </summary>
  public static void A2_UpdateConflict()
  {
   LTS.HeadLine("Erzeugen Sie einen Änderungskonflikt. (Hinweis: Verwenden Sie zwei verschiedene DataContext-Instanzen). Fangen Sie den Konflikt ab und geben Sie die Ursache des Konfliktes aus.");

   long FL_FlugNr = 103;

   // 1. Datenkontext
   DatenbankDataContext db1 = new DatenbankDataContext();
   FL_Fluege f1 = (from fl in db1.FL_Flueges
                   where fl.FL_FlugNr == FL_FlugNr
                   select fl).Single<FL_Fluege>();

   Console.WriteLine("Kontext 1 hat geladen: " + f1.FL_FreiePlaetze + " freie Plätze!");

   // 2. Datenkontext
   DatenbankDataContext db2 = new DatenbankDataContext();
   FL_Fluege f2 = (from fl in db2.FL_Flueges
                   where fl.FL_FlugNr == FL_FlugNr
                   select fl).Single<FL_Fluege>();
   Console.WriteLine("Kontext 2 hat geladen: " + f2.FL_FreiePlaetze + " freie Plätze!");

   // Änderung im zweiten Datenkontext und Speichern

   f2.FL_FreiePlaetze--;
   Console.WriteLine("Kontext 2 will speichern: " + f2.FL_FreiePlaetze + " freie Plätze!");
   db2.SubmitChanges(ConflictMode.ContinueOnConflict);
   Console.WriteLine("Kontext 2 hat gespeichert: " + f2.FL_FreiePlaetze + " freie Plätze!");

   bool SaveAgain = false;

   // Änderung im ersten Datenkontext und Speichern
   try
   {
    f1.FL_FreiePlaetze -= 3;
    Console.WriteLine("Kontext 1 will speichern: " + f1.FL_FreiePlaetze + " freie Plätze!");
    db1.SubmitChanges(ConflictMode.ContinueOnConflict);
    Console.WriteLine("Kontext 1 hat gespeichert: " + f1.FL_FreiePlaetze + " freie Plätze!");
   }
   catch (ChangeConflictException ex)
   {
    Console.ForegroundColor = ConsoleColor.Red;
    Console.WriteLine("FEHLER: " + ex.Message);

    foreach (ObjectChangeConflict x in db1.ChangeConflicts)
    {
     Console.WriteLine("Konflikt bei Objekt: " + (x.Object as FL_Fluege).FL_FlugNr);

     foreach (MemberChangeConflict m in x.MemberConflicts)
     {
      Console.WriteLine("\t" + m.Member.Name + ": Original: " + m.OriginalValue + " Current: " + m.CurrentValue + " Database: " + m.DatabaseValue);
     }

     Console.WriteLine("Wie soll der Konflikt gelöst werden? ");
     Console.WriteLine("h = Speichern des Zustand des Hauptspeichers. Überschreiben der Datenbank.");
     Console.WriteLine("d = Beibehalten des Zustands der Datenbank. Verwerfen der Änderungen im Hauptspeicher.");
     Console.WriteLine("m = Mischen der Änderungen.");
     ConsoleKeyInfo key = Console.ReadKey();
     Console.WriteLine();
     switch (key.Key)
     {
      case ConsoleKey.H: x.Resolve(RefreshMode.KeepCurrentValues); SaveAgain = true; break;
      case ConsoleKey.D: x.Resolve(RefreshMode.OverwriteCurrentValues); SaveAgain = true; break;
      case ConsoleKey.M: x.Resolve(RefreshMode.KeepChanges); SaveAgain = true; break;
     }
    }
   }
   Console.ForegroundColor = ConsoleColor.White;

   // Notwendig, da Resolve() nur Hauptspeicher anpasst!
   if (SaveAgain) db1.SubmitChanges();

   Console.WriteLine("Endzustand in Kontext 1: " + f1.FL_FreiePlaetze + " freie Plätze!");
   Console.WriteLine("Endzustand in Kontext 2: " + f2.FL_FreiePlaetze + " freie Plätze!");
   DatenbankDataContext db3 = new DatenbankDataContext();
   FL_Fluege f3 = (from fl in db3.FL_Flueges
                   where fl.FL_FlugNr == FL_FlugNr
                   select fl).Single<FL_Fluege>();
   Console.WriteLine("Endzustand in der Datenbank: " + f3.FL_FreiePlaetze + " freie Plätze!");

   // ENDE
   db1.Dispose();
   db2.Dispose();
   db3.Dispose();
   Console.ForegroundColor = ConsoleColor.Gray;
  }


  public static void A3_Update_Detach()
  {

   LTS.HeadLine("Laden Sie einen Flug. Schließen Sie den Datacontext. Ändern Sie das Objekt. Speichern Sie das Objekt durch einen neuen DataContext in die Datenbank.");


   long FL_FlugNr = 103;

   DatenbankDataContext db1 = new DatenbankDataContext();
   var menge = from fl in db1.FL_Flueges
               where fl.FL_FlugNr == FL_FlugNr
               select fl;

   Console.WriteLine(menge);
   Console.WriteLine("Anzahl Flüge: " + menge.Count<FL_Fluege>());

   FL_Fluege f = menge.Single<FL_Fluege>();
   Console.WriteLine("Before: " + f.FL_FlugNr + ": " + f.FL_Abflugort + " -> " + f.FL_Zielort + " Freie Plätze: " + f.FL_FreiePlaetze);

   // Context vernichten
   db1.Dispose();

   // Flug f_neu = f; // falsch!!!! Nur Referenzkopie!

   FL_Fluege f_neu = new FL_Fluege() { FL_FlugNr = f.FL_FlugNr, FL_FreiePlaetze = f.FL_FreiePlaetze, FL_Abflugort = f.FL_Abflugort, FL_Zielort = f.FL_Zielort, Timestamp = f.Timestamp }; // unzureichend: Alle Properties!

   //FL_Fluege f_neu = f.Clone(); // GEHT NICHT MEHR IN RTM!!!

   f_neu.FL_FreiePlaetze -= 1; ;
   Console.WriteLine("Object modfied!");

   Console.WriteLine("After: " + f_neu.FL_FlugNr + ": " + f_neu.FL_Abflugort + " -> " + f_neu.FL_Zielort + " Freie Plätze: " + f_neu.FL_FreiePlaetze);

   Console.WriteLine("New Context!");

   DatenbankDataContext db2 = new DatenbankDataContext();
   Console.WriteLine(db2.DeferredLoadingEnabled);
   //db2.ObjectTrackingEnabled = false;

   // db2.Flugs.Attach(f_neu, f); // ohne Timestamp!
   db2.FL_Flueges.Attach(f_neu, true); // geht nur mit Timestamp!

   Console.WriteLine("Modified: " + db2.GetChangeSet().Updates.Count);

   db2.SubmitChanges();

   // Context vernichten
   db2.Dispose();

   Console.WriteLine("Saved!");

   DatenbankDataContext db3 = new DatenbankDataContext();
   var f_after = (from fl in db3.FL_Flueges
                  where fl.FL_FlugNr == FL_FlugNr
                  select fl).Single<FL_Fluege>();

   Console.WriteLine("Check: " + f_after.FL_FlugNr + ": " + f_after.FL_Abflugort + " -> " + f_after.FL_Zielort + " Freie Plätze: " + f_after.FL_FreiePlaetze);

  }

  /// <summary>
  /// Multi-Tier-Beispiel / Detached Objets
  /// </summary>
  public static void A3B_Update_Detach_Collection()
  {

   LTS.HeadLine("Laden Sie 10 Flüge Schließen Sie den Datacontext. Ändern Sie fünf Objekte. Speichern Sie das Objekte durch einen neuen DataContext in die Datenbank.");

   long Hoechste_FlugNummer = 109;

   // ---- Server
   DatenbankDataContext db1 = new DatenbankDataContext();
   db1.DeferredLoadingEnabled = false;
   var menge = from fl in db1.FL_Flueges
               where fl.FL_FlugNr <= Hoechste_FlugNummer
               select fl;

   Console.WriteLine(menge);
   Console.WriteLine("Anzahl Flüge: " + menge.Count<FL_Fluege>());

   IEnumerable<FL_Fluege> fluege = menge.ToList();

   Console.WriteLine("---- Ausgangszustand: ");
   foreach (FL_Fluege f in fluege)
   {

    Console.WriteLine("SERVER VORHER: " + f.FL_FlugNr + ": " + f.FL_Abflugort + " -> " + f.FL_Zielort + " Freie Plätze: " + f.FL_FreiePlaetze + " (" + f.GetHashCode() + ")");

    foreach (GF_GebuchteFluege g in f.GF_GebuchteFlueges)
    {
     Console.WriteLine("- " + g.GF_Buchungscode + "=" + g.GF_Preis);
    }
   }

   // ---- Daten serialisieren und Context vernichten
   Console.WriteLine("Simuliere Offline-Szenario");
   List<FL_Fluege> Fluege_Serialisiert = fluege.ToList();
   Console.WriteLine("Vernichte Kontext...");
   db1.Dispose();

   // ---- Client
   Console.WriteLine("Start des Clients");
   Console.WriteLine("Offline Veränderung der ersten fünf (!) Datensätze...");
   for (int i = 0; i < 5; i++)
   {
    Fluege_Serialisiert[i].FL_FreiePlaetze--;

    foreach (GF_GebuchteFluege g in Fluege_Serialisiert[i].GF_GebuchteFlueges)
    {
     g.GF_Preis = 100.00m;
     Console.WriteLine("Neuer Preis: " + g.GF_Buchungscode + "=" + g.GF_Preis);
    }
   }
   Console.WriteLine("Einige Objekte verändert");

   Console.WriteLine("---- neuer Zustand: ");
   foreach (FL_Fluege f in Fluege_Serialisiert)
   {
    Console.WriteLine("CLIENT: " + f.FL_FlugNr + ": " + f.FL_Abflugort + " -> " + f.FL_Zielort + " Freie Plätze: " + f.FL_FreiePlaetze + " (" + f.GetHashCode() + ")");
   }
   Console.WriteLine("Ende des Clients");

   // ---- Server
   Console.WriteLine("--- Zurück auf dem Server");

   Console.WriteLine("Neuer Kontext!");

   DatenbankDataContext db2 = new DatenbankDataContext();

   // Alternative 1: // geht nur mit Timestamp!
   db2.FL_Flueges.AttachAll(Fluege_Serialisiert, true);

   // Alternative 2: // geht nur mit Timestamp!
   //foreach (FL_Fluege f in Menge_Neu)
   //{
   // db2.FL_Flueges.Attach(f, true); 
   //}

   // Alternative 3: // ohne Timestamp!
   // db2.Flugs.Attach(f_neu, f); 

   Console.WriteLine("Anzahl der zurückgesendeten Datensätze: " + db2.GetChangeSet().Updates.Count);

   db2.SubmitChanges();

   // Context vernichten
   db2.Dispose();

   Console.WriteLine("Gespeichert!");

   DatenbankDataContext db3 = new DatenbankDataContext();
   var f_after = (from fl in db3.FL_Flueges
                  where fl.FL_FlugNr <= Hoechste_FlugNummer
                  select fl);

   foreach (FL_Fluege f in f_after)
   {

    Console.WriteLine("SERVER NACHHER: " + f.FL_FlugNr + ": " + f.FL_Abflugort + " -> " + f.FL_Zielort + " Freie Plätze: " + f.FL_FreiePlaetze + " (" + f.GetHashCode() + ")");

    foreach (GF_GebuchteFluege g in f.GF_GebuchteFlueges)
    {
     Console.WriteLine("- " + g.GF_Buchungscode + "=" + g.GF_Preis);
    }
   }

  }

  /// <summary>
  /// Navigation zwischen Objekten mit Lazy Loading
  /// </summary>
  public static void A4b_Lazy()
  {
   LTS.HeadLine("Nachladen der Spalte Ankunftszeit");

   DatenbankDataContext db = new DatenbankDataContext();
   db.Log = Console.Out;

   var q = from f in db.FL_Flueges
           where f.GF_GebuchteFlueges.Count > 0
           select f;

   foreach (FL_Fluege f in q)
   {
    Console.WriteLine(f.FL_FlugNr);
    Console.WriteLine(f.Fl_StartZeit);
   }
  }

  private static void A4_Navigation_Lazy()
  {
   LTS.HeadLine("Geben Sie alle Flüge mit Ihren Buchungen und jeweils allen Passagierdaten aus. Wieviele SQL-Befehle werden dafür abgesendet?");

   DatenbankDataContext db = new DatenbankDataContext();

   DataLoadOptions lo = new DataLoadOptions();
   lo.LoadWith<FL_Fluege>(f => f.GF_GebuchteFlueges);
   db.LoadOptions = lo;

   var q = from f in db.FL_Flueges
           where f.GF_GebuchteFlueges.Count > 0
           select f;



   Console.WriteLine(q);
   //Console.WriteLine("Anzahl Flüge: " + q.Count<);

   foreach (var f in q)
   {
    Console.WriteLine(f.FL_FlugNr + ": " + f.FL_Abflugort + " -> " + f.FL_Zielort);
    foreach (var b in f.GF_GebuchteFlueges)
    {
     Console.WriteLine(" - " + b.GF_Buchungscode + ": " + b.PS_Passagier.PE_Person.PE_Name);
    }
   }
  }

  /// <summary>
  /// Navigation zwischen Objekten mit Eager Loading
  /// </summary>
  private static void A5_Navigation_Eager()
  {
   LTS.HeadLine("Geben Sie alle Flüge mit Ihren Buchungen und jeweils allen Passagierdaten aus. Laden Sie dabei die Flüge zusammen mit allen Passagierdaten (Verhindern Sie Lazy Loading! Wieviele SQL-Befehle werden dafür abgesendet?");

   DatenbankDataContext db = new DatenbankDataContext();
   // db.DeferredLoadingEnabled = false;
   db.Log = Console.Out;
   DataLoadOptions lo = new DataLoadOptions();
   lo.LoadWith<FL_Fluege>(x => x.GF_GebuchteFlueges);
   lo.LoadWith<GF_GebuchteFluege>(x => x.PS_Passagier);
   lo.LoadWith<PS_Passagier>(x => x.PE_Person);
   db.LoadOptions = lo;

   var q = from f in db.FL_Flueges
           where f.GF_GebuchteFlueges.Count > 0
           select f;

   //Console.WriteLine(q);
   //Console.WriteLine("Anzahl Flüge: " + q.Count<);

   foreach (var f in q)
   {
    print(f.FL_FlugNr + ": " + f.FL_Abflugort + " -> " + f.FL_Zielort);
    foreach (var b in f.GF_GebuchteFlueges)
    {
     print(" - " + b.GF_Buchungscode + ": " + b.PS_Passagier.PE_Person.PE_Name);
    }
   }
  }
  private static void print(string s)
  {
   Console.ForegroundColor = ConsoleColor.Yellow;
   Console.WriteLine(s);
   Console.ForegroundColor = ConsoleColor.Gray;
  }


  /// <summary>
  /// Anlegen eines einzelnen Flugs
  /// </summary>
  private static void A6_Add()
  {
   LTS.HeadLine("Fügen Sie einen neuen Flug ein. Löschen Sie den Flug vorher, falls er schon existiert.");

   int Nr = 997;

   DatenbankDataContext db = new DatenbankDataContext();
   db.Log = Console.Out;
   //db.ObjectTrackingEnabled = false;
   var x = from f in db.FL_Flueges where f.FL_FlugNr == Nr select f;

   FL_Fluege fl = x.FirstOrDefault<FL_Fluege>();
   if (fl != null)
   {
    db.FL_Flueges.DeleteOnSubmit(fl);
    db.SubmitChanges();
   }
   //db.SubmitChanges();
   Console.WriteLine(db.FL_Flueges.Count<FL_Fluege>());

   FL_Fluege f2 = new FL_Fluege();
   f2.FL_FlugNr = Nr;
   f2.FL_Abflugort = "x";
   f2.FL_Zielort = "y";
   db.FL_Flueges.InsertOnSubmit(f2);
   db.SubmitChanges();

   Console.WriteLine(db.FL_Flueges.Count<FL_Fluege>());
  }


  /// <summary>
  /// Beispiel für direkte SQL-Ausführung
  /// </summary>
  public static void CleanUp()
  {
   string CONNSTRING = ConfigurationManager.ConnectionStrings["de.WWWings.Test.Properties.Settings.WWWIngsConnectionString"].ConnectionString;
   SqlConnection conn = new SqlConnection(CONNSTRING);
   conn.Open();
   SqlCommand cmd1 = new SqlCommand("Delete  from GF_GebuchteFluege where GF_FL_FlugNr > 1000", conn);

   cmd1.ExecuteNonQuery();

   SqlCommand cmd2 = new SqlCommand("Delete  from FL_Fluege where FL_FlugNr > 1000", conn);

   cmd2.ExecuteNonQuery();

  }



  /// <summary>
  /// Menge von Flügen erzeugen
  /// </summary>
  public static void A7_AddCollection_Fluege()
  {
   LTS.HeadLine("Fügen Sie 10000 neue Flüge ein (aus einer Menge).");

   string[] Flughaefen = { "Berlin", "Frankfurt", "München", "Hamburg", "Köln/Bonn", "Rom", "London", "Paris", "Mailand", "Prag", "Moskau", "New York", "Seattle", "Essen/Mülheim", "Kapstadt", "Madrid", "Oslo", "Dallas" };
   System.Random rnd = new Random();

   int Start = 1000;
   int Anz = 10000;

   DatenbankDataContext db = new DatenbankDataContext();

   Console.WriteLine(db.FL_Flueges.Count<FL_Fluege>());
   // Alle Flüge über Nummer 1000 löschen!
   var y = from b in db.GF_GebuchteFlueges where b.GF_FL_FlugNr >= 1000 select b;
   foreach (GF_GebuchteFluege b in y)
   {
    db.GF_GebuchteFlueges.DeleteOnSubmit(b);
   }

   var x = from f in db.FL_Flueges where f.FL_FlugNr >= 1000 select f;

   foreach (FL_Fluege f in x)
   {
    db.FL_Flueges.DeleteOnSubmit(f);
    db.SubmitChanges(ConflictMode.ContinueOnConflict);
   }


   Console.WriteLine(db.FL_Flueges.Count<FL_Fluege>());

   //db.Log = Console.Out;

   List<FL_Fluege> liste = new List<FL_Fluege>();

   for (int i = Start; i < (Start + Anz); i++)
   {

    if (i % 1000 == 0) Console.WriteLine(i);
    FL_Fluege f2 = new FL_Fluege();
    f2.FL_FlugNr = i;
    f2.FL_Abflugort = Flughaefen[rnd.Next(0, Flughaefen.Length - 1)];
    f2.FL_Zielort = Flughaefen[rnd.Next(0, Flughaefen.Length - 1)]; ;
    f2.FL_FreiePlaetze = Convert.ToInt16(new System.Random(i).Next(250));
    f2.FL_Plaetze = 250;
    f2.FL_Datum = DateTime.Now.AddDays((double)f2.FL_FreiePlaetze).AddMinutes((double)f2.FL_FreiePlaetze * 7);
    f2.FL_NichtRaucherFlug = true;
    liste.Add(f2);


   }

   db.FL_Flueges.InsertAllOnSubmit(liste);
   db.SubmitChanges();

   Console.WriteLine(db.FL_Flueges.Count<FL_Fluege>());
  }


  /// <summary>
  /// Menge von Flügen mit äbhängigen Buchungen ergänzen
  /// </summary>
  public static void A8_Add_Fluege_Buchungen()
  {
   LTS.HeadLine("Fügen Sie 1000 neue Flüge und für jeden drei neue Buchungen ein.");

   int Start = 1000;
   int Anz = 2000;

   // Datenkontext
   DatenbankDataContext db = new DatenbankDataContext();
   db.Log = Console.Out;

   Console.WriteLine("Flüge: " + db.FL_Flueges.Count<FL_Fluege>());
   Console.WriteLine("Buchungen: " + db.GF_GebuchteFlueges.Count<GF_GebuchteFluege>());

   // -----------------------------------------------------
   // Aufräumen von früheren Durchgängen: Alle Flüge über Nummer 1000 löschen!
   var y = from b in db.GF_GebuchteFlueges where b.GF_FL_FlugNr >= 1000 select b;
   foreach (GF_GebuchteFluege b in y)
   {
    db.GF_GebuchteFlueges.DeleteOnSubmit(b);
   }
   var x = from f in db.FL_Flueges where f.FL_FlugNr >= 1000 select f;
   foreach (FL_Fluege f in x)
   {
    db.FL_Flueges.DeleteOnSubmit(f);
   }
   db.SubmitChanges();
   // Achtung: In der Praxis besser:
   db.ExecuteCommand("Delete from GF_GebuchteFlueges where GF_FL_FlugNr >= 1000");
   db.ExecuteCommand("Delete from FL_Flueges where FL_FlugNr >= 1000");
   // -----------------------------------------------------

   Console.WriteLine("Flüge: " + db.FL_Flueges.Count<FL_Fluege>());
   Console.WriteLine("Buchungen: " + db.GF_GebuchteFlueges.Count<GF_GebuchteFluege>());

   // Suche drei Passagiere
   PS_Passagier p1 = (from px in db.PS_Passagiers where px.PE_Person.PE_Name == "Schulze" select px).FirstOrDefault();
   PS_Passagier p3 = (from px in db.PS_Passagiers where px.PE_Person.PE_Name == "Meier" select px).FirstOrDefault();
   PS_Passagier p2 = (from px in db.PS_Passagiers where px.PE_Person.PE_Name == "Müller" select px).FirstOrDefault();


   // Neue Liste von Flügen erzeugen
   for (int i = Start; i < (Start + Anz); i++)
   {
    if (i % 1000 == 0) Console.WriteLine(i);

    // Flug anlegen
    FL_Fluege f2 = new FL_Fluege();
    f2.FL_FlugNr = i;
    f2.FL_Abflugort = "x";
    f2.FL_Zielort = "y";
    db.FL_Flueges.InsertOnSubmit(f2);

    // Buchung #1 anlegen
    GF_GebuchteFluege b1 = new GF_GebuchteFluege();
    //b.GF_FL_FlugNr = i;
    //b.GF_PS_ID = 1;
    b1.FL_Fluege = f2;
    b1.PS_Passagier = p1;
    db.GF_GebuchteFlueges.InsertOnSubmit(b1);
    //f2.GF_GebuchteFlueges.Add(b);

    // Buchung #2 anlegen
    GF_GebuchteFluege b2 = new GF_GebuchteFluege();
    //b.GF_FL_FlugNr = i;
    //b.GF_PS_ID = 30;
    b2.FL_Fluege = f2;
    b2.PS_Passagier = p2;
    f2.GF_GebuchteFlueges.Add(b2);

    // Buchung #3 anlegen
    GF_GebuchteFluege b3 = new GF_GebuchteFluege();
    b3.GF_FL_FlugNr = i;
    b3.GF_PS_ID = 90;
    f2.GF_GebuchteFlueges.Add(b3);
    db.SubmitChanges();
    db.Refresh(RefreshMode.KeepCurrentValues);

    //Console.WriteLine("Mem Flüge: " + db.FL_Flueges.Count());
    //Console.WriteLine("Mem Buchungen: " + db.GF_GebuchteFlueges.Count());
   }

   Console.WriteLine("Flüge: " + db.FL_Flueges.Count<FL_Fluege>());
   Console.WriteLine("Buchungen: " + db.GF_GebuchteFlueges.Count<GF_GebuchteFluege>());
   Console.WriteLine(db.FL_Flueges.Count<FL_Fluege>());
  }

  /// <summary>
  /// Forward Engineering Demo
  /// Das Beispiel zeigt das Anlegen einer Datenbank auf Basis eines Datenkontextes. Wenn die Datenbank bereits besteht, wird sie gelöscht und dann neu angelegt. Die Datenbank kann dann direkt befüllt und abgefragt werden.
  /// </summary>
  public static void A9_CreateDatabase()
  {
   LTS.HeadLine("Erzeugen Sie eine neue Datenbank auf Basis der DBML-Datei und befüllen Sie die Daten über LINQ-to-SQL mit Flügen.");

   // Verbindung zu einer Datenbank, die es noch gar nicht gibt
   DatenbankDataContext db = new DatenbankDataContext(@"Data Source=.\sqlexpress;Database=WWWIngs_Neu;Trusted_Connection=true");

   // TODO: Compact:   //DatenbankDataContext db = new DatenbankDataContext(@"www.sdf");

   // Datenbank löschen, falls sie schon existiert.
   if (db.DatabaseExists())
   {
    Console.WriteLine("Datenbank ist bereits vorhanden!");
    db.DeleteDatabase();
    Console.WriteLine("Datenbank gelöscht!");
   }

   // Datenbank anlegen
   db.CreateDatabase();
   Console.WriteLine("Datenbank erzeugt!");

   // Daten erzeugen
   Console.WriteLine("Erzeuge Daten...");
   for (int i = 100; i < 200; i++)
   {
    FL_Fluege f2 = new FL_Fluege();
    f2.FL_FlugNr = i;
    f2.FL_Abflugort = "Rom";
    f2.FL_Zielort = "Paris";
    db.FL_Flueges.InsertOnSubmit(f2);
    db.SubmitChanges();
    Console.WriteLine("Flug erzeugt: " + i);
   }

   // Daten abfragen
   var Ergebnis =
    from f in db.FL_Flueges
    where f.FL_Abflugort == "Rom"
    select f;

   // Daten ausgeben
   LTS.Print(Ergebnis, "");

   // Ende
   db.Dispose();
  }


  /// <summary>
  /// Zusatz-Namenseraum System.Linq.Dynamic
  /// </summary>
  public static void DynQuery_Run()
  {

   DatenbankDataContext db = new DatenbankDataContext();
   db.Log = Console.Out;

   var menge1 = from fl in db.FL_Flueges
                where fl.FL_FlugNr < 110 && fl.FL_Datum != null
                select fl;

   object[] Parameter = new object[] { 250, 300 };
   var menge2 = menge1.Where("FL_Plaetze = @0 and FL_FreiePlaetze <= @1", Parameter);

   int w1 = 250;
   int w2 = 300;
   var menge3 = menge1.Where(f => f.FL_Plaetze == w1);
   var menge4 = menge3.Where(f => f.FL_FreiePlaetze < w2);


   var menge5 = menge1.
    Select("new (Fl_Flugnr, FL_FreiePlaetze, FL_AbflugOrt, FL_Zielort, TimeStamp");
   IEnumerable<FL_Fluege> menge6 = menge5.Convert<FL_Fluege>(db.FL_Flueges);

   //Console.WriteLine(menge5.Count());
   Console.WriteLine(menge6.Count());

   foreach (var f in menge6)
   {
    Console.WriteLine(f.FL_FlugNr + ": " + f.FL_FreiePlaetze + " von " + f.FL_Abflugort + " nach " + f.FL_Zielort);
    f.FL_FreiePlaetze--;
   }
   db.Log = Console.Out;
   db.SubmitChanges();

  }





  public static void Konverter()
  {
   DatenbankDataContext db = new DatenbankDataContext();
   db.Log = Console.Out;

   Console.WriteLine("var");
   var Fluege = from fl in db.FL_Flueges
                where fl.FL_FlugNr < 110
                select fl;
   Console.WriteLine(Fluege.ToString());
   Console.WriteLine(Fluege.Count());
   Console.WriteLine(Fluege.ToString());
   Console.WriteLine("IEnum direkt");
   Console.WriteLine(Fluege.ToString());
   IEnumerable<FL_Fluege> Fluege2 = from fl in db.FL_Flueges
                                    where fl.FL_FlugNr < 110
                                    select fl;
   Console.WriteLine(Fluege2.ToString());
   Console.WriteLine(Fluege.Count());
   Console.WriteLine("IEnum von Methode");
   IEnumerable<FL_Fluege> Fluege3 = Konverter2() as IEnumerable<FL_Fluege>;
   Console.WriteLine(Fluege3.ToString());



  }

  public static IEnumerable<FL_Fluege> Konverter2()
  {
   DatenbankDataContext db = new DatenbankDataContext();

   var Fluege2 = from fl in db.FL_Flueges
                 where fl.FL_FlugNr < 110
                 select fl;
   Console.WriteLine(Fluege2.ToString());
   return Fluege2;
  }






  public static void SQLDirect()
  {
   // Verbindungsaufbau
   DatenbankDataContext db = new DatenbankDataContext();

   // Protokollierung ein
   db.Log = Console.Out;

   //// Untypisiertes Ergebnis
   //System.Collections.IEnumerable Fluege1 = db.ExecuteQuery(typeof(FL_Fluege),"Select * from FL_Fluege where FL_Abflugort = 'Rom'");

   // Typisiertes Ergebnis
   IEnumerable<FL_Fluege> Fluege2 = db.ExecuteQuery<FL_Fluege>("Select FL_FlugNr, FL_ZielOrt from FL_Fluege where FL_Abflugort = 'Rom'");

   // Typisiertes Ergebnis
   IEnumerable<FL_Fluege> Fluege3 = db.ExecuteQuery<FL_Fluege>("Select * from FL_Fluege where FL_FlugNr = 102");

   // Ausgabe
   foreach (FL_Fluege Flug in Fluege3)
   {
    Console.WriteLine(Flug.FL_FlugNr + ": " + Flug.FL_Abflugort + " -> " + Flug.FL_Zielort + ": " + Flug.FL_FreiePlaetze + " freie Plätze");
    Flug.FL_AnzahlStarts++;
    Flug.FL_Abflugort += " Fiumicino";
   }


   db.SubmitChanges();

   // Ende
   db.Dispose();
  }


  public static void Demo_BOExtensions()
  {
   LTS.HeadLine("Rufen Sie zusätzliche Attribute oder Methoden auf.");

   DatenbankDataContext db = new DatenbankDataContext();

   var menge1 = from fl in db.FL_Flueges
                where fl.FL_FlugNr < 110 && fl.FL_Datum != null
                select new { FlugNr = fl.FL_FlugNr, Auslastung = fl.Auslastung };

   foreach (var f in menge1)
   {
    Console.WriteLine(f.FlugNr + ": " + f.Auslastung);
   }

   // GEHT NICHT:
   //var menge2 = from fl in db.FL_Flueges
   //             where fl.IstAusgebucht == false
   //             select new { FlugNr = fl.FL_FlugNr, Auslastung = fl.Auslastung };


   var menge2 = from fl in db.FL_Flueges.AsEnumerable()
                where fl.IstAusgebucht == false
                select new { FlugNr = fl.FL_FlugNr, Auslastung = fl.Auslastung };

   foreach (var f in menge2)
   {
    Console.WriteLine(f.FlugNr + ": " + f.Auslastung);

   }

  }


  public static void Demo_Identity()
  {
   LTS.HeadLine("Object Identity");

   DatenbankDataContext db = new DatenbankDataContext();

   var menge = from fl in db.FL_Flueges
               where fl.FL_FlugNr == 110
               select fl;

   FL_Fluege f1 = menge.Single<FL_Fluege>();

   Console.WriteLine("Before: " + f1.FL_FlugNr + " = " + f1.FL_FreiePlaetze);

   f1.FL_FreiePlaetze--;

   Console.WriteLine("After: " + f1.FL_FlugNr + " = " + f1.FL_FreiePlaetze);

   var menge2 = from fl in db.FL_Flueges
                where fl.FL_FlugNr == 110
                select fl;

   FL_Fluege f2 = menge.Single<FL_Fluege>();


   Console.WriteLine("Reload: " + f1.FL_FlugNr + " = " + f1.FL_FreiePlaetze);
  }

  public static void Demo_Logging()
  {
   LTS.HeadLine("Object Identity");

   DatenbankDataContext db = new DatenbankDataContext();

   var query = from fl in db.FL_Flueges
               where fl.FL_FlugNr <= 110
               select fl;
   Console.WriteLine(query);
   db.Log = Console.Out;

   foreach (FL_Fluege f in query)
   {
    f.FL_FreiePlaetze--;
   }

   Console.WriteLine(db.GetChangeSet());


   foreach (FL_Fluege f in query)
    Console.WriteLine(db.GetCommand(query).CommandText);
  }


  public static void Demo_DelayLoadSpalte()
  {
   LTS.HeadLine("Demo_DelayLoadSpalte");

   DatenbankDataContext db = new DatenbankDataContext();

   var query = from fl in db.FL_Flueges
               where fl.FL_FlugNr <= 110
               select fl;

   db.Log = Console.Out;

   foreach (FL_Fluege f in query)
   {
    Console.WriteLine(f.FL_Datum);
   }


  }


  //public static void SP_Demo()
  //{
  // LTS.HeadLine("SP Demo");

  // DatenbankDataContext db = new DatenbankDataContext();

  // int e1 = db.DeleteFlightsGreater1000();
  // Console.WriteLine(e1);

  // ISingleResult<GetRouteResult> e2 = db.GetRoute(101);
  // Console.WriteLine(e2.ReturnValue);
  // foreach (GetRouteResult f in e2)
  // {
  //  Console.WriteLine(f.FL_AbfLugOrt + "-->" + f.FL_ZielOrt);
  // }
  //}
 }
}
