﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using System.Data;

namespace de.WWWings.Test.LINQtoDS
{
 class LinqToDSDemo
 {

  public void Dataset_Einfach()
  {
   Console.WriteLine("Alle Flüge");

   string CONNSTRING = @"Data Source=.\sqlexpress;Initial Catalog=WWWings4;Integrated Security=True;Connect Timeout=30;User Instance=True";
   const string SQL = "Select * from FL_Fluege";

   // --- Verbindung aufbauen
   SqlConnection conn = new SqlConnection(CONNSTRING);
   conn.Open();
   // --- Leeres DataSet erzeugen
   DataSet ds = new DataSet();
   // --- Datenadapter erzeugen
   SqlDataAdapter da = new SqlDataAdapter(SQL, CONNSTRING);

   // --- Daten abholen
   da.Fill(ds, "Fluege");

   // --- Verbindung jetzt schon schließen!
   conn.Close();

   // --- Tabelle aus einem vorher befüllten DataSet
   DataTable AlleFluege = ds.Tables["Fluege"];

   Object o = AlleFluege.AsEnumerable();
   Console.WriteLine(o.GetType().FullName);
   // --- LINQ Abfrage
   //IEnumerable<DataRow> FluegeVonRom2 =
   //    from Flug in AlleFluege.AsEnumerable()
   //    where Flug.Field<DateTime>("FL_Datum") > 
   //    new DateTime(2008, 1, 1) && 
   //    Flug.Field<string>("FL_AbflugOrt") == "Rom"
   //    select Flug;

   IEnumerable<DataRow> FluegeVonRom =
    from Flug in AlleFluege.AsEnumerable()
    where Convert.ToDateTime(Flug["FL_Datum"]) >
    new DateTime(2008, 1, 1) &&
    Flug["FL_AbflugOrt"].ToString() == "Rom"
    select Flug;



   // --- Iteration über Daten
   foreach (DataRow dr in FluegeVonRom)
   {
    Console.WriteLine("Flugnummer: " + Convert.ToInt32(dr["FL_FlugNr"]) + ": " + dr["FL_Abflugort"].ToString() + "->" + dr.Field<string>("FL_Zielort") + ". Freie Plätze: " + dr.Field<Int32>("Fl_FreiePlaetze"));

    dr.SetField<DateTime>("FL_Datum", DateTime.Now);

    int? FL_FreiePlaetze = dr.Field<int?>("FL_FreiePlaetze");
    if (!FL_FreiePlaetze.HasValue)
    {
     Console.WriteLine("Kein StartDatum gesetzt!");
     dr.SetField<int?>("FL_FreiePlaetze", dr.Field<Int32?>("FL_Plaetze"));
    };

   }

   Console.WriteLine("Variante -------------------");
   DataTable FluegeVonRomTabelle = FluegeVonRom.CopyToDataTable<DataRow>();

   // --- Iteration über Daten
   foreach (DataRow dr in FluegeVonRomTabelle.Rows)
   {
    Console.WriteLine("Flugnummer: " + dr["FL_FlugNr"] + ": " + dr["FL_Abflugort"].ToString() + "->" + dr["FL_Zielort"] + ". Freie Plätze: " + dr["Fl_FreiePlaetze"]);
   }

  }

  public void Dataset_TDS()
  {
   Console.WriteLine("Flüge von ROM (TDS mit lokalem Filter)");

   // Tabellenadapter instanzieren 
   TDS.WWWingsTDSTableAdapters.FL_FluegeTableAdapter ta = new de.WWWings.Test.TDS.WWWingsTDSTableAdapters.FL_FluegeTableAdapter();
   // Typisiertes DataSet instanzieren
   TDS.WWWingsTDS.FL_FluegeDataTable AlleFluege = new de.WWWings.Test.TDS.WWWingsTDS.FL_FluegeDataTable();
   // Tabelle laden
   ta.Fill(AlleFluege);

   // Filter anwenden
   DataRow[] FluegeVonRom = AlleFluege.Select("FL_Abflugort = 'Rom' and FL_Datum > '1/1/2008'");

   // --- Iteration über Daten
   foreach (TDS.WWWingsTDS.FL_FluegeRow dr in FluegeVonRom)
   {
    Console.WriteLine("Flugnummer: " + dr.FL_FlugNr + ": " + dr.FL_Abflugort + "->" + dr.FL_Zielort + ". Freie Plätze: " + dr.FL_FreiePlaetze);

    if (!dr.IsFl_StartZeitNull()) Console.WriteLine("Kein StartDatum gesetzt!");
   }
  }

  public void Dataset_TDS_LINQ()
  {
   Console.WriteLine("Flüge von ROM (TDS mit lokalem Filter über LINQ-to-DataSet)");

   // Tabellenadapter instanzieren 
   TDS.WWWingsTDSTableAdapters.FL_FluegeTableAdapter ta = new de.WWWings.Test.TDS.WWWingsTDSTableAdapters.FL_FluegeTableAdapter();
   // Typisiertes DataSet instanzieren
   TDS.WWWingsTDS.FL_FluegeDataTable AlleFluege = new de.WWWings.Test.TDS.WWWingsTDS.FL_FluegeDataTable();
   // Tabelle laden
   ta.Fill(AlleFluege);

   var FluegeVonRom =
    from Flug in AlleFluege
    where Flug.FL_Datum > new DateTime(2008, 1, 1)
    && Flug.FL_Abflugort == "Rom"
    select Flug;
   
   // --- Iteration über Daten
   foreach (TDS.WWWingsTDS.FL_FluegeRow dr in FluegeVonRom)
   {
    Console.WriteLine("Flugnummer: " + dr.FL_FlugNr + ": " + dr.FL_Abflugort + "->" + dr.FL_Zielort + ". Freie Plätze: " + dr.FL_FreiePlaetze);

    if (!dr.IsFl_StartZeitNull()) Console.WriteLine("Kein StartDatum gesetzt!");
   }
  }

  public void Dataset_Beziehungen()
  {
   Console.WriteLine("Passagiere mit Ihren Flügen");

   // --- Parameter
   string CONNSTRING = @"Data Source=.\sqlexpress;Initial Catalog=WWWings4;Integrated Security=True;Connect Timeout=30;User Instance=True";
   const string SQL1 = "Select * from AllePassagiere";
   const string SQL2 = "Select * from GF_GebuchteFluege";

   // --- Verbindung aufbauen
   SqlConnection conn = new SqlConnection(CONNSTRING);
   conn.Open();
   // --- Leeres DataSet erzeugen
   DataSet ds = new DataSet();
   // --- Datenadapter erzeugen
   SqlDataAdapter da1 = new SqlDataAdapter(SQL1, CONNSTRING);
   SqlDataAdapter da2 = new SqlDataAdapter(SQL2, CONNSTRING);
   // --- Daten abholen
   da1.Fill(ds, "AllePassagiere");
   da2.Fill(ds, "GebuchteFluege");
   // --- Verbindung jetzt schon schließen!
   conn.Close();

   // --- Zeiger auf Tabellen
   DataTable AllePassagiere = ds.Tables["AllePassagiere"];
   DataTable GebuchteFluege = ds.Tables["GebuchteFluege"];

   // --- Join definieren
   var Abfrage =
       from p in AllePassagiere.AsEnumerable()
       join b in GebuchteFluege.AsEnumerable()
       on p.Field<int>("PS_ID") equals
           b.Field<int>("GF_PS_ID")
       where p.Field<int>("AnzahlFluege") > 0
       select new
       {
        FlugNr =
            b.Field<int>("GF_FL_FlugNr"),
        Vorname =
            p.Field<string>("PE_Vorname"),
        Name =
            p.Field<string>("PE_Name"),
        PassagierID =
            p.Field<int>("PS_ID")
       };

   // --- Ergebnis anzeigen
   foreach (var Ergebnis in Abfrage)
   {
    Console.WriteLine("Pasagier: " + Ergebnis.PassagierID + " Name: " + Ergebnis.Name + " Vorname: " + Ergebnis.Vorname + " gebucht auf Flug " + Ergebnis.FlugNr);
   }
  }


 }
}
