﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Linq.Dynamic;
using System.Linq.Expressions;

namespace de.WWWings.Test.LINQExtensions
{
 /// <summary>
 /// Quelle: http://stackoverflow.com/questions/389094/how-to-create-a-dynamic-linq-join-extension-method
 /// </summary>
static class DynamicJoin
 {
  public static IQueryable Join(this IQueryable outer, IEnumerable inner, string outerSelector, string innerSelector, string resultsSelector, params object[] values)
  {
   if (inner == null) throw new ArgumentNullException("inner");
   if (outerSelector == null) throw new ArgumentNullException("outerSelector");
   if (innerSelector == null) throw new ArgumentNullException("innerSelector");
   if (resultsSelector == null) throw new ArgumentNullException("resultsSelctor");

   LambdaExpression outerSelectorLambda = System.Linq.Dynamic.DynamicExpression.ParseLambda(outer.ElementType, null, outerSelector, values);
   LambdaExpression innerSelectorLambda = System.Linq.Dynamic.DynamicExpression.ParseLambda(inner.AsQueryable().ElementType, null, innerSelector, values);

   ParameterExpression[] parameters = new ParameterExpression[] { 
            Expression.Parameter(outer.ElementType, "outer"), Expression.Parameter(inner.AsQueryable().ElementType, "inner") };
   LambdaExpression resultsSelectorLambda = System.Linq.Dynamic.DynamicExpression.ParseLambda(parameters, null, resultsSelector, values);

   return outer.Provider.CreateQuery(
       Expression.Call(
           typeof(Queryable), "Join",
           new Type[] { outer.ElementType, inner.AsQueryable().ElementType, outerSelectorLambda.Body.Type, resultsSelectorLambda.Body.Type },
           outer.Expression, inner.AsQueryable().Expression, Expression.Quote(outerSelectorLambda), Expression.Quote(innerSelectorLambda), Expression.Quote(resultsSelectorLambda)));
  }


  //The generic overload. 
  public static IQueryable<T> Join<T>(this IQueryable<T> outer, IEnumerable<T> inner, string outerSelector, string innerSelector, string resultsSelector, params object[] values)
  {
   return (IQueryable<T>)Join((IQueryable)outer, (IEnumerable)inner, outerSelector, innerSelector, resultsSelector, values);
  } 

 }
}
