﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using System.Reflection;
using System.Data.Metadata.Edm;
using System.Data;
using System.Data.Objects;
using System.Data.Objects.DataClasses;

namespace ITVisions
{

    public static class RSLinqExtensions
    {
      [System.Diagnostics.DebuggerStepThrough]
        public static IEnumerable<T> SelectPartial<T>(this IQueryable<T> source, Expression<Func<T,object>> selectExpression){
            bool addEntityKey = false;
            string entitySetName = null;
            NewExpression body = selectExpression.Body as NewExpression;

            if (body == null) {
                throw new ArgumentOutOfRangeException("selectExpression", "The select Expression has to be of type [System.Linq.Expressions.NewExpression].");
            }

            if (body.Arguments.Any(p => !(p is MemberExpression) || (p is MemberExpression && !(((MemberExpression)p).Expression is ParameterExpression)))) 
            { 
                throw new ArgumentOutOfRangeException("selectExpression", "The select Expression can only contain MemberExpressions for the given Parameter.");
            }

            var assignments = body.Arguments.Cast<MemberExpression>().Select((p, i) => new { Member = body.Members[i] as PropertyInfo, Assignment = p.Member });

            if (source is ObjectQuery)
            {
                var context = ((ObjectQuery)source).Context;
                addEntityKey = source.ElementType.GetInterfaces().Contains(typeof(IEntityWithKey));

                EntityType entityType = context.MetadataWorkspace.GetItems<System.Data.Metadata.Edm.EntityTypeBase>(System.Data.Metadata.Edm.DataSpace.CSpace)
                            .First(p => p.Name == source.ElementType.Name) as EntityType;
                EntitySetBase entitySet = null;

                if (!entityType.KeyMembers.Select(p => p.Name).All(p => assignments.Any(x => x.Assignment.Name == p))) {
                    throw new MissingMemberException("All primary key columns have to be mapped.");
                }

                EdmType tmpType = entityType;
                while (entitySet == null && tmpType != null)
                {


                    entitySet = context.MetadataWorkspace.GetItems<EntityContainer>(System.Data.Metadata.Edm.DataSpace.CSpace)
                            .First().BaseEntitySets
                            .FirstOrDefault(p => p.ElementType == tmpType);
                    tmpType = tmpType.BaseType;
                }

                entitySetName = string.Format("{0}.{1}", entitySet.EntityContainer.Name, entitySet.Name);
            }
            IEnumerable<object> materializedSource = source.Select(selectExpression);

            var parameterExpression = Expression.Parameter(typeof(object));
            var memberBindExpression = assignments.Select(p => Expression.Bind(p.Assignment,Expression.Property(Expression.Convert(parameterExpression,body.Type),p.Member))).ToList();

            NewExpression newExpression = null;
            if (source is ObjectQuery)
            {
                var context = ((ObjectQuery)source).Context;

                if (context.ContextOptions.ProxyCreationEnabled)
                {
                    context.CreateProxyTypes(new[] { source.ElementType });
                    var proxyType = ObjectContext.GetKnownProxyTypes()
                        .Where(p => ObjectContext.GetObjectType(p) == source.ElementType)
                        .FirstOrDefault();
                    if (proxyType != null)
                    {
                        newExpression = Expression.New(proxyType);
                    }
                    else {
                        newExpression = Expression.New(source.ElementType);
                    }
                }
                else 
                {
                    newExpression = Expression.New(source.ElementType);
                }
                
            }
            else 
            {
                newExpression = Expression.New(source.ElementType);
            }

            var lambda = Expression.Lambda<Func<object,T>>(Expression.MemberInit(newExpression, memberBindExpression),parameterExpression);

            foreach (var item in materializedSource.Select(lambda.Compile())) {
                if (source is ObjectQuery)
                {
                    var context = ((System.Data.Objects.ObjectQuery<T>)source).Context;
                    context.AttachTo(entitySetName,item);
                    context.ObjectStateManager.ChangeObjectState(item, System.Data.EntityState.Unchanged);
                }
                yield return item;
            }
        }
    }
}
