using System;
using System.Collections.Generic;
using System.Text;
using System.Data;

namespace de.WWWings.ORM
{

 /// <summary>
 /// Hndische Implementierung des Objek-Relationalen Mappings fr die Tabelle FL_Fluege auf die Klassen Flug und FlugMenge.<remarks>
 /// In World Wide Wings wird bewut keins der am Markt verfgbaren Drittanbieterwerkzeuge fr ORM eingesetzt, da erstens die Komplexitt des Beispiels nicht weiter erhht werden soll und zweitens eine Marktbereinigung zu erwarten ist, sobald Microsoft die angekndigten ORM-Werkzeuge auf den Markt bringt.</remarks>
 /// </summary>
 class ORM_Flug
 {

  /// <summary>
  /// Mapping fr DataRow "zu Fu programmiert!"
  /// </summary>
  public static Flug MapRow(System.Data.DataRow dr)
  {
   Flug f = new Flug();
   f.FlugNr = Convert.ToInt64(dr["fl_flugnr"]);
   f.AbflugOrt = dr["fl_AbflugOrt"].ToString() ?? "n/a";
   f.ZielOrt = dr["fl_ZielOrt"].ToString() ?? "n/a";
   f.Plaetze = Convert.ToInt64(dr["fl_Plaetze"]);
   f.FreiePlaetze = Convert.ToInt64(dr["fl_FreiePlaetze"]);
   f.Datum = Convert.ToDateTime(dr["Fl_Datum"]);
   return f;
  }

  /// <summary>
  /// Mapping fr DataRreader
  /// </summary>
  public static Flug MapRow(System.Data.IDataReader dr)
  {
   Flug f = new Flug();
   f.FlugNr = Convert.ToInt64(dr["fl_flugnr"]);
   f.AbflugOrt = dr["fl_AbflugOrt"].ToString() ?? "n/a";
   f.ZielOrt = dr["fl_ZielOrt"].ToString() ?? "n/a";
   f.Plaetze = GetLong(dr, "fl_Plaetze");
   f.FreiePlaetze = GetLong(dr, "fl_FreiePlaetze");
   f.Datum = GetDateTime(dr, "Fl_Datum");
   if (!Flug.Fluege.ContainsKey(f.FlugNr)) Flug.Fluege.Add(f.FlugNr, f);
   return f;
  }

  public static long GetLong(IDataReader dr, string Name)
  {
   if (dr[Name] is DBNull) return 0;
  return Convert.ToInt64(dr[Name]);

  }

  public static DateTime GetDateTime(IDataReader dr, string Name)
  {
   if (dr[Name] is DBNull) return DateTime.MinValue;
   return Convert.ToDateTime(dr[Name]);

  }

  /// <summary>
  /// Mapping fr DataTable
  /// </summary>
  /// <param name="dt">DataTable-Objekt</param>
  /// <returns>Menge von Flugobjekten</returns>
  public static List<Flug> MapTable(System.Data.DataTable dt)
  {
   List<Flug> ff = new List<Flug>();
   foreach (System.Data.DataRow dr in dt.Rows)
   {
    Flug f = MapRow(dr);
    ff.Add(f);
   }
   return ff;
  }
 }
}
