﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using de.WWWings.Test.Modelle.EF6Vererbung;

namespace de.WWWings.Test.Modelle.EF6Vererbung
{
 class EF6VerbungDemos
 {



  public static void DemoLazyLoadingAutomatisch()
  {
   using (WWWings6VererbungEntities modell = new WWWings6VererbungEntities())
   {
    int Count = 0;
    // Lade die Liste aller Flüge
    var ListeDerFluege = (from f in modell.Flug select f).ToList(); 

    // Schleife über alle Flüge
    foreach (Flug f in ListeDerFluege.ToList())
    {
     Count++;

     // Ausgabe: Flugdaten mit Pilotname
     if (f.Pilot != null)
     {
     //  Console.WriteLine("{5}. Flug {0} von {1} nach {2} am {3} mit Pilot {4}.", f.FlugNr, f.Abflugort, f.Zielort, f.Datum.ToShortDateString(), f.Pilot.Vorname + " " + f.Pilot.Name, Count);
     }
     else
     {
     //  Console.WriteLine("{5}. Flug {0} von {1} nach {2} am {3}. Der Pilot steht noch nicht fest.", f.FlugNr, f.Abflugort, f.Zielort, f.Datum.ToShortDateString(), Count);
     }
    } // Ende foreach
   } // Ende using
  } // Ende Methode


  public static void DemoLazyLoadingExplizit1()
  {
   using (WWWings6VererbungEntities modell = new WWWings6VererbungEntities())
   {
    // Lazy Loading abschalten 
    modell.ContextOptions.LazyLoadingEnabled = false;

    int Count = 0;
    // Lade die Liste aller Flüge
    var ListeDerFluege = (from f in modell.Flug select f).ToList();

    // Schleife über alle Flüge
    foreach (Flug f in ListeDerFluege.ToList())
    {
     Count++;
     
     // Explizites Laden
     //f.PilotReference.Load();
   
     modell.LoadProperty(f, fo => fo.Pilot);

     // Ausgabe: Flugdaten mit Pilotname
     if (f.Pilot != null)
     {
     //  Console.WriteLine("{5}. Flug {0} von {1} nach {2} am {3} mit Pilot {4}.", f.FlugNr, f.Abflugort, f.Zielort, f.Datum.ToShortDateString(), f.Pilot.Vorname + " " + f.Pilot.Name, Count);
     }
     else
     {
     //  Console.WriteLine("{5}. Flug {0} von {1} nach {2} am {3}. Der Pilot steht noch nicht fest.", f.FlugNr, f.Abflugort, f.Zielort, f.Datum.ToShortDateString(), Count);
     }
    } // Ende foreach
   } // Ende using
  } // Ende Methode


  public static void DemoLazyLoadingExplizit2()
  {
   using (WWWings6VererbungEntities modell = new WWWings6VererbungEntities())
   {
    // Lazy Loading abschalten 
    modell.ContextOptions.LazyLoadingEnabled = false;

    int Count = 0;
    // Lade die Liste aller Flüge
    var ListeDerFluege = (from f in modell.Flug select f).ToList();
    
    // Schleife über alle Flüge
    foreach (Flug f in ListeDerFluege.ToList())
    {
     Count++;

     // Explizites Laden, wenn noch nicht vorhanden
     if (!f.PilotReference.IsLoaded) f.PilotReference.Load();

     // Ausgabe: Flugdaten mit Pilotname
     if (f.Pilot != null)
     {
     //  Console.WriteLine("{5}. Flug {0} von {1} nach {2} am {3} mit Pilot {4}.", f.FlugNr, f.Abflugort, f.Zielort, f.Datum.ToShortDateString(), f.Pilot.Vorname + " " + f.Pilot.Name, Count);
     }
     else
     {
     //  Console.WriteLine("{5}. Flug {0} von {1} nach {2} am {3}. Der Pilot steht noch nicht fest.", f.FlugNr, f.Abflugort, f.Zielort, f.Datum.ToShortDateString(), Count);
     }
    } // Ende foreach
   } // Ende using
  } // Ende Methode

  public static void DemoEagerLoading()
  {
   using (WWWings6VererbungEntities modell = new WWWings6VererbungEntities())
   {
    int Count = 0;

    // Lazy Loading abschalten 
    modell.ContextOptions.LazyLoadingEnabled = false;

    // Lade die Liste aller Flüge
    var ListeDerFluege = (from f in modell.Flug.Include("Pilot") select f).ToList();
   
    // Schleife über alle Flüge
    foreach (Flug f in ListeDerFluege.ToList())
    {
     Count++;

     // Ausgabe: Flugdaten mit Pilotname
     if (f.Pilot != null)
     {
     //  Console.WriteLine("{5}. Flug {0} von {1} nach {2} am {3} mit Pilot {4}.", f.FlugNr, f.Abflugort, f.Zielort, f.Datum.ToShortDateString(), f.Pilot.Vorname + " " + f.Pilot.Name, Count);
     }
     else
     {
     //  Console.WriteLine("{5}. Flug {0} von {1} nach {2} am {3}. Der Pilot steht noch nicht fest.", f.FlugNr, f.Abflugort, f.Zielort, f.Datum.ToShortDateString(), Count);
     }
    } // Ende foreach
   } // Ende using
  } // Ende Methode

  public static void DemoRelationshipFixup()
  {
   using (WWWings6VererbungEntities modell = new WWWings6VererbungEntities())
   {
    int Count = 0;

    // Lazy Loading abschalten 
    modell.ContextOptions.LazyLoadingEnabled = false;
    
    // Sehr früheres Laden in den Zwischenspeicher (Cache) des Kontextes
    var VorabGeladenePiloten = modell.Person.OfType<Pilot>().ToList();

    // Lade die Liste aller Flüge
    var ListeDerFluege = (from f in modell.Flug select f).ToList(); 

    // Schleife über alle Flüge
    foreach (Flug f in ListeDerFluege.ToList())
    {
     Count++;

     // Ausgabe: Flugdaten mit Pilotname
     if (f.Pilot != null)
     {
     //  Console.WriteLine("{5}. Flug {0} von {1} nach {2} am {3} mit Pilot {4}.", f.FlugNr, f.Abflugort, f.Zielort, f.Datum.ToShortDateString(), f.Pilot.Vorname + " " + f.Pilot.Name, Count);
     }
     else
     {
     //  Console.WriteLine("{5}. Flug {0} von {1} nach {2} am {3}. Der Pilot steht noch nicht fest.", f.FlugNr, f.Abflugort, f.Zielort, f.Datum.ToShortDateString(), Count);
     }
    } // Ende foreach
   } // Ende using
  } // Ende Methode



  public static void RelFixUpDemoMN()
  {
   using (WWWings6VererbungEntities modell = new WWWings6VererbungEntities())
   {
    var VorabGeladeneFluege = modell.Flug.ToList(); // Include("Passagier") hilft auch nicht
    var VorabGeladenePiloten = modell.Person.OfType<Pilot>().ToList();


    var ListeDerPassagiereMitNamenSchulz = from p in modell.Person.OfType<Passagier>() where p.Name.Contains("Schulz") select p;

    foreach (Passagier p in ListeDerPassagiereMitNamenSchulz.ToList())
    {
    //  Console.WriteLine("Passagier {0} {1}", p.Vorname, p.Name);
    //  Console.WriteLine("------------------------------------");
     // Zeige für alle Passagiere, die Meier heißen, die Flüge mit Piloten
     foreach (Flug f in p.Fluege)
     {
      if (f.Pilot != null)
      {
      //  Console.WriteLine(" Gebucht auf Flug {0} von {1} nach {2} am {3} mit Pilot {4}.", f.FlugNr, f.Abflugort, f.Zielort, f.Datum.ToShortDateString(), f.Pilot.Vorname + " " + f.Pilot.Name);
      }
      else
      {
      //  Console.WriteLine(" Gebucht auf Flug {0} von {1} nach {2} am {3}. Der Pilot steht noch nicht fest.", f.FlugNr, f.Abflugort, f.Zielort, f.Datum.ToShortDateString());
      }

     }
    }
   }
  }







  public static void Run2()
  {


   // Modell instanziieren unter Verwendung der Verbindungszeichenfolge aus Konfigurationsdatei
   using (WWWings6VererbungEntities modell = new WWWings6VererbungEntities())
   {
    string ort = "Rom";
    string name = "Müller";

    // Abfrage definieren 
    var fluege = from f in modell.Flug
                 where f.Abflugort == ort && f.Passagiere.Count > 0 && f.Passagiere.Any(p => p.Name == name)
                 select f;
    // Ergebnis ausgeben
    foreach (Flug f in fluege)
    {
    //  Console.WriteLine("Flug Nr {0} von {1} nach {2} hat {3} freie Plätze!", f.FlugNr, f.Abflugort, f.Zielort, f.FreiePlaetze);

     // Passagiere ausgeben
     foreach (Passagier ps in f.Passagiere)
     {
     //  Console.WriteLine(" - " + ps.Name);

      // Entferne den Passagier mit gesuchtem Namen
      if (ps.Name == name) f.Passagiere.Remove(ps);
     }

     // Platzanzahl erhöhen
     f.FreiePlaetze++;
    }

    // Alle Änderung speichern
    modell.SaveChanges();
   } // Ende using-Block -> Dispose() wird aufgerufen

  }


  public static void Run1()
  {
  //  Console.WriteLine("Get Passagier...");

   WWWings6VererbungEntities modell = new WWWings6VererbungEntities();

   var q = modell.Person.OfType<Passagier>().Take(1);

   foreach (Passagier p in q)
   {
   //  Console.WriteLine(p.Name);
   }


  }


  /// <summary>
  /// Klausuraufgabe FH Münster, Probeklausur
  /// </summary>
  public static void Klausuraufgabe1()
  {
   using (WWWings6VererbungEntities model = new WWWings6VererbungEntities())
   {
    List<Passagier> PassagiereMitS = (from p in model.Person.OfType<Passagier>()
                                      where p.Vorname.StartsWith("S")
                                      orderby p.Geburtstag descending
                                      select p).Take(200).ToList();

    foreach (var Passagier in PassagiereMitS)
    {
     Console.WriteLine(Passagier.Vorname + " " + Passagier.Name + " ist geboren am " + Passagier.Geburtstag.Value.ToShortDateString());
    }
   }

  }

  /// <summary>
  /// Klausuraufgabe FH Münster, Klausur 30.1.2012
  /// </summary>
  public static void Klausuraufgabe2()
  {
   using (WWWings6VererbungEntities model = new WWWings6VererbungEntities())
   {
    List<Flug> Fluege = (from f in model.Flug
                         where f.Abflugort.StartsWith("M") &&
                         f.FreiePlaetze >= 3 &&
                         f.NichtRaucherFlug == true
                         orderby f.Datum descending
                         select f).Skip(10).Take(10).ToList();

    foreach (var f in Fluege)
    {
     Console.WriteLine(f.FlugNr + " " + f.Abflugort + " " + f.FreiePlaetze);
    }
   }
  }

     /// <summary>
  /// Klausuraufgabe FH Münster, Klausur 2.7.2012
  /// </summary>
  public static void Klausuraufgabe3()
  {
   using (WWWings6VererbungEntities flugModel = new WWWings6VererbungEntities())
   {
    List<Pilot> Piloten = (from p in flugModel.Person.OfType<Pilot>()
                         where p.Name.StartsWith("r") &&
                         p.Fluege.Count >= 3 &&
                         (p.FlugscheinSeit.Value-DateTime.Now).Days>100
                         orderby p.Geburtstag descending
                         select p).Skip(5).Take(5).ToList();

    foreach (var p in Piloten)
    {
     Console.WriteLine(p.Name);
    }
   }
  }

 }
}
