﻿using System;
using System.Collections.Generic;
using System.Text;
using WWWIngs.Core.Domain;
using NHibernate.Cfg;
using NHibernate;
using WWW_NHibernate.NHibernate.Domain;

namespace WWW_NHibernate
{
    class Beispiele_Einfach
    {


   


       

        public static void AllDemos()
        {
            // ------ Vorbereitung
            Configuration cfg = new Configuration();
            // Laden aller eingebetten Mapping-Dateien in dieser Assembly
            cfg.AddAssembly(System.Reflection.Assembly.GetExecutingAssembly());
            ISessionFactory factory = cfg.BuildSessionFactory();
            ISession session = factory.OpenSession(new Interceptor());

            // ------ Fluganzahl (SQL Direkt)
            IList<Flug> Menge = session.CreateSQLQuery(
              "Select * from FL_Fluege where FL_Abflugort='Rom'")
              .AddEntity(typeof(Flug)).List<Flug>();

            Console.WriteLine("Anzahl Flüge: " + Menge.Count);

            // ------ einen Flug Löschen
            Head("Lösche einen Flug...");

            // Get liefert Objekt oder null
            Flug f7 = (Flug)session.Get(typeof(Flug), 999);
            if (f7 != null)
            {
                session.Delete(f7);
                session.Flush();
            }


            // ------ Wege, ein Objekt zu laden
            Flug f101;
            // Get liefert Objekt oder null
            f101 = (Flug)session.Get(typeof(Flug), 101);
            f101 = session.Get<Flug>(101);
            // Load(): liefert Exception beim Zugriff auf Attribute, wenn Objekt nicht existiert
            f101 = (Flug)session.Load(typeof(Flug), 101);
            f101 = session.Load<Flug>(101);

            // ------ Prüfe auf Existenz eines Fluges
            Head("Gibt es einen Flug?");

            // Get(): liefert Objekt oder null
            Flug f9 = (Flug)session.Get(typeof(Flug), 888);
            if (f9 == null) Console.WriteLine("Flug 888 nicht vorhanden!");

            // Load(): liefert Exception beim Zugriff auf Attribute
            try
            {
                Flug f10 = (Flug)session.Load(typeof(Flug), 888);
                int? x = f10.FLFreiePlaetze;
            }
            catch (Exception)
            {
                Console.WriteLine("Flug 888 nicht vorhanden!");
            }



            // ------ Fluganzahl
            Menge = session.CreateSQLQuery("Select * from FL_Fluege").AddEntity(typeof(Flug)).List<Flug>();
            Console.WriteLine("Anzahl Flüge: " + Menge.Count);
            session.Close();

            // ------ Neue Session
            Head("Neue Session!");
            session = factory.OpenSession(new Interceptor());

            // ------ Flüge verändern in Transaktion
            Head("Verändere Flüge in Transaktion...");

            ITransaction trans = session.BeginTransaction();
            Flug f1 = (Flug)session.Load(typeof(Flug), 402);
            Info(f1);
            f1.FLFreiePlaetze -= 2;
            Info(f1);
            Flug f2 = session.Get<Flug>(403);
            Info(f2);
            f2.FLFreiePlaetze -= 2;
            Info(f2);
            trans.Commit();

            // ------ Neuer Flug
            Head("Neuer Flug...");
            Flug f = new Flug();
            f.FLFlugNr = 999;
            f.FLAbflugort = "x";
            f.FLZielort = "y";
            session.Save(f);


            // ------ Fluganzahl
            Menge = session.CreateSQLQuery("Select * from FL_Fluege").AddEntity(typeof(Flug)).List<Flug>();
            Console.WriteLine("Anzahl Flüge: " + Menge.Count);

            Head("---Kontrolle:");

            Flug f3 = (Flug)session.Load(typeof(Flug), 402);
            Info(f3);
            Flug f4 = (Flug)session.Load(typeof(Flug), 403);
            Info(f4);

            session.Close();

            // ------ Neue Session
            Head("Neue Session!");
            session = factory.OpenSession(new Interceptor());

            Head("Auflisten mit Lazy Loading");
            IEnumerable<Flug> Menge2 = session.CreateQuery(
             "from Flug as flug where flug.FLAbflugort = 'Rom'").Enumerable<Flug>();

            foreach (Flug f12 in Menge2)
            {
                Info(f12);
            }

            session.Close();
            // ------ Neue Session
            Head("Neue Session!");

            session = factory.OpenSession(new Interceptor());

            Head("Auflisten mit Eager Loading");
            IList<Flug> Menge3 = session.CreateQuery(
             "from Flug as flug where flug.FLAbflugort = 'Rom'").List<Flug>();

            foreach (Flug f12 in Menge3)
            {
                Info(f12);
            }


            session.Close();

            // ------ Neue Session
            Head("Neue Session!");
            session = factory.OpenSession(new Interceptor());

            Head("Auflisten mit Paging");
            IQuery q = session.CreateQuery("from Flug as flug where flug.FLAbflugort = 'Rom'");
            q.SetFirstResult(5);
            q.SetMaxResults(2);
            IList<Flug> Menge4 = q.List<Flug>();

            foreach (Flug f12 in Menge4)
            {
                Info(f12);
            }


            session.Close();
        }

        public static void Info(Flug f)
        {
            Console.WriteLine("Flug #{0} von {1} nach {2} hat {3} freie Plätze.", f.FLFlugNr, f.FLAbflugort, f.FLZielort, f.FLFreiePlaetze);
        }

        public static void Head(string s)
        {
            Console.ForegroundColor = ConsoleColor.Yellow;
            Console.WriteLine(s);
            Console.ForegroundColor = ConsoleColor.Gray;

        }
    }
}
