using System;
using System.DirectoryServices;
using ActiveDs;
using de.ITVisions.DemoViewer;

// Autor: Dr. Holger Schwichtenberg, http://www.IT-Visions.de
// Hinweis: Ersetzen Sie demo.print durch eine Ausgabefunktion ihrer Wahl oder laden Sie das Gesamtprojekt herunter unter
// http://www.IT-Visions.de/WWWings


namespace Demos.NET2.FCL
{

 [Demo()]
 public class Verzeichnisdienste
 {

  private const string LDAProot = "LDAP://xfilesserver/DC=FBI,DC=net";
  private const string LDAP_USER = "LDAP://xfilesserver/CN=Fox Mulder,OU=Agents,DC=FBI,DC=net";
  private const string LDAP_OU = "LDAP://xfilesserver/OU=agents,DC=FBI,DC=net";

  public void Windows_Authentifizierung()
  {


   Demo.Print(WindowsAuthentifizierung.IsADSIAuthentifizierbar("LDAP://xfilesserver/dc=fbi,dc=net", "fbi", "FoxMulder", "ILoveScully123").ToString());

   Demo.Print(WindowsAuthentifizierung.IsADSIAuthentifizierbar("WinNT://xfilesserver", "fbi", "FoxMulder", "ILoveScully123").ToString());

   Demo.Print(WindowsAuthentifizierung.IsWin32Authentifizierbar("E01", "ab", "123").ToString());
  }




  public void WinNT_Benutzer_Info()
  {

   DirectoryEntry o = null;
   // --- Zugriff auf Eintrag
   o = new DirectoryEntry("WinNT://localhost/HS");
   // --- Basis-Daten des Eintrags
   Demo.Print("Name: " + o.Name);
   Demo.Print("Pfad: " + o.Path);
   Demo.Print("Klasse:" + o.SchemaClassName);
   // --- Ausgabe der Verzeichnisattribute
   foreach (var p in o.Properties.PropertyNames)
   {
    Demo.Print(p.ToString() + " = " + o.Properties[p.ToString()][0].ToString());
   }
  }


  //[AutoRun()]
  public void ADS_Benutzer_Info()
  {

   DirectoryEntry o = null;
   // --- Zugriff auf Eintrag
   o = new DirectoryEntry(LDAP_USER);
   // --- Basis-Daten des Eintrags
   Demo.Print("Name: " + o.Name);
   Demo.Print("Pfad: " + o.Path);
   Demo.Print("Klasse:" + o.SchemaClassName);
   // --- Ausgabe der Verzeichnisattribute
   Demo.Print("SAMAccountName: " + o.Properties["samAccountName"][0].ToString());
   Demo.Print("Description: " + getAtt(o, "Description"));
   Demo.Print("telephoneNumber: " + getAtt(o, "telephoneNumber"));
   Demo.Print("AccountExpires: " + getAtt(o, "AccountExpires"));
   Demo.Print("AccountExpires: " + getDateTime(o, "AccountExpires"));
   Demo.Print("City: " + getAtt(o, "l"));
   // --- Native COM-ADSI
   Demo.Print(((o.NativeObject) as IADs).ADsPath.ToString());

  }

  public string getDateTime(DirectoryEntry o, string a)
  {
   LargeInteger li = (LargeInteger)(o.Properties[a][0]);
   if ((li.HighPart == 0) && (li.LowPart == 0))
    return "n/a";
   long dt_zahl = ((System.Convert.ToInt64(li.HighPart) << 32) + System.Convert.ToInt64(li.LowPart));
   System.DateTime dt = DateTime.FromFileTime(dt_zahl);
   return dt.ToString();
  }

  public void ADS_Benutzer_Liste()
  {

   DirectoryEntry c = new DirectoryEntry(LDAP_OU);
   foreach (DirectoryEntry o in c.Children)
   {
    Demo.Print(o.SchemaClassName + ": " + o.Path.ToString());
   }

  }

  public void ADS_Benutzer_Aendern()
  {

   DirectoryEntry o = null;
   // --- Zugriff auf Eintrag
   o = new DirectoryEntry(LDAP_USER);
   // --- Wert holen und verndern
   string Beschreibungstext = getAtt(o, "Description");
   long AnzahlFaelle = long.Parse(Beschreibungstext.Substring(Beschreibungstext.IndexOf(":") + 1, 2));
   AnzahlFaelle += 1;
   o.Properties["Description"][0] = "Anzahl gelster Flle: " + AnzahlFaelle;
   //o.Properties("telephoneNumber").Remove(o.Properties("telephoneNumber").Item(0))

   o.Properties["telephoneNumber"].Add("001 123 123456");
   o.Properties["telephoneNumber"].Add("001 11111111");
   o.Properties["otherTelephone"].Add("001 11111111");
   o.Properties["otherTelephone"].Add("001 22222222");
   o.Properties["otherTelephone"].Add("001 33333333");

   Demo.Print("Fox Mulder hat nun " + AnzahlFaelle + " Flle gelst!");
   o.CommitChanges();

  }

  public void ADS_OU_Anlegen()
  {
   DirectoryEntry r = null;
   DirectoryEntry ou = null;
   r = new DirectoryEntry(LDAProot);
   ou = r.Children.Add("ou=Directors", "organizationalUnit");
   ou.CommitChanges();
   Demo.Print("OU angelegt! " + ou.Path);
  }

  public string getAtt(DirectoryEntry de, string attributname)
  {
   PropertyValueCollection werteliste = null;
   //INSTANT C# NOTE: Commented this declaration since looping variables in 'foreach' loops are declared in the 'foreach' header in C#
   //   object wert = null;
   string ergebnis = "";
   werteliste = de.Properties[attributname];
   foreach (object wert in werteliste)
   {
    if (ergebnis.Length == 0)
     ergebnis = wert.ToString();
    else
     ergebnis = ergebnis + ";" + wert.ToString();
    return (ergebnis);
   }
   return "";
  }


  // Anlegen eines Benutzers im Active Directory
  public void ADS_Benutzer_Anlegen()
  {
   // AD-spezifische Parameter, bitte anpassen!
   const string LDAP_OU = "LDAP://xfilesserver/OU=Directors,DC=FBI,DC=net";
   const string NAME = "Walter Skinner";
   const string PASSWORD = "abc123/123";
   const string LDAP_Gruppe = "LDAP://xfilesserver/CN=All Directors,DC=FBI,DC=net";
   const string ORT = "Washington";
   const string EMAIL = "WSkinner@fbi.org";
   const string TEL = "++01 123 123456";
   ADS_Benutzer_Anlegen(LDAP_OU, NAME, PASSWORD, TEL, ORT, EMAIL, LDAP_Gruppe);
  }

  public void ADS_Benutzer_MassenAnlegen()
  {
   // AD-spezifische Parameter, bitte anpassen!
   const string LDAP_OU = "LDAP://xfilesserver/OU=Aliens,DC=FBI,DC=net";
   const string PASSWORD = "abc123/123";
   const string LDAP_Gruppe = "LDAP://xfilesserver/CN=All Aliens,DC=FBI,DC=net";
   const string TEL = "123";

   System.DateTime start = DateTime.Now;
   for (long a = 1; a <= 100; a++)
   {
    string zahl = string.Format("{0:0000}", a);
    string name = "Alien" + zahl;
    string email = "Alien" + zahl + "@universe.org";
    string ort = "Planet " + a % 10;
    Demo.PrintHeader(name);
    ADS_Benutzer_Anlegen(LDAP_OU, name, PASSWORD, TEL, ort, email, LDAP_Gruppe);
   }
   System.DateTime ende = DateTime.Now;
   Demo.Print("Dauer in Sekunden: " + (ende - start).Seconds);

  }


  public void Test()
  {
   System.DateTime start = new System.DateTime(2005, 10, 7);
   System.DateTime ende = new System.DateTime(2006, 7, 15);
   Demo.Print((ende - start).Days.ToString());
  }

  // Anlegen eines Benutzers im Active Directory
  public DirectoryEntry ADS_Benutzer_Anlegen(string LDAP_OU, string Name, string Kennwort, string Tel, string Ort, string Email, string LDAP_Gruppe)
  {
   // AD-spezifische Parameter, bitte anpassen!

   //  Anlegen eines User-Objects im Active Directory
   Demo.Print("# Anlegen des Benutzerkontos: " + Name);

   // Zugriff auf IADS
   DirectoryEntry ou = new DirectoryEntry(LDAP_OU);
   // Zugriff auf IADSContainer
   DirectoryEntries c = ou.Children;
   // Neues Objekt erzeugen
   DirectoryEntry u = null;
   try
   {
    u = c.Find("cn=" + Name);
   }
   catch (Exception ex)
   {
   }
   if (u != null)
   {
    Demo.Print("Benutzer existiert schon und wird daher gelscht!");
    c.Remove(u);
   }
   u = c.Add("cn=" + Name, "user");
   // Verzeichnisattribute festlegen
   //u.Properties("sAMAccountName").Add(Name)
   u.Properties["l"].Add(Ort);
   u.Properties["telephoneNumber"].Add(Tel);
   u.Properties["mail"].Add(Email);
   u.Properties["displayName"].Add(Name);
   // nderungen speichern
   u.CommitChanges();
   Demo.Print("Benutzer angelegt: " + u.Path.ToString());
   // Kennwort setzen
   //u.Invoke("SetPassword", Kennwort)
   //u.NativeObject.SetPassword(Kennwort)
   ((ActiveDs.IADsUser)u.NativeObject).SetPassword(Kennwort);
   // Konto aktivieren

   // Konto aktivieren
   ((ActiveDs.IADsUser)u.NativeObject).AccountDisabled = false;
   u.CommitChanges();
   Demo.Print("Benutzer aktiviert!");
   // AD-Benutzer einer AD-Gruppe hinzufgen
   // Zugriff auf Eintrag
   DirectoryEntry g = new DirectoryEntry(LDAP_Gruppe);
   // IADSGroup::Add() aufrufen
   //g.Invoke("Add", u.Path.ToString())
   ((ActiveDs.IADsGroup)g.NativeObject).Add(u.Path.ToString());
   // Besttigung
   Demo.Print("Benutzer zu Gruppe " + g.Name + " hinzugefgt!");
   return u;
  }


  public void ADS_Gruppe_Anlegen()
  {
   const string NAME = "All Directors";

   DirectoryEntry o = null;
   DirectoryEntry gr = null;
   DirectoryEntries c = null;

   o = new DirectoryEntry(LDAProot);
   c = o.Children;

   // === Fall bercksichtigen, dass Gruppe schon da
   try
   {
    gr = c.Find("cn=" + NAME, "group");
   }
   catch (Exception ex)
   {
   }
   if (gr != null)
   {
    Demo.Print("Gruppe existiert schon und wird daher gelscht!");
    c.Remove(gr);
   }
   // === Gruppe anlegen
   gr = c.Add("cn=" + NAME, "group");
   gr.Properties["sAMAccountName"].Add(NAME);
   gr.CommitChanges();
   Demo.Print("Gruppe angelegt! " + gr.Path);

  }


  public void ADS_Objekt_Loeschen()
  {
   DirectoryEntry o = null;
   o = new DirectoryEntry(LDAP_OU);
   o.DeleteTree();
   Demo.Print("Gelscht " + o.Path.ToString());
  }


  // Ausfhren einer LDAP-Suche im AD

  public void LDAP_Suche()
  {

   Demo.Print("Suchanfrage im ADS");

   // Suche alle gesperrten Benutzer
   const string BEDINGUNG = "(&(objectCategory=person)(objectClass=user)(userAccountControl:1.2.840.113556.1.4.803:=2))";
   
   Demo.Print("Suche: " + BEDINGUNG + " in " + LDAProot);

   // Instanziierung der Suchklasse
   DirectorySearcher suche = new DirectorySearcher();
   // Festlegung des Ausgangspunkts
   suche.SearchRoot = new DirectoryEntry(LDAProot);
   // Festlegung der LDAP-Query
   suche.Filter = BEDINGUNG;
   // Suchtiefe festlegen

   suche.SearchScope = SearchScope.Subtree;
   // Ergebnisattribute festlegen (!!! funktioniert entgegen der Dokumentation nicht!!!)
   suche.PropertiesToLoad.Add("displayName");
   suche.PropertiesToLoad.Add("l");
   suche.PropertiesToLoad.Add("description");
   suche.PropertiesToLoad.Add("samaccountname");
   // Suche starten

   SearchResultCollection ergebnisliste = suche.FindAll();
   // Ergebnismenge ausgeben
   Demo.Print("Anzahl Ergebnisobjekte: " + ergebnisliste.Count);
   foreach (SearchResult ergebnis in ergebnisliste)
   {
    //For Each de As System.Collections.DictionaryEntry In ergebnis.Properties
    // Demo.Print(de.Key.ToString() & "=" & (CType(de.Value, ResultPropertyValueCollection))(0).ToString())
    //Next de

    //If ergebnis.Properties.Contains("displayName") AndAlso ergebnis.Properties.Contains("l") Then
    // Demo.Print(ergebnis.Properties("displayName")(0).ToString() & " wohnt in " & ergebnis.Properties("l")(0).ToString())
    //Else
    // Demo.Print(ergebnis.Properties("samaccountname")(0).ToString())
    //End If

    // Einfacher

    Demo.Print(getSAtt(ergebnis, "samaccountname") + " wohnt in " + getSAtt(ergebnis, "l"));


   }
  }

  public string getSAtt(SearchResult se, string attributname)
  {
   ResultPropertyValueCollection werteliste = null;
   //INSTANT C# NOTE: Commented this declaration since looping variables in 'foreach' loops are declared in the 'foreach' header in C#
   //   object wert = null;
   string ergebnis = "";
   werteliste = se.Properties[attributname];
   if (se.Properties.Contains(attributname))
   {
    foreach (object wert in werteliste)
    {
     if (ergebnis.Length == 0)
      ergebnis = wert.ToString();
     else
      ergebnis = ergebnis + ";" + wert.ToString();
    }
   }
   return (ergebnis);
  }

 }
}
