using System;
using System.DirectoryServices;
using ActiveDs;
using de.ITVisions.DemoViewer;

// Autor: Dr. Holger Schwichtenberg, http://www.IT-Visions.de
// Hinweis: Ersetzen Sie demo.print durch eine Ausgabefunktion ihrer Wahl oder laden Sie das Gesamtprojekt herunter unter
// http://www.IT-Visions.de/WWWings

// Achtung: "Hochzhlen" der Attributnamen ist Work-Around zum Testen, weil Lschen von Schemamodifikationen nicht mehr mglich ist

namespace Demos.NET2.FCL
{


 internal class Verzeichnisdienste_ADS_Schema
 {

  private const int ADS_PROPERTY_APPEND = 3;
  private const int DS_INSTANCETYPE_NC_IS_WRITEABLE = 4;
  private long Counter = 0;

  public DirectoryEntry DeactivateSchemaObject(string strSchemaNamingContext, string Name)
  {
   DirectoryEntry objAtt = null;
   try
   {
    Demo.Print("Suche Schema-Objekt: " + Name);
    DirectoryEntry de = new DirectoryEntry(strSchemaNamingContext);
    objAtt = de.Children.Find("cn=" + Name);
   }
   catch (Exception ex)
   {
   }
   if (objAtt != null)
   {
    try
    {
     objAtt.Properties["isDefunct"].Add(true);
     objAtt.CommitChanges();
     Demo.Print("Schema-Objekt deaktiviert: " + Name);
    }
    catch (Exception ex)
    {
    }
   }
   return objAtt;
  }

  public void AttributAnlegen(ActiveDs.IADs objRoot, ActiveDs.IADs objSchema, string strSchemaNamingContext, ref string strAttributeName, long ID)
  {
   ActiveDs.IADs objNewAttribute = null;
   string strCNAttributeName = null;
   string AttrName = null;
   Counter = 0;
   // === Sicherstellen, dass es keine Attribute mit diesem Namen mehr gibt
   DirectoryEntry objAtt = null;
   do
   {
    Counter += 1;
    AttrName = strAttributeName + Counter;
    strCNAttributeName = "CN=" + AttrName;
    objAtt = (DirectoryEntry)(DeactivateSchemaObject(objSchema.ADsPath, AttrName));
   } while (objAtt != null);

   objNewAttribute = (IADs) ((IADsContainer)objSchema).Create("attributeSchema", strCNAttributeName);
   string AttID = "1.2.840.113556.1.4." + ID + "." + Counter;

   Demo.Print("Attribut anlegen: " + AttrName + " (" + AttID + ") ");
   // String 2.5.5.5 / 19
   // Int 2.5.5.9 / 2
   // Basiswerte
   objNewAttribute.Put("attributeID", AttID);
   objNewAttribute.Put("oMSyntax", 2);
   objNewAttribute.Put("attributeSyntax", "2.5.5.9");
   objNewAttribute.Put("isSingleValued", true);

   // Weitere Werte
   //objNewAttribute.Put("instanceType", DS_INSTANCETYPE_NC_IS_WRITEABLE)
   //objNewAttribute.Put("attributeID", "2.2.840.113556.1.4.265")
   //objNewAttribute.Put("attributeID", "1.2.840.113556.1.4.265")
   //objNewAttribute.Put("rangeUpper", 32768)
   //objNewAttribute.Put("showInAdvancedViewOnly", True)
   //objNewAttribute.Put("adminDisplayName", strAttributeName)
   //objNewAttribute.Put("adminDescription", strAttributeName)
   //objNewAttribute.Put("searchFlags", 0)
   //objNewAttribute.Put("lDAPDisplayName", strAttributeName)
   //objNewAttribute.Put("name", strAttributeName)
   //objNewAttribute.Put("systemOnly", False)
   //objNewAttribute.Put("systemFlags", 16)
   //objNewAttribute.Put("objectCategory", "CN=Attribute-Schema," & strSchemaNamingContext)
   objNewAttribute.SetInfo();

   Demo.Print("Attribut erzeugt: " + objNewAttribute.Name);

   // Update schema cache.

   objRoot.Put("schemaUpdateNow", 1);
   objRoot.SetInfo();
   strAttributeName = AttrName;
   Demo.Print("Schema aktualisiert!");
  }

  public void ADS_AttributeAnlegen()
  {
   // Global declarations.

   string strAttributeName1 = "Planet";
   string strAttributeName2 = "Universum";
   IADs objRoot = null;
   IADs objSchema = null;
   string strSchemaNamingContext = null; // Schema DN.

   // === Pfade zum Schema ermitteln
   objRoot = (IADs)(new DirectoryEntry("LDAP://xfilesserver/RootDSE").NativeObject);
   strSchemaNamingContext = objRoot.Get("schemaNamingContext").ToString();
   objSchema = (IADs)(new DirectoryEntry("LDAP://xfilesserver/" + strSchemaNamingContext).NativeObject);

   Demo.Print("Schema: " + objSchema.ADsPath);

   //======== 1. Attribut
   AttributAnlegen(objRoot, objSchema, strSchemaNamingContext, ref strAttributeName1, 7002);

   //======== 2. Attribut
   AttributAnlegen(objRoot, objSchema, strSchemaNamingContext, ref strAttributeName2, 7003);

  }

  public void ADS_KlasseAnlegen()
  {

   long AlienCounter = 4;
   string strClassName = "Alien" + AlienCounter;
   string strAttributeName1 = "Planet10";
   string strAttributeName2 = "Universum8";
   IADs objRoot = null;
   IADs objSchema = null;
   string strSchemaNamingContext = null; // Schema DN.

   // === Pfade zum Schema ermitteln
   objRoot = (IADs)(new DirectoryEntry("LDAP://xfilesserver/RootDSE").NativeObject);
   strSchemaNamingContext = objRoot.Get("schemaNamingContext").ToString();
   objSchema = (IADs)(new DirectoryEntry("LDAP://xfilesserver/" + strSchemaNamingContext).NativeObject);

   Demo.Print("Schema: " + objSchema.ADsPath);

   // === Neue Klasse

   IADs objNewClass = null; // New class.
   string strCNClassName = null; // Class CN.

   // Create new class.
   strCNClassName = "CN=" + strClassName;
   objNewClass = (IADs) ((IADsContainer)objSchema).Create("classSchema", strCNClassName);

   // Set selected values for class.
   objNewClass.Put("instanceType", DS_INSTANCETYPE_NC_IS_WRITEABLE);
   objNewClass.Put("subClassOf", "contact");
   objNewClass.Put("governsID", "1.2.840.113556.1.5.8000." + (100 + AlienCounter));
   objNewClass.Put("rDNAttID", "cn");
   objNewClass.Put("showInAdvancedViewOnly", true);
   objNewClass.Put("adminDisplayName", strClassName);
   objNewClass.Put("adminDescription", strClassName);
   objNewClass.Put("objectClassCategory", 1);
   objNewClass.Put("lDAPDisplayName", strClassName);
   objNewClass.Put("name", strClassName);
   objNewClass.Put("systemOnly", false);
   // objNewClass.PutEx(ADS_PROPERTY_APPEND, _
   //             "systemPossSuperiors", "organizationalUnit")
   //objNewClass.Put("systemMayContain", strAttributeName)
   objNewClass.PutEx(ADS_PROPERTY_APPEND, "mayContain", new object[] { strAttributeName1, strAttributeName2 });

   objNewClass.Put("systemMustContain", "cn");
   objNewClass.Put("defaultSecurityDescriptor", "D:(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;DA)" + "(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;SY)" + "(A;;RPLCLORC;;;AU)");
   objNewClass.Put("systemFlags", 16);
   objNewClass.Put("defaultHidingValue", false);
   objNewClass.Put("objectCategory", "CN=Class-Schema," + strSchemaNamingContext);
   objNewClass.Put("defaultObjectCategory", "CN=Person," + strSchemaNamingContext);
   objNewClass.SetInfo();
   Demo.Print("Klasse angelegt: " + objNewClass.Name);


   objRoot.Put("schemaUpdateNow", 1);
   objRoot.SetInfo();
   Demo.Print("Schema aktualisiert!");
  }

  public void ADS_Alien_MassenAnlegen()
  {
   const string LDAP_OU = "LDAP://xfilesserver/OU=Aliens,DC=FBI,DC=net";

   System.DateTime start = DateTime.Now;
   for (int a = 1; a <= 100; a++)
   {
    string zahl = string.Format("{0:0000}", a);
    string name = "Alien" + zahl;
    int Planet = a % 10;
    int Universum = a % 50;
    Demo.PrintHeader(name);
    ADS_Alien_Anlegen(LDAP_OU, name, Planet, Universum);
   }
   System.DateTime ende = DateTime.Now;
   Demo.Print("Dauer in Sekunden: " + (ende - start).Seconds);
  }

  public void ADS_Alien_Anlegen(string LDAP_OU, string name, int Planet, int Universum)
  {
   //  Anlegen eines User-Objects im Active Directory
   Demo.Print("# Anlegen des Aliens: " + name);
   // Zugriff auf IADS
   DirectoryEntry ou = new DirectoryEntry(LDAP_OU);
   // Zugriff auf IADSContainer
   DirectoryEntries c = ou.Children;
   // Neues Objekt erzeugen
   DirectoryEntry u = null;
   try
   {
    u = c.Find("cn=" + name);
   }
   catch (Exception ex)
   {
   }
   if (u != null)
   {
    Demo.Print("Alien existiert schon und wird daher gelscht!");
    c.Remove(u);
   }
   u = c.Add("cn=" + name, "Alien4");
   // Verzeichnisattribute festlegen
   u.Properties["Planet10"].Add(Planet);
   u.Properties["Universum8"].Add(Universum);
   // nderungen speichern
   u.CommitChanges();
   Demo.Print("Alien wurde angelegt!");
  }


 }
}