﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace VerschiedeneDemos_CS.FCL
{
 public class LINQtoXML
 {

  public static void XML_Literal_Demo_Fragment()
  {

   // Ein XML-Fragment kann man einfach definieren...

   XElement x = System.Xml.Linq.XElement.Parse(
   @"
<Flug ID='347'>
 <Abflugort>Madrid</Abflugort>
 <Zielort>Paris</Zielort>
 <FreiePlaetze>1</FreiePlaetze>
 <Details>
  <Nichtraucher>true</Nichtraucher>
  <Plaetze>250</Plaetze><EingerichtetAm/>
 </Details>
 <Passagiere>
  <Passagier>Müller</Passagier>
  <Passagier>Meier</Passagier>
  <Passagier>Schulze</Passagier>
 </Passagiere>
</Flug>");


   //  Zugriff auf ein Attribut
   Console.WriteLine(x.Attribute("ID").Value);
   //  Zugriff auf Elemente
   Console.WriteLine(x.Element("Abflugort").Value); // Madrid
   Console.WriteLine(x.Element("Zielort").Value); // Paris
   Console.WriteLine(x.Element("Details").Element("Plaetze").Value); // 250
   Console.WriteLine(x.Element("Passagiere").Element("Passagier").Value); // Müller
   Console.WriteLine(x.Element("Passagiere").Elements("Passagier").ElementAt(1).Value); // Meier

   //  Schleife über Elemente
   foreach (XElement p in x.Element("Passagiere").Elements("Passagier"))
    Console.WriteLine(p.Value);


   Console.WriteLine(x.Element("Passagiere").Elements("Passagier").GetType().FullName);
  }


  public static void XML_Fluege_Add()
  {
   // Bestehende Flugliste laden
   XElement Fluege = XElement.Load("_Daten/flugliste.xml");

   // Hinzufügen eines Flugelements
   Fluege.Add(new XElement("Flug",
    new XAttribute("ID",123),
    new XElement("Abflugort", "Berlin"),
    new XElement("Zielort", "Moskau"),
    new XElement("FreiePlaetze", 250),
    new XElement("Details",
                new XElement("Nichtraucher",true),
                new XElement("Plaetze",250)),
   new XElement("Passagiere")
   ));

   // Liste speichern
   Fluege.Save("_Daten/flugliste.xml");
   Console.WriteLine("Saved: " + Fluege.Value);
  }

  public static void XML_Fluege_Select()
  {

   // Natürlich kann man Dokumente laden...
   XElement Fluege = XElement.Load("_Daten/flugliste.xml");

   // LINQ mit mehreren Ergebnissen
   IEnumerable<XElement> query = from f in Fluege.Elements("Flug")
                                 where Convert.ToInt32(f.Element("FreiePlaetze").Value) > 100
                                 select f;

   foreach (XElement result in query)
    Console.WriteLine(result.Attribute("ID").Value + "=" + result.Element("FreiePlaetze").Value);

   // LINQ mit einem Ergebnisknoten
   XElement ErsterFlugVonRom = (from f in Fluege.Elements("Flug")
                                where f.Element("Abflugort").Value == "Rom"
                                select f).FirstOrDefault();

   Console.WriteLine(ErsterFlugVonRom.Attribute("ID").Value);

   // Projektion
   var query2 = from f in Fluege.Elements("Flug")
                where Convert.ToInt32(f.Element("FreiePlaetze").Value) > 100
                select new { ID = f.Attribute("ID").Value, Plaetze = f.Element("FreiePlaetze").Value };

   foreach (var result in query2)
    Console.WriteLine(result.ID + "=" + result.Plaetze);

   // Summe
   long SummeFreiePlaetze = (from f in Fluege.Elements("Flug") select f).Sum(x => (long)x.Element("FreiePlaetze"));

   Console.WriteLine(SummeFreiePlaetze);
  }

  /// <summary>
  /// Verändern eines XML-Dokuments
  /// </summary>
  public static void XML_Fluege_Change()
  {
   // Bestehende Flugliste laden
   XElement Fluege = XElement.Load("_Daten/flugliste.xml");

   // LINQ mit einem Ergebnisknoten
   XElement flug = (from f in Fluege.Elements("Flug")
                    where Convert.ToInt64(f.Attribute("ID").Value) == 101
                    select f).FirstOrDefault();

   if (flug == null) return; // Kein Flug gefunden

   // Freie Plätze reduzieren
   Console.WriteLine("Freie Plätze vorher: " + flug.Element("FreiePlaetze").Value);
   flug.Element("FreiePlaetze").Value = (Convert.ToInt64(flug.Element("FreiePlaetze").Value) - 1).ToString();
   Console.WriteLine("Freie Plätze nachher: " + flug.Element("FreiePlaetze").Value);

   // Liste komplett (!) speichern
   Fluege.Save("_Daten/flugliste.xml");
   Console.WriteLine("Gespeichert: " + Fluege.ToString());
  }
 }
}
