﻿using System;
using System.Linq;

using WwwingModel;

namespace de.WWWingsInit
{
 class DBInit_V6_mit_EF
 {

  /// <summary>
  /// Menge von Flügen erzeugen
  /// </summary>
  /// 
  static string[] Flughaefen = { "Berlin", "Frankfurt", "München", "Hamburg", "Köln/Bonn", "Rom", "London", "Paris", "Mailand", "Prag", "Moskau", "New York", "Seattle", "Essen/Mülheim", "Kapstadt", "Madrid", "Oslo", "Dallas" };
  const int ANZFluege = 500;
  const int ANZPass = 200;

  public static void Init_DB()
  {


   Console.WriteLine("DB Version 6 Initialisieren");


   WwwingEntities db = new WwwingEntities();

   
   Console.WriteLine("Flüge zu Beginn: " + db.Flug.Count());
   Console.WriteLine("Passagiere zu Beginn: " + db.Passagier.Count());
   // Alle Buchungen löschen!
   db.ExecuteStoreCommand("Delete from Flug_Passagier");
   db.ExecuteStoreCommand("Delete from Flug");
   db.ExecuteStoreCommand("Delete from pilot");
   db.ExecuteStoreCommand("Delete from Mitarbeiter");
   db.ExecuteStoreCommand("Delete from Passagier");
   db.ExecuteStoreCommand("Delete from Person");

   Console.WriteLine("Flüge nach Löschen: " + db.Flug.Count());
   Console.WriteLine("Passagiere nach Löschen: " + db.Passagier.Count());

   Init_Passagiere(db);



   Init_Fluege(db);

   Init_Buchungen(db);

   Init_Mitarbeiter(db);

   Init_Piloten(db);
   Init_PilotenZuFluegen(db);

  }

  private static System.Random Init_Fluege( WwwingEntities  db)
  {
   Console.WriteLine("Erzeuge Flüge...");
   var Passagiere = from p in db.Passagier select p;
   System.Random rnd = new Random();
   var Fluege = from f in db.Flug select f;
   int Start = 100;

   for (int i = Start; i < (Start + ANZFluege); i++)
   {

    if (i % 100 == 0) Console.WriteLine(i);
    Flug FlugNeu = new Flug();
    FlugNeu.Flugnr = i;
    FlugNeu.Abflugort = Flughaefen[rnd.Next(0, Flughaefen.Length - 1)];
    FlugNeu.Zielort = Flughaefen[rnd.Next(0, Flughaefen.Length - 1)]; ;
    if (FlugNeu.Abflugort == FlugNeu.Zielort) { i--; continue; } // Keine Rundflüge!
    FlugNeu.Freieplaetze = Convert.ToInt16(new System.Random(i).Next(250));
    FlugNeu.Plaetze = 250;
    FlugNeu.Datum = DateTime.Now.AddDays((double)FlugNeu.Freieplaetze).AddMinutes((double)FlugNeu.Freieplaetze * 7);
    FlugNeu.Ankunft = FlugNeu.Datum.AddHours(2);
    FlugNeu.Nichtraucherflug = true;


    db.Flug.AddObject(FlugNeu);
    try
    {
     db.SaveChanges();
    }
    catch (Exception ex)
    {
     Console.WriteLine("   " + FlugNeu.Flugnr + ":" + FlugNeu.Abflugort + " -> " + FlugNeu.Zielort + ": " + ex.Message.ToString());

    }

   }
   Console.WriteLine("Flüge nach dem Einfügen: " + db.Flug.Count());
   return rnd;
  }

  private static void Init_Buchungen( WwwingEntities  db)
  {
   int k = 0;
   System.Random rnd5 = new Random();
   Console.WriteLine("Erzeuge Buchungen...");
   var Fluege2 = from f in db.Flug select f;

   Passagier[] PassagierArray = db.Passagier.ToArray();
   foreach (Flug f in Fluege2.ToList())
   {
    k++;
    if (k % 100 == 0) Console.WriteLine(k);

    for (int j = 1; j < 10; j++)
    {
     Random rnd2 = new System.Random(System.DateTime.Now.Millisecond);
     // Wähle Zufällig einen Passagier
     Passagier p = PassagierArray[Convert.ToInt16(rnd5.Next(PassagierArray.Count()))];
     Console.WriteLine(f.Flugnr + " ->  " + p.Personid);
     // Buchung anlegen
     f.Passagiers.Add(p);

     try // Weil doppelte möglich sind durch Zufallsauswahl!
     {
      db.SaveChanges();
     }
     catch (Exception ex)
     {
      Console.WriteLine(ex.InnerException.Message);
      f.Passagiers.Remove(p);
     }
    }

    

   }
  }

  private static void Init_PilotenZuFluegen( WwwingEntities  db)
  {
   Pilot[] Piloten = db.Pilot.ToArray();
   System.Random rnd4 = new Random();
   Console.WriteLine("Piloten zu Flügen...");
   foreach (Flug f in db.Flug)
   {
    f.PilotPersonid = Piloten[rnd4.Next(db.Pilot.Count() - 1)].Personid;
   }
   db.SaveChanges();
   Console.WriteLine("Piloten zugeordnet für Flüge!");
  }

  private static void Init_Piloten( WwwingEntities  db)
  {
   Console.WriteLine("Erzeuge Piloten...");
   System.Random rnd3 = new Random();
   // Alle Mitarbeiter > 250 werden zu Mitarbeitern
   foreach (Person p in db.Person.Where(pe => pe.Personid >= (ANZPass *0.75m)))
   {
    Pilot pi = new Pilot();
    pi.Personid = p.Personid;
    pi.Flugscheinseit = DateTime.Now.AddDays(-rnd3.Next(2000));
    pi.Flugscheintyp = "A";
    pi.Flugschule = "Essen";
    pi.Flugstunden = rnd3.Next(5000);
    db.Pilot.AddObject(pi);
   }
   db.SaveChanges();
   Console.WriteLine("Piloten nach dem Einfügen: " + db.Pilot.Count());
  }

  private static void Init_Mitarbeiter( WwwingEntities  db)
  {
   Person ErstePerson = null;
   Console.WriteLine("Erzeuge Mitarbeiter...");
   System.Random rnd2 = new Random();
   // Alle Personen > 200 werden zu Mitarbeitern
   foreach (Person p in db.Person.Where(pe => pe.Personid >= (ANZPass/2)).ToList())
   {
    if (ErstePerson == null) ErstePerson = p;
    Mitarbeiter mi = new Mitarbeiter();
    mi.Personid = p.Personid;
    mi.Einstellungsdatum = DateTime.Now.AddDays(-rnd2.Next(1000));
    mi.Vorgesetzterpersonid = ErstePerson.Personid;
    mi.Mitarbeiternr = p.Personid;
    db.Mitarbeiter.AddObject(mi);
    db.SaveChanges();

   }
   Console.WriteLine("Mitarbeiter nach dem Einfügen: " + db.Mitarbeiter.Count());
  }

  private static void Init_Passagiere( WwwingEntities  db)
  {
   // Passagiere Anlegen
   Console.WriteLine("Erzeuge Passagiere...");
   // Quelle: http://de.wikipedia.org/wiki/Liste_der_h%C3%A4ufigsten_Familiennamen_in_Deutschland

   string[] Nachnamen = { "Müller", "Schmidt", "Schneider", "Fischer", "Weber", "Meyer", "Wagner", "Becker", "Schulz", "Hoffmann", "Schäfer", "Koch", "Bauer", "Richter", "Klein", "Wolf", "Schröder", "Neumann", "Schwarz", "Zimmermann" };
   // Quelle: http://www.beliebte-vornamen.de/3467-alle-spitzenreiter.htm

   string[] Vornamen = { "Leon", "Hannah", "Lukas", "Anna", "Leonie", "Marie", "Niklas", "Sarah", "Jan", "Laura", "Julia", "Lisa", "Kevin" };


   // oder: Politiker-Namenskarusell :-)
   //string[] Nachnamen = { "Gysi", "Stoiber", "Koch", "Steinmeyer", "Schröder", "Merkel", "Westerwelle", "Beck", "Lafontaine", "Trittin", "Roth" };
   //string[] Vornamen = { "Edmund", "Olaf", "Roland", "Gerhard", "Angela", "Joschka", "Guido", "Gregor", "Kurt", "Frank-Walter", "Oskar", "Jürgen", "Claudia" };
   System.Random rnd2 = new Random();


   //DBUtil.ClearTable("GF_GebuchteFluege");
   //DBUtil.ClearTable("Passagier");


   string[] Staedte = { "Berlin", "Hamburg", "München", "Köln", "Frankfurt", "Stuttgart", "Dortmund", "Essen", "Düsseldorf", "Bremen", "Hannover", "Leipzig", "Dresden", "Nürnberg", "Duisburg", "Bochum", "Wuppertal", "Bielefeld", "Bonn", "Mannheim" };
   string[] Kundenstatus = { "A", "B", "C" };

   for (int PNummer = 1; PNummer <= ANZPass; PNummer++)
   {
    string Vorname = Vornamen[rnd2.Next(0, Vornamen.Length - 1)]; ;
    string Nachname = Nachnamen[rnd2.Next(0, Vornamen.Length - 1)]; ;
    Person pe = new Person();
    pe.Name = Nachname;
    pe.Vorname = Vorname;
    pe.Geburtstag = new DateTime(1940, 1, 1).AddDays(Convert.ToInt32(new Random(DateTime.Now.Millisecond).Next(20000)));
    pe.Personid = PNummer;
    pe.Land = "DE";
    pe.Stadt = Staedte[rnd2.Next(0, Staedte.Length - 1)];
    pe.Email = Vorname.Substring(0, 1) + "." + Nachname + "@world-wide-wings.de";
    db.Person.AddObject(pe);

    if (PNummer < (ANZPass / 2))
    {
     Passagier ps = new Passagier();
     ps.Personid = PNummer;
     ps.Kundeseit = pe.Geburtstag.Value.AddHours(Convert.ToInt32(new Random(DateTime.Now.Millisecond).Next(5000)));
     ps.Kundenstatus = Kundenstatus[rnd2.Next(2)];
     db.Passagier.AddObject(ps);
    }

    db.SaveChanges();

    System.Console.WriteLine(PNummer + ": " + Vorname + " " + Nachname);

   }
   Console.WriteLine("Passagiere nach dem Anfügen: " + db.Passagier.Count<Passagier>());
  }

 }
}
